@@ -20,7 +20,6 @@
 #include <common/initcall.h>
 #include <common/memory.h>
 #include <common/mini-clist.h>
-#include <common/version.h>
 
 #include <types/global.h>
 
@@ -84,17 +83,12 @@ enum {
  */
 #define PROMEX_MAX_METRIC_LENGTH 512
 
-/* Some labels for build_info */
-#define PROMEX_VERSION_LABEL "version=\"" HAPROXY_VERSION "\""
-#define PROMEX_BUILDINFO_LABEL PROMEX_VERSION_LABEL
-
 /* Matrix used to dump global metrics. Each metric points to the next one to be
  * processed or 0 to stop the dump. */
 const int promex_global_metrics[INF_TOTAL_FIELDS] = {
-	[INF_NAME]                           = INF_BUILD_INFO,
+	[INF_NAME]                           = INF_NBTHREAD,
 	[INF_VERSION]                        = 0,
 	[INF_RELEASE_DATE]                   = 0,
-	[INF_BUILD_INFO]                     = INF_NBTHREAD,
 	[INF_NBTHREAD]                       = INF_NBPROC,
 	[INF_NBPROC]                         = INF_PROCESS_NUM,
 	[INF_PROCESS_NUM]                    = INF_UPTIME_SEC,
@@ -373,7 +367,7 @@ const int promex_srv_metrics[ST_F_TOTAL_FIELDS] = {
 	[ST_F_WRETR]          = ST_F_WREDIS,
 	[ST_F_WREDIS]         = ST_F_WREW,
 	[ST_F_STATUS]         = ST_F_SCUR,
-	[ST_F_WEIGHT]         = ST_F_CHECK_STATUS,
+	[ST_F_WEIGHT]         = ST_F_CHKFAIL,
 	[ST_F_ACT]            = 0,
 	[ST_F_BCK]            = 0,
 	[ST_F_CHKFAIL]        = ST_F_CHKDOWN,
@@ -391,9 +385,9 @@ const int promex_srv_metrics[ST_F_TOTAL_FIELDS] = {
 	[ST_F_RATE]           = 0,
 	[ST_F_RATE_LIM]       = 0,
 	[ST_F_RATE_MAX]       = ST_F_LASTSESS,
-	[ST_F_CHECK_STATUS]   = ST_F_CHECK_CODE,
-	[ST_F_CHECK_CODE]     = ST_F_CHECK_DURATION,
-	[ST_F_CHECK_DURATION] = ST_F_CHKFAIL,
+	[ST_F_CHECK_STATUS]   = 0,
+	[ST_F_CHECK_CODE]     = 0,
+	[ST_F_CHECK_DURATION] = 0,
 	[ST_F_HRSP_1XX]       = ST_F_HRSP_2XX,
 	[ST_F_HRSP_2XX]       = ST_F_HRSP_3XX,
 	[ST_F_HRSP_3XX]       = ST_F_HRSP_4XX,
@@ -456,7 +450,6 @@ const struct ist promex_inf_metric_names[INF_TOTAL_FIELDS] = {
 	[INF_NAME]                           = IST("name"),
 	[INF_VERSION]                        = IST("version"),
 	[INF_RELEASE_DATE]                   = IST("release_date"),
-	[INF_BUILD_INFO]                     = IST("build_info"),
 	[INF_NBTHREAD]                       = IST("nbthread"),
 	[INF_NBPROC]                         = IST("nbproc"),
 	[INF_PROCESS_NUM]                    = IST("relative_process_id"),
@@ -491,7 +484,7 @@ const struct ist promex_inf_metric_names[INF_TOTAL_FIELDS] = {
 	[INF_MAX_SSL_RATE]                   = IST("max_ssl_rate"),
 	[INF_SSL_FRONTEND_KEY_RATE]          = IST("current_frontend_ssl_key_rate"),
 	[INF_SSL_FRONTEND_MAX_KEY_RATE]      = IST("max_frontend_ssl_key_rate"),
-	[INF_SSL_FRONTEND_SESSION_REUSE_PCT] = IST("frontend_ssl_reuse"),
+	[INF_SSL_FRONTEND_SESSION_REUSE_PCT] = IST("frontent_ssl_reuse"),
 	[INF_SSL_BACKEND_KEY_RATE]           = IST("current_backend_ssl_key_rate"),
 	[INF_SSL_BACKEND_MAX_KEY_RATE]       = IST("max_backend_ssl_key_rate"),
 	[INF_SSL_CACHE_LOOKUPS]              = IST("ssl_cache_lookups_total"),
@@ -556,7 +549,7 @@ const struct ist promex_st_metric_names[ST_F_TOTAL_FIELDS] = {
 	[ST_F_RATE_MAX]       = IST("max_session_rate"),
 	[ST_F_CHECK_STATUS]   = IST("check_status"),
 	[ST_F_CHECK_CODE]     = IST("check_code"),
-	[ST_F_CHECK_DURATION] = IST("check_duration_seconds"),
+	[ST_F_CHECK_DURATION] = IST("check_duration_milliseconds"),
 	[ST_F_HRSP_1XX]       = IST("http_responses_total"),
 	[ST_F_HRSP_2XX]       = IST("http_responses_total"),
 	[ST_F_HRSP_3XX]       = IST("http_responses_total"),
@@ -618,8 +611,7 @@ const struct ist promex_st_metric_names[ST_F_TOTAL_FIELDS] = {
 const struct ist promex_inf_metric_desc[INF_TOTAL_FIELDS] = {
 	[INF_NAME]                           = IST("Product name."),
 	[INF_VERSION]                        = IST("HAProxy version."),
-	[INF_RELEASE_DATE]                   = IST("HAProxy release date."),
-	[INF_BUILD_INFO]                     = IST("HAProxy build info."),
+	[INF_RELEASE_DATE]                   = IST("HAProxy realease date."),
 	[INF_NBTHREAD]                       = IST("Configured number of threads."),
 	[INF_NBPROC]                         = IST("Configured number of processes."),
 	[INF_PROCESS_NUM]                    = IST("Relative process id, starting at 1."),
@@ -717,9 +709,9 @@ const struct ist promex_st_metric_desc[ST_F_TOTAL_FIELDS] = {
 	[ST_F_RATE]           = IST("Current number of sessions per second over last elapsed second."),
 	[ST_F_RATE_LIM]       = IST("Configured limit on new sessions per second."),
 	[ST_F_RATE_MAX]       = IST("Maximum observed number of sessions per second."),
-	[ST_F_CHECK_STATUS]   = IST("Status of last health check (HCHK_STATUS_* values)."),
+	[ST_F_CHECK_STATUS]   = IST("Status of last health check (If a check is running, the status will be reported, prefixed with '* ')."),
 	[ST_F_CHECK_CODE]     = IST("layer5-7 code, if available of the last health check."),
-	[ST_F_CHECK_DURATION] = IST("Total duration of the latest server health check, in seconds."),
+	[ST_F_CHECK_DURATION] = IST("Time in ms took to finish last health check."),
 	[ST_F_HRSP_1XX]       = IST("Total number of HTTP responses."),
 	[ST_F_HRSP_2XX]       = IST("Total number of HTTP responses."),
 	[ST_F_HRSP_3XX]       = IST("Total number of HTTP responses."),
@@ -782,7 +774,6 @@ const struct ist promex_inf_metric_labels[INF_TOTAL_FIELDS] = {
 	[INF_NAME]                           = IST(""),
 	[INF_VERSION]                        = IST(""),
 	[INF_RELEASE_DATE]                   = IST(""),
-	[INF_BUILD_INFO]                     = IST(PROMEX_BUILDINFO_LABEL),
 	[INF_NBTHREAD]                       = IST(""),
 	[INF_NBPROC]                         = IST(""),
 	[INF_PROCESS_NUM]                    = IST(""),
@@ -939,7 +930,6 @@ const struct ist promex_inf_metric_types[INF_TOTAL_FIELDS] = {
 	[INF_NAME]                           = IST("untyped"),
 	[INF_VERSION]                        = IST("untyped"),
 	[INF_RELEASE_DATE]                   = IST("untyped"),
-	[INF_BUILD_INFO]                     = IST("gauge"),
 	[INF_NBTHREAD]                       = IST("gauge"),
 	[INF_NBPROC]                         = IST("gauge"),
 	[INF_PROCESS_NUM]                    = IST("gauge"),
@@ -1037,8 +1027,8 @@ const struct ist promex_st_metric_types[ST_F_TOTAL_FIELDS] = {
 	[ST_F_RATE]           = IST("untyped"),
 	[ST_F_RATE_LIM]       = IST("gauge"),
 	[ST_F_RATE_MAX]       = IST("gauge"),
-	[ST_F_CHECK_STATUS]   = IST("gauge"),
-	[ST_F_CHECK_CODE]     = IST("gauge"),
+	[ST_F_CHECK_STATUS]   = IST("untyped"),
+	[ST_F_CHECK_CODE]     = IST("untyped"),
 	[ST_F_CHECK_DURATION] = IST("gauge"),
 	[ST_F_HRSP_1XX]       = IST("counter"),
 	[ST_F_HRSP_2XX]       = IST("counter"),
@@ -1279,9 +1269,6 @@ static int promex_dump_global_metrics(struct appctx *appctx, struct htx *htx)
 #endif
 	while (appctx->st2 && appctx->st2 < INF_TOTAL_FIELDS) {
 		switch (appctx->st2) {
-			case INF_BUILD_INFO:
-				metric = mkf_u32(FN_GAUGE, 1);
-				break;
 			case INF_NBTHREAD:
 				metric = mkf_u32(FO_CONFIG|FS_SERVICE, global.nbthread);
 				break;
@@ -2025,22 +2012,6 @@ static int promex_dump_srv_metrics(struct appctx *appctx, struct htx *htx)
 						weight = (sv->cur_eweight * px->lbprm.wmult + px->lbprm.wdiv - 1) / px->lbprm.wdiv;
 						metric = mkf_u32(FN_AVG, weight);
 						break;
-					case ST_F_CHECK_STATUS:
-						if ((sv->check.state & (CHK_ST_ENABLED|CHK_ST_PAUSED)) != CHK_ST_ENABLED)
-							goto next_sv;
-						metric = mkf_u32(FN_OUTPUT, sv->check.status);
-						break;
-					case ST_F_CHECK_CODE:
-						if ((sv->check.state & (CHK_ST_ENABLED|CHK_ST_PAUSED)) != CHK_ST_ENABLED)
-							goto next_sv;
-						metric = mkf_u32(FN_OUTPUT, (sv->check.status < HCHK_STATUS_L57DATA) ? 0 : sv->check.code);
-						break;
-					case ST_F_CHECK_DURATION:
-						if (sv->check.status < HCHK_STATUS_CHECKED)
-						    goto next_sv;
-						secs = (double)sv->check.duration / 1000.0;
-						metric = mkf_flt(FN_DURATION, secs);
-						break;
 					case ST_F_CHKFAIL:
 						metric = mkf_u64(FN_COUNTER, sv->counters.failed_checks);
 						break;
@@ -2142,12 +2113,13 @@ static int promex_dump_srv_metrics(struct appctx *appctx, struct htx *htx)
 static int promex_dump_metrics(struct appctx *appctx, struct stream_interface *si, struct htx *htx)
 {
 	int ret;
+	int flags = appctx->ctx.stats.flags;
 
 	switch (appctx->st1) {
 		case PROMEX_DUMPER_INIT:
 			appctx->ctx.stats.px = NULL;
 			appctx->ctx.stats.sv = NULL;
-			appctx->ctx.stats.flags |= (PROMEX_FL_METRIC_HDR|PROMEX_FL_INFO_METRIC);
+			appctx->ctx.stats.flags = (flags|PROMEX_FL_METRIC_HDR|PROMEX_FL_INFO_METRIC);
 			appctx->st2 = promex_global_metrics[INF_NAME];
 			appctx->st1 = PROMEX_DUMPER_GLOBAL;
 			/* fall through */
@@ -2164,8 +2136,7 @@ static int promex_dump_metrics(struct appctx *appctx, struct stream_interface *s
 
 			appctx->ctx.stats.px = proxies_list;
 			appctx->ctx.stats.sv = NULL;
-			appctx->ctx.stats.flags &= ~PROMEX_FL_INFO_METRIC;
-			appctx->ctx.stats.flags |= (PROMEX_FL_METRIC_HDR|PROMEX_FL_STATS_METRIC);
+			appctx->ctx.stats.flags = (flags|PROMEX_FL_METRIC_HDR|PROMEX_FL_STATS_METRIC);
 			appctx->st2 = promex_front_metrics[ST_F_PXNAME];
 			appctx->st1 = PROMEX_DUMPER_FRONT;
 			/* fall through */
@@ -2182,7 +2153,7 @@ static int promex_dump_metrics(struct appctx *appctx, struct stream_interface *s
 
 			appctx->ctx.stats.px = proxies_list;
 			appctx->ctx.stats.sv = NULL;
-			appctx->ctx.stats.flags |= PROMEX_FL_METRIC_HDR;
+			appctx->ctx.stats.flags = (flags|PROMEX_FL_METRIC_HDR|PROMEX_FL_STATS_METRIC);
 			appctx->st2 = promex_back_metrics[ST_F_PXNAME];
 			appctx->st1 = PROMEX_DUMPER_BACK;
 			/* fall through */
@@ -2199,7 +2170,7 @@ static int promex_dump_metrics(struct appctx *appctx, struct stream_interface *s
 
 			appctx->ctx.stats.px = proxies_list;
 			appctx->ctx.stats.sv = (appctx->ctx.stats.px ? appctx->ctx.stats.px->srv : NULL);
-			appctx->ctx.stats.flags |= PROMEX_FL_METRIC_HDR;
+			appctx->ctx.stats.flags = (flags|PROMEX_FL_METRIC_HDR|PROMEX_FL_STATS_METRIC);
 			appctx->st2 = promex_srv_metrics[ST_F_PXNAME];
 			appctx->st1 = PROMEX_DUMPER_SRV;
 			/* fall through */
@@ -2216,7 +2187,7 @@ static int promex_dump_metrics(struct appctx *appctx, struct stream_interface *s
 
 			appctx->ctx.stats.px = NULL;
 			appctx->ctx.stats.sv = NULL;
-			appctx->ctx.stats.flags &= ~(PROMEX_FL_METRIC_HDR|PROMEX_FL_INFO_METRIC|PROMEX_FL_STATS_METRIC);
+			appctx->ctx.stats.flags = flags;
 			appctx->st2 = 0;
 			appctx->st1 = PROMEX_DUMPER_DONE;
 			/* fall through */
@@ -2249,8 +2220,7 @@ static int promex_parse_uri(struct appctx *appctx, struct stream_interface *si)
 	struct channel *res = si_ic(si);
 	struct htx *req_htx, *res_htx;
 	struct htx_sl *sl;
-	char *p, *key, *value;
-	const char *end;
+	const char *p, *end;
 	struct buffer *err;
 	int default_scopes = PROMEX_FL_SCOPE_ALL;
 	int len;
@@ -2264,72 +2234,57 @@ static int promex_parse_uri(struct appctx *appctx, struct stream_interface *si)
 	if (!p)
 		goto end;
 	end = HTX_SL_REQ_UPTR(sl) + HTX_SL_REQ_ULEN(sl);
+	len = end-p;
 
-	/* copy the query-string */
-	len = end - p;
+	/* Decode the query-string */
 	chunk_reset(&trash);
 	memcpy(trash.area, p, len);
 	trash.area[len] = 0;
+	len = url_decode(trash.area);
+	if (len == -1)
+		goto error;
 	p = trash.area;
-	end = trash.area + len;
+	end = p + len;
 
 	/* Parse the query-string */
-	while (p < end && *p && *p != '#') {
-		value = NULL;
-
-		/* decode parameter name */
-		key = p;
-		while (p < end && *p != '=' && *p != '&' && *p != '#')
+	while (p < end) {
+		if (*p == '&')
 			++p;
-		/* found a value */
-		if (*p == '=') {
-			*(p++) = 0;
-			value = p;
-		}
-		else if (*p == '&')
-			*(p++) = 0;
-		else if (*p == '#')
-			*p = 0;
-		len = url_decode(key, 1);
-		if (len == -1)
-			goto error;
-
-		/* decode value */
-		if (value) {
-			while (p < end && *p != '=' && *p != '&' && *p != '#')
-				++p;
-			if (*p == '=')
-				goto error;
-			if (*p == '&')
-				*(p++) = 0;
-			else if (*p == '#')
-				*p = 0;
-			len = url_decode(value, 1);
-			if (len == -1)
-				goto error;
-		}
-
-		if (!strcmp(key, "scope")) {
+		else if (*p == 's' && (end-p) >= 6 && !memcmp(p, "scope=", 6)) {
 			default_scopes = 0; /* at least a scope defined, unset default scopes */
-			if (!value)
-				goto error;
-			else if (*value == 0)
+			p += 6;             /* now p point on the parameter value */
+			len = 0;            /* len is the value length */
+			while ((p+len) < end && *(p+len) != '&')
+				++len;
+
+			if (len == 0)
 				appctx->ctx.stats.flags &= ~PROMEX_FL_SCOPE_ALL;
-			else if (*value == '*')
+			else if (len == 1 && *p == '*')
 				appctx->ctx.stats.flags |= PROMEX_FL_SCOPE_ALL;
-			else if (!strcmp(value, "global"))
-				appctx->ctx.stats.flags |= PROMEX_FL_SCOPE_GLOBAL;
-			else if (!strcmp(value, "server"))
-				appctx->ctx.stats.flags |= PROMEX_FL_SCOPE_SERVER;
-			else if (!strcmp(value, "backend"))
+			else if (len == 6) {
+				if (!memcmp(p, "global", len))
+					appctx->ctx.stats.flags |= PROMEX_FL_SCOPE_GLOBAL;
+				else if (!memcmp(p, "server", len))
+					appctx->ctx.stats.flags |= PROMEX_FL_SCOPE_SERVER;
+			}
+			else if (len == 7 && !memcmp(p, "backend", len))
 				appctx->ctx.stats.flags |= PROMEX_FL_SCOPE_BACK;
-			else if (!strcmp(value, "frontend"))
+			else if (len == 8 && !memcmp(p, "frontend", len))
 				appctx->ctx.stats.flags |= PROMEX_FL_SCOPE_FRONT;
 			else
 				goto error;
+
+			p += len;
 		}
-		else if (!strcmp(key, "no-maint"))
+		else if (*p == 'n' && (end-p) >= 8 && !memcmp(p, "no-maint", 8)) {
 			appctx->ctx.stats.flags |= PROMEX_FL_NO_MAINT_SRV;
+			p += 8;
+		}
+		else {
+			/* ignore all other params for now */
+			while (p < end && *p != '&')
+				p++;
+		}
 	}
 
   end:
@@ -2337,7 +2292,7 @@ static int promex_parse_uri(struct appctx *appctx, struct stream_interface *si)
 	return 1;
 
   error:
-	err = &htx_err_chunks[HTTP_ERR_400];
+	err = &http_err_chunks[HTTP_ERR_400];
 	channel_erase(res);
 	res->buf.data = b_data(err);
 	memcpy(res->buf.area, b_head(err), b_data(err));
@@ -2440,7 +2395,6 @@ static void promex_appctx_handle_io(struct appctx *appctx)
 				goto out;
 			}
 			channel_add_input(res, 1);
-			res->flags |= CF_EOI;
 			appctx->st0 = PROMEX_ST_END;
 			/* fall through */
 
@@ -2465,7 +2419,6 @@ static void promex_appctx_handle_io(struct appctx *appctx)
 	res->flags |= CF_READ_NULL;
 	si_shutr(si);
 	si_shutw(si);
-	goto out;
 }
 
 struct applet promex_applet = {
@@ -2483,10 +2436,6 @@ static enum act_parse_ret service_parse_prometheus_exporter(const char **args, i
 		memprintf(err, "Prometheus exporter service only available on 'http-request' rulesets");
 		return ACT_RET_PRS_ERR;
 	}
-	if (!(px->options2 & PR_O2_USE_HTX)) {
-		memprintf(err, "Prometheus exporter service only available for HTX proxies");
-		return ACT_RET_PRS_ERR;
-	}
 
 	/* Add applet pointer in the rule. */
 	rule->applet = promex_applet;
