@@ -188,7 +188,7 @@ static inline void si_release_endpoint(struct stream_interface *si)
 	else if ((appctx = objt_appctx(si->end))) {
 		if (appctx->applet->release && !si_state_in(si->state, SI_SB_DIS|SI_SB_CLO))
 			appctx->applet->release(appctx);
-		appctx_free(appctx); /* we share the connection pool */
+		appctx_free(appctx);
 	} else if ((conn = objt_conn(si->end))) {
 		conn_stop_tracking(conn);
 		conn_full_close(conn);
@@ -250,15 +250,6 @@ static inline int si_rx_blocked(const struct stream_interface *si)
 	return !!(si->flags & SI_FL_RXBLK_ANY);
 }
 
-
-/* Returns non-zero if the stream interface's Rx path is blocked because of lack
- * of room in the input buffer.
- */
-static inline int si_rx_blocked_room(const struct stream_interface *si)
-{
-	return !!(si->flags & SI_FL_RXBLK_ROOM);
-}
-
 /* Returns non-zero if the stream interface's endpoint is ready to receive */
 static inline int si_rx_endp_ready(const struct stream_interface *si)
 {
@@ -539,7 +530,7 @@ static inline int si_connect(struct stream_interface *si, struct connection *con
 
 	/* needs src ip/port for logging */
 	if (si->flags & SI_FL_SRC_ADDR)
-		conn_get_from_addr(conn);
+		conn_get_src(conn);
 
 	return ret;
 }
