@@ -38,6 +38,8 @@
 #include <types/global.h>
 #include <types/task.h>
 
+#include <proto/fd.h>
+
 /* Principle of the wait queue.
  *
  * We want to be able to tell whether an expiration date is before of after the
@@ -91,7 +93,7 @@ extern unsigned int niced_tasks;  /* number of niced tasks in the run queue */
 extern struct pool_head *pool_head_task;
 extern struct pool_head *pool_head_tasklet;
 extern struct pool_head *pool_head_notification;
-extern THREAD_LOCAL struct task *curr_task; /* task currently running or NULL */
+extern THREAD_LOCAL struct task_per_thread *sched; /* current's thread scheduler context */
 #ifdef USE_THREAD
 extern struct eb_root timers;      /* sorted timers tree, global */
 extern struct eb_root rqueue;      /* tree constituting the run queue */
@@ -103,8 +105,6 @@ extern struct task_per_thread task_per_thread[MAX_THREADS];
 __decl_hathreads(extern HA_SPINLOCK_T rq_lock);  /* spin lock related to run queue */
 __decl_hathreads(extern HA_RWLOCK_T wq_lock);    /* RW lock related to the wait queue */
 
-static inline struct task *task_unlink_wq(struct task *t);
-static inline void task_queue(struct task *task);
 
 /* return 0 if task is in run queue, otherwise non-zero */
 static inline int task_in_rq(struct task *t)
@@ -134,11 +134,11 @@ static inline void task_wakeup(struct task *t, unsigned int f)
 	struct eb_root *root;
 
 	if (t->thread_mask == tid_bit || global.nbthread == 1)
-		root = &task_per_thread[tid].rqueue;
+		root = &sched->rqueue;
 	else
 		root = &rqueue;
 #else
-	struct eb_root *root = &task_per_thread[tid].rqueue;
+	struct eb_root *root = &sched->rqueue;
 #endif
 
 	state = _HA_ATOMIC_OR(&t->state, f);
@@ -150,60 +150,10 @@ static inline void task_wakeup(struct task *t, unsigned int f)
 	}
 }
 
-/* change the thread affinity of a task to <thread_mask>.
- * This may only be done from within the running task itself or during its
- * initialization. It will unqueue and requeue the task from the wait queue
- * if it was in it. This is safe against a concurrent task_queue() call because
- * task_queue() itself will unlink again if needed after taking into account
- * the new thread_mask.
- */
+/* change the thread affinity of a task to <thread_mask> */
 static inline void task_set_affinity(struct task *t, unsigned long thread_mask)
 {
-	if (unlikely(task_in_wq(t))) {
-		task_unlink_wq(t);
-		t->thread_mask = thread_mask;
-		task_queue(t);
-	}
-	else
-		t->thread_mask = thread_mask;
-}
-
-/* Atomically drop the TASK_RUNNING bit while ensuring that any wakeup that
- * happened since the flag was set will result in the task being queued (if
- * it wasn't already). This is used to safely drop the flag from within the
- * scheduler. The flag <f> is combined with existing flags before the test so
- * that it's possible to inconditionally wakeup the task and drop the RUNNING
- * flag if needed.
- */
-#define task_drop_running(t, f) _task_drop_running(t, f, __FILE__, __LINE__)
-static inline void _task_drop_running(struct task *t, unsigned int f, const char *file, int line)
-{
-	unsigned short state, new_state;
-
-	state = _HA_ATOMIC_LOAD(&t->state);
-
-	while (1) {
-		new_state = state | f;
-		if (new_state & TASK_WOKEN_ANY)
-			new_state |= TASK_QUEUED;
-
-		if (_HA_ATOMIC_CAS(&t->state, &state, new_state & ~TASK_RUNNING))
-			break;
-	}
-
-	if ((new_state & ~state) & TASK_QUEUED) {
-#ifdef USE_THREAD
-		struct eb_root *root;
-
-		if (t->thread_mask == tid_bit || global.nbthread == 1)
-			root = &task_per_thread[tid].rqueue;
-		else
-			root = &rqueue;
-#else
-		struct eb_root *root = &task_per_thread[tid].rqueue;
-#endif
-		__task_wakeup(t, root);
-	}
+	t->thread_mask = thread_mask;
 }
 
 /*
@@ -219,15 +169,15 @@ static inline struct task *__task_unlink_wq(struct task *t)
 }
 
 /* remove a task from its wait queue. It may either be the local wait queue if
- * the task is bound to a single thread or the global queue. If the task uses a
- * shared wait queue, the global wait queue lock is used.
+ * the task is bound to a single thread (in which case there's no locking
+ * involved) or the global queue, with locking.
  */
 static inline struct task *task_unlink_wq(struct task *t)
 {
 	unsigned long locked;
 
 	if (likely(task_in_wq(t))) {
-		locked = t->state & TASK_SHARED_WQ;
+		locked = atleast2(t->thread_mask);
 		if (locked)
 			HA_RWLOCK_WRLOCK(TASK_WQ_LOCK, &wq_lock);
 		__task_unlink_wq(t);
@@ -253,7 +203,7 @@ static inline struct task *__task_unlink_rq(struct task *t)
 		global_rqueue_size--;
 	} else
 #endif
-		task_per_thread[tid].rqueue_size--;
+		sched->rqueue_size--;
 	eb32sc_delete(&t->rq);
 	if (likely(t->nice))
 		_HA_ATOMIC_SUB(&niced_tasks, 1);
@@ -278,10 +228,22 @@ static inline struct task *task_unlink_rq(struct task *t)
 
 static inline void tasklet_wakeup(struct tasklet *tl)
 {
-	if (!LIST_ISEMPTY(&tl->list))
-		return;
-	LIST_ADDQ(&task_per_thread[tid].task_list, &tl->list);
-	_HA_ATOMIC_ADD(&tasks_run_queue, 1);
+	if (likely(tl->tid < 0)) {
+		/* this tasklet runs on the caller thread */
+		if (LIST_ISEMPTY(&tl->list)) {
+			LIST_ADDQ(&task_per_thread[tid].task_list, &tl->list);
+			_HA_ATOMIC_ADD(&tasks_run_queue, 1);
+		}
+	} else {
+		/* this tasklet runs on a specific thread */
+		if (MT_LIST_ADDQ(&task_per_thread[tl->tid].shared_tasklet_list, (struct mt_list *)&tl->list) == 1) {
+			_HA_ATOMIC_ADD(&tasks_run_queue, 1);
+			if (sleeping_thread_mask & (1UL << tl->tid)) {
+				_HA_ATOMIC_AND(&sleeping_thread_mask, ~(1UL << tl->tid));
+				wake_thread(tl->tid);
+			}
+		}
+	}
 
 }
 
@@ -291,12 +253,14 @@ static inline void tasklet_wakeup(struct tasklet *tl)
 static inline void tasklet_insert_into_tasklet_list(struct tasklet *tl)
 {
 	_HA_ATOMIC_ADD(&tasks_run_queue, 1);
-	LIST_ADDQ(&task_per_thread[tid].task_list, &tl->list);
+	LIST_ADDQ(&sched->task_list, &tl->list);
 }
 
 /* Remove the tasklet from the tasklet list. The tasklet MUST already be there.
  * If unsure, use tasklet_remove_from_tasklet_list() instead. If used with a
  * plain task, the caller must update the task_list_size.
+ * This should only be used by the thread that owns the tasklet, any other
+ * thread should use tasklet_cancel().
  */
 static inline void __tasklet_remove_from_tasklet_list(struct tasklet *t)
 {
@@ -306,15 +270,14 @@ static inline void __tasklet_remove_from_tasklet_list(struct tasklet *t)
 
 static inline void tasklet_remove_from_tasklet_list(struct tasklet *t)
 {
-	if (likely(!LIST_ISEMPTY(&t->list)))
-		__tasklet_remove_from_tasklet_list(t);
+	if (MT_LIST_DEL((struct mt_list *)&t->list))
+		_HA_ATOMIC_SUB(&tasks_run_queue, 1);
 }
 
 /*
  * Initialize a new task. The bare minimum is performed (queue pointers and
  * state).  The task is returned. This function should not be used outside of
- * task_new(). If the thread mask contains more than one thread, TASK_SHARED_WQ
- * is set.
+ * task_new().
  */
 static inline struct task *task_init(struct task *t, unsigned long thread_mask)
 {
@@ -322,8 +285,6 @@ static inline struct task *task_init(struct task *t, unsigned long thread_mask)
 	t->rq.node.leaf_p = NULL;
 	t->state = TASK_SLEEPING;
 	t->thread_mask = thread_mask;
-	if (atleast2(thread_mask))
-		t->state |= TASK_SHARED_WQ;
 	t->nice = 0;
 	t->calls = 0;
 	t->call_date = 0;
@@ -333,15 +294,23 @@ static inline struct task *task_init(struct task *t, unsigned long thread_mask)
 	return t;
 }
 
+/* Initialize a new tasklet. It's identified as a tasklet by ->nice=-32768. It
+ * is expected to run on the calling thread by default, it's up to the caller
+ * to change ->tid if it wants to own it.
+ */
 static inline void tasklet_init(struct tasklet *t)
 {
 	t->nice = -32768;
 	t->calls = 0;
 	t->state = 0;
 	t->process = NULL;
+	t->tid = -1;
 	LIST_INIT(&t->list);
 }
 
+/* Allocate and initialize a new tasklet, local to the thread by default. The
+ * caller may assing its tid if it wants to own the tasklet.
+ */
 static inline struct tasklet *tasklet_new(void)
 {
 	struct tasklet *t = pool_alloc(pool_head_tasklet);
@@ -373,8 +342,8 @@ static inline struct task *task_new(unsigned long thread_mask)
  */
 static inline void __task_free(struct task *t)
 {
-	if (t == curr_task) {
-		curr_task = NULL;
+	if (t == sched->current) {
+		sched->current = NULL;
 		__ha_barrier_store();
 	}
 	pool_free(pool_head_task, t);
@@ -402,12 +371,13 @@ static inline void task_destroy(struct task *t)
 	/* There's no need to protect t->state with a lock, as the task
 	 * has to run on the current thread.
 	 */
-	if (t == curr_task || !(t->state & (TASK_QUEUED | TASK_RUNNING)))
+	if (t == sched->current || !(t->state & (TASK_QUEUED | TASK_RUNNING)))
 		__task_free(t);
 	else
 		t->process = NULL;
 }
 
+/* Should only be called by the thread responsible for the tasklet */
 static inline void tasklet_free(struct tasklet *tl)
 {
 	if (!LIST_ISEMPTY(&tl->list)) {
@@ -415,22 +385,23 @@ static inline void tasklet_free(struct tasklet *tl)
 		_HA_ATOMIC_SUB(&tasks_run_queue, 1);
 	}
 
-	if ((struct task *)tl == curr_task) {
-		curr_task = NULL;
-		__ha_barrier_store();
-	}
 	pool_free(pool_head_tasklet, tl);
 	if (unlikely(stopping))
 		pool_flush(pool_head_tasklet);
 }
 
+static inline void tasklet_set_tid(struct tasklet *tl, int tid)
+{
+	tl->tid = tid;
+}
+
 void __task_queue(struct task *task, struct eb_root *wq);
 
 /* Place <task> into the wait queue, where it may already be. If the expiration
  * timer is infinite, do nothing and rely on wake_expired_task to clean up.
- * If the task uses a shared wait queue, it's queued into the global wait queue,
- * protected by the global wq_lock, otherwise by it necessarily belongs to the
- * current thread'sand is queued without locking.
+ * If the task is bound to a single thread, it's assumed to be bound to the
+ * current thread's queue and is queued without locking. Otherwise it's queued
+ * into the global wait queue, protected by locks.
  */
 static inline void task_queue(struct task *task)
 {
@@ -447,7 +418,7 @@ static inline void task_queue(struct task *task)
 		return;
 
 #ifdef USE_THREAD
-	if (task->state & TASK_SHARED_WQ) {
+	if (atleast2(task->thread_mask)) {
 		HA_RWLOCK_WRLOCK(TASK_WQ_LOCK, &wq_lock);
 		if (!task_in_wq(task) || tick_is_lt(task->expire, task->wq.key))
 			__task_queue(task, &timers);
@@ -455,9 +426,8 @@ static inline void task_queue(struct task *task)
 	} else
 #endif
 	{
-		BUG_ON((task->thread_mask & tid_bit) == 0); // should have TASK_SHARED_WQ
 		if (!task_in_wq(task) || tick_is_lt(task->expire, task->wq.key))
-			__task_queue(task, &task_per_thread[tid].timers);
+			__task_queue(task, &sched->timers);
 	}
 }
 
@@ -472,7 +442,7 @@ static inline void task_schedule(struct task *task, int when)
 		return;
 
 #ifdef USE_THREAD
-	if (task->state & TASK_SHARED_WQ) {
+	if (atleast2(task->thread_mask)) {
 		/* FIXME: is it really needed to lock the WQ during the check ? */
 		HA_RWLOCK_WRLOCK(TASK_WQ_LOCK, &wq_lock);
 		if (task_in_wq(task))
@@ -485,13 +455,12 @@ static inline void task_schedule(struct task *task, int when)
 	} else
 #endif
 	{
-		BUG_ON((task->thread_mask & tid_bit) == 0); // should have TASK_SHARED_WQ
 		if (task_in_wq(task))
 			when = tick_first(when, task->expire);
 
 		task->expire = when;
 		if (!task_in_wq(task) || tick_is_lt(task->expire, task->wq.key))
-			__task_queue(task, &task_per_thread[tid].timers);
+			__task_queue(task, &sched->timers);
 	}
 }
 
@@ -594,14 +563,14 @@ static inline int notification_registered(struct list *wake)
 static inline int thread_has_tasks(void)
 {
 	return (!!(global_tasks_mask & tid_bit) |
-	        (task_per_thread[tid].rqueue_size > 0) |
-	        !LIST_ISEMPTY(&task_per_thread[tid].task_list));
+	        (sched->rqueue_size > 0) |
+	        !LIST_ISEMPTY(&sched->task_list) | !MT_LIST_ISEMPTY(&sched->shared_tasklet_list));
 }
 
 /* adds list item <item> to work list <work> and wake up the associated task */
-static inline void work_list_add(struct work_list *work, struct list *item)
+static inline void work_list_add(struct work_list *work, struct mt_list *item)
 {
-	LIST_ADDQ_LOCKED(&work->head, item);
+	MT_LIST_ADDQ(&work->head, item);
 	task_wakeup(work->task, TASK_WOKEN_OTHER);
 }
 
