@@ -147,15 +147,10 @@ struct dns_answer_item {
 	int16_t         priority;                    /* SRV type priority */
 	uint16_t        weight;                      /* SRV type weight */
 	uint16_t        port;                        /* SRV type port */
-	uint16_t        data_len;                    /* number of bytes in the <data> field below */
-	union {
-		struct sockaddr_in in4;              /* IPv4 address for RTYPE_A */
-		struct sockaddr_in6 in6;             /* IPv6 address for RTYPE_AAAA */
-		char target[DNS_MAX_NAME_SIZE+1];    /* Response data: SRV or CNAME type target */
-	} data;
+	uint16_t        data_len;                    /* number of bytes in target below */
+	struct sockaddr address;                     /* IPv4 or IPv6, network format */
+	char            target[DNS_MAX_NAME_SIZE+1]; /* Response data: SRV or CNAME type target */
 	time_t          last_seen;                   /* When was the answer was last seen */
-	struct dns_answer_item *ar_item;             /* pointer to a RRset from the additional section, if exists */
-	struct list     attached_servers;            /* attached server head */
 	struct list     list;
 };
 
@@ -254,6 +249,7 @@ struct dns_options {
 	int pref_net_nb; /* The number of registered preferred networks. */
 	int accept_duplicate_ip; /* flag to indicate whether the associated object can use an IP address
 				    already set to an other object of the same group */
+	int ignore_weight; /* flag to indicate whether to ignore the weight within the record */
 };
 
 /* Resolution structure associated to single server and used to manage name
