@@ -35,7 +35,7 @@
 #include <types/filters.h>
 #include <types/hlua.h>
 #include <types/obj_type.h>
-#include <types/proto_http.h>
+#include <types/http_ana.h>
 #include <types/proxy.h>
 #include <types/queue.h>
 #include <types/server.h>
@@ -91,7 +91,6 @@
 #define SF_IGNORE_PRST	0x00080000	/* ignore persistence */
 
 #define SF_SRV_REUSED   0x00100000	/* the server-side connection was reused */
-#define SF_SRV_REUSED_ANTICIPATED  0x00200000  /* the connection was reused but the mux is not ready yet */
 
 
 /* flags for the proxy of the master CLI */
@@ -147,17 +146,18 @@ struct stream {
 	struct list back_refs;          /* list of users tracking this stream */
 	struct buffer_wait buffer_wait; /* position in the list of objects waiting for a buffer */
 
-	struct freq_ctr call_rate;      /* stream task call rate without making progress */
+	struct freq_ctr call_rate;      /* stream task call rate */
+
+	short store_count;
+	enum obj_type obj_type;         /* object type == OBJ_TYPE_STREAM */
+	/* 1 unused bytes here */
 
 	struct {
 		struct stksess *ts;
 		struct stktable *table;
 	} store[8];                     /* tracked stickiness values to store */
-	int store_count;
-
-	enum obj_type obj_type;         /* object type == OBJ_TYPE_STREAM */
-	/* 3 unused bytes here */
 
+	struct sockaddr_storage *target_addr;   /* the address to join if not null */
 	struct stkctr stkctr[MAX_SESS_STKCTR];  /* content-aware stick counters */
 
 	struct strm_flt strm_flt;               /* current state of filters active on this stream */
