@@ -23,6 +23,8 @@
 
 unsigned int nb_applets = 0;
 
+DECLARE_POOL(pool_head_appctx,  "appctx",  sizeof(struct appctx));
+
 /* Callback used to wake up an applet when a buffer is available. The applet
  * <appctx> is woken up if an input buffer was requested for the associated
  * stream interface. In this case the buffer is immediately allocated and the
@@ -61,7 +63,6 @@ struct task *task_run_applet(struct task *t, void *context, unsigned short state
 	struct appctx *app = context;
 	struct stream_interface *si = app->owner;
 	unsigned int rate;
-	size_t count;
 
 	if (app->state & APPLET_WANT_DIE) {
 		__appctx_free(app);
@@ -84,17 +85,8 @@ struct task *task_run_applet(struct task *t, void *context, unsigned short state
 	if (!si_alloc_ibuf(si, &app->buffer_wait))
 		si_rx_endp_more(si);
 
-	count = co_data(si_oc(si));
 	app->applet->fct(app);
 
-	/* now check if the applet has released some room and forgot to
-	 * notify the other side about it.
-	 */
-	if (count != co_data(si_oc(si))) {
-		si_oc(si)->flags |= CF_WRITE_PARTIAL | CF_WROTE_DATA;
-		si_rx_room_rdy(si_opposite(si));
-	}
-
 	/* measure the call rate and check for anomalies when too high */
 	rate = update_freq_ctr(&app->call_rate, 1);
 	if (rate >= 100000 && app->call_rate.prev_ctr && // looped more than 100k times over last second
