@@ -48,7 +48,7 @@
 #include <proto/obj_type.h>
 #include <proto/payload.h>
 #include <proto/protocol.h>
-#include <proto/proto_http.h>
+#include <proto/http_ana.h>
 #include <proto/proto_tcp.h>
 #include <proto/proxy.h>
 #include <proto/queue.h>
@@ -60,6 +60,8 @@
 #include <proto/ssl_sock.h>
 #include <proto/task.h>
 
+#define TRACE_SOURCE &trace_strm
+
 int be_lastsession(const struct proxy *be)
 {
 	if (be->be_counters.last_sess)
@@ -321,6 +323,8 @@ static struct server *get_server_ph_post(struct stream *s, const struct server *
 	unsigned int hash = 0;
 	struct channel  *req  = &s->req;
 	struct proxy    *px   = s->be;
+	struct htx      *htx = htxbuf(&req->buf);
+	struct htx_blk  *blk;
 	unsigned int     plen = px->lbprm.arg_len;
 	unsigned long    len;
 	const char      *params, *p, *start, *end;
@@ -328,36 +332,18 @@ static struct server *get_server_ph_post(struct stream *s, const struct server *
 	if (px->lbprm.tot_weight == 0)
 		return NULL;
 
-	if (!IS_HTX_STRM(s)) {
-		struct http_txn *txn = s->txn;
-		struct http_msg *msg = &txn->req;
+	p = params = NULL;
+	len = 0;
+	for (blk = htx_get_first_blk(htx); blk; blk = htx_get_next_blk(htx, blk)) {
+		enum htx_blk_type type = htx_get_blk_type(blk);
+		struct ist v;
 
-		len  = http_body_bytes(msg);
-		p = params = c_ptr(req, -http_data_rewind(msg));
-
-		if (len == 0)
-			return NULL;
-		if (len > b_wrap(&req->buf) - p)
-			len = b_wrap(&req->buf) - p;
-
-	}
-	else {
-		struct htx *htx = htxbuf(&req->buf);
-		struct htx_blk *blk;
-
-		p = params = NULL;
-		len = 0;
-		for (blk = htx_get_first_blk(htx); blk; blk = htx_get_next_blk(htx, blk)) {
-			enum htx_blk_type type = htx_get_blk_type(blk);
-			struct ist v;
-
-			if (type != HTX_BLK_DATA)
-				continue;
-			v = htx_get_blk_value(htx, blk);
-			p = params = v.ptr;
-			len = v.len;
-			break;
-		}
+		if (type != HTX_BLK_DATA)
+			continue;
+		v = htx_get_blk_value(htx, blk);
+		p = params = v.ptr;
+		len = v.len;
+		break;
 	}
 
 	while (len > plen) {
@@ -428,6 +414,8 @@ static struct server *get_server_hh(struct stream *s, const struct server *avoid
 	unsigned long    len;
 	const char      *p;
 	const char *start, *end;
+	struct htx *htx = htxbuf(&s->req.buf);
+	struct http_hdr_ctx ctx = { .blk = NULL };
 
 	/* tot_weight appears to mean srv_count */
 	if (px->lbprm.tot_weight == 0)
@@ -437,40 +425,17 @@ static struct server *get_server_hh(struct stream *s, const struct server *avoid
 	if (px->lbprm.tot_used == 1)
 		goto hash_done;
 
-	if (!IS_HTX_STRM(s)) {
-		struct http_txn *txn = s->txn;
-		struct hdr_ctx   ctx = { .idx = 0 };
+	http_find_header(htx, ist2(px->lbprm.arg_str, plen), &ctx, 0);
 
-		/* if the message is chunked, we skip the chunk size, but use the value as len */
-		http_find_header2(px->lbprm.arg_str, plen, c_ptr(&s->req, -http_hdr_rewind(&txn->req)),
-				  &txn->hdr_idx, &ctx);
+	/* if the header is not found or empty, let's fallback to round robin */
+	if (!ctx.blk || !ctx.value.len)
+		return NULL;
 
-		/* if the header is not found or empty, let's fallback to round robin */
-		if (!ctx.idx || !ctx.vlen)
-			return NULL;
-
-		/* Found the param_name in the headers.
-		 * we will compute the hash based on this value ctx.val.
-		 */
-		len = ctx.vlen;
-		p = (char *)ctx.line + ctx.val;
-	}
-	else {
-		struct htx *htx = htxbuf(&s->req.buf);
-		struct http_hdr_ctx ctx = { .blk = NULL };
-
-		http_find_header(htx, ist2(px->lbprm.arg_str, plen), &ctx, 0);
-
-		/* if the header is not found or empty, let's fallback to round robin */
-		if (!ctx.blk || !ctx.value.len)
-			return NULL;
-
-		/* Found a the param_name in the headers.
-		 * we will compute the hash based on this value ctx.val.
-		 */
-		len = ctx.value.len;
-		p   = ctx.value.ptr;
-	}
+	/* Found a the param_name in the headers.
+	 * we will compute the hash based on this value ctx.val.
+	 */
+	len = ctx.value.len;
+	p   = ctx.value.ptr;
 
 	if (!px->lbprm.arg_opt1) {
 		hash = gen_hash(px, p, len);
@@ -574,7 +539,8 @@ static struct server *get_server_rnd(struct stream *s, const struct server *avoi
 	curr = NULL;
 	do {
 		prev = curr;
-		hash = ha_random32();
+		/* ensure all 32 bits are covered as long as RAND_MAX >= 65535 */
+		hash = ((uint64_t)random() * ((uint64_t)RAND_MAX + 1)) ^ random();
 		curr = chash_get_server_hash(px, hash, avoid);
 		if (!curr)
 			break;
@@ -663,10 +629,6 @@ int assign_server(struct stream *s)
 
 						srv = tmpsrv;
 						s->target = &srv->obj_type;
-						if (conn->flags & CO_FL_SESS_IDLE) {
-							conn->flags &= ~CO_FL_SESS_IDLE;
-							s->sess->idle_conns--;
-						}
 						goto out_ok;
 					}
 				}
@@ -718,27 +680,26 @@ int assign_server(struct stream *s)
 			switch (s->be->lbprm.algo & BE_LB_PARM) {
 			case BE_LB_HASH_SRC:
 				conn = objt_conn(strm_orig(s));
-				if (conn && conn->addr.from.ss_family == AF_INET) {
+				if (conn && conn_get_src(conn) && conn->src->ss_family == AF_INET) {
 					srv = get_server_sh(s->be,
-							    (void *)&((struct sockaddr_in *)&conn->addr.from)->sin_addr,
+							    (void *)&((struct sockaddr_in *)conn->src)->sin_addr,
 							    4, prev_srv);
 				}
-				else if (conn && conn->addr.from.ss_family == AF_INET6) {
+				else if (conn && conn_get_src(conn) && conn->src->ss_family == AF_INET6) {
 					srv = get_server_sh(s->be,
-							    (void *)&((struct sockaddr_in6 *)&conn->addr.from)->sin6_addr,
+							    (void *)&((struct sockaddr_in6 *)conn->src)->sin6_addr,
 							    16, prev_srv);
 				}
+				else {
+					/* unknown IP family */
+					err = SRV_STATUS_INTERNAL;
+					goto out;
+				}
 				break;
 
 			case BE_LB_HASH_URI:
 				/* URI hashing */
-				if (!s->txn || s->txn->req.msg_state < HTTP_MSG_BODY)
-					break;
-				if (!IS_HTX_STRM(s))
-					srv = get_server_uh(s->be,
-							    c_ptr(&s->req, -http_uri_rewind(&s->txn->req)),
-							    s->txn->req.sl.rq.u_l, prev_srv);
-				else {
+				if (IS_HTX_STRM(s) && s->txn->req.msg_state >= HTTP_MSG_BODY) {
 					struct ist uri;
 
 					uri = htx_sl_req_uri(http_get_stline(htxbuf(&s->req.buf)));
@@ -748,29 +709,21 @@ int assign_server(struct stream *s)
 
 			case BE_LB_HASH_PRM:
 				/* URL Parameter hashing */
-				if (!s->txn || s->txn->req.msg_state < HTTP_MSG_BODY)
-					break;
-
-				if (!IS_HTX_STRM(s))
-					srv = get_server_ph(s->be,
-							    c_ptr(&s->req, -http_uri_rewind(&s->txn->req)),
-							    s->txn->req.sl.rq.u_l, prev_srv);
-				else {
+				if (IS_HTX_STRM(s) && s->txn->req.msg_state >= HTTP_MSG_BODY) {
 					struct ist uri;
 
 					uri = htx_sl_req_uri(http_get_stline(htxbuf(&s->req.buf)));
 					srv = get_server_ph(s->be, uri.ptr, uri.len, prev_srv);
-				}
 
-				if (!srv && s->txn->meth == HTTP_METH_POST)
-					srv = get_server_ph_post(s, prev_srv);
+					if (!srv && s->txn->meth == HTTP_METH_POST)
+						srv = get_server_ph_post(s, prev_srv);
+				}
 				break;
 
 			case BE_LB_HASH_HDR:
 				/* Header Parameter hashing */
-				if (!s->txn || s->txn->req.msg_state < HTTP_MSG_BODY)
-					break;
-				srv = get_server_hh(s, prev_srv);
+				if (IS_HTX_STRM(s) && s->txn->req.msg_state >= HTTP_MSG_BODY)
+					srv = get_server_hh(s, prev_srv);
 				break;
 
 			case BE_LB_HASH_RDP:
@@ -819,7 +772,7 @@ int assign_server(struct stream *s)
 	else if ((s->be->options & PR_O_HTTP_PROXY)) {
 		conn = cs_conn(objt_cs(s->si[1].end));
 
-		if (conn && is_addr(&conn->addr.to)) {
+		if (conn && conn->dst && is_addr(conn->dst)) {
 			/* in proxy mode, we need a valid destination address */
 			s->target = &s->be->obj_type;
 		} else {
@@ -845,7 +798,7 @@ out_ok:
 			sess_change_server(s, srv);
 		} else {
 			if (may_dequeue_tasks(conn_slot, s->be))
-				process_srv_queue(conn_slot, 0);
+				process_srv_queue(conn_slot);
 		}
 	}
 
@@ -864,39 +817,36 @@ out_ok:
  *
  * Upon successful return, the stream flag SF_ADDR_SET is set. This flag is
  * not cleared, so it's to the caller to clear it if required.
- *
- * The caller is responsible for having already assigned a connection
- * to si->end.
- *
  */
-int assign_server_address(struct stream *s, struct connection *srv_conn)
+int assign_server_address(struct stream *s)
 {
 	struct connection *cli_conn = objt_conn(strm_orig(s));
 
 	DPRINTF(stderr,"assign_server_address : s=%p\n",s);
 
+	if (!sockaddr_alloc(&s->target_addr))
+		return SRV_STATUS_INTERNAL;
+
 	if ((s->flags & SF_DIRECT) || (s->be->lbprm.algo & BE_LB_KIND)) {
 		/* A server is necessarily known for this stream */
 		if (!(s->flags & SF_ASSIGNED))
 			return SRV_STATUS_INTERNAL;
 
-		srv_conn->addr.to = __objt_server(s->target)->addr;
-		set_host_port(&srv_conn->addr.to, __objt_server(s->target)->svc_port);
+		*s->target_addr = __objt_server(s->target)->addr;
+		set_host_port(s->target_addr, __objt_server(s->target)->svc_port);
 
-		if (!is_addr(&srv_conn->addr.to) && cli_conn) {
+		if (!is_addr(s->target_addr) && cli_conn) {
 			/* if the server has no address, we use the same address
 			 * the client asked, which is handy for remapping ports
 			 * locally on multiple addresses at once. Nothing is done
 			 * for AF_UNIX addresses.
 			 */
-			conn_get_to_addr(cli_conn);
-
-			if (cli_conn->addr.to.ss_family == AF_INET) {
-				((struct sockaddr_in *)&srv_conn->addr.to)->sin_family = AF_INET;
-				((struct sockaddr_in *)&srv_conn->addr.to)->sin_addr = ((struct sockaddr_in *)&cli_conn->addr.to)->sin_addr;
-			} else if (cli_conn->addr.to.ss_family == AF_INET6) {
-				((struct sockaddr_in6 *)&srv_conn->addr.to)->sin6_family = AF_INET6;
-				((struct sockaddr_in6 *)&srv_conn->addr.to)->sin6_addr = ((struct sockaddr_in6 *)&cli_conn->addr.to)->sin6_addr;
+			if (!conn_get_dst(cli_conn)) {
+				/* do nothing if we can't retrieve the address */
+			} else if (cli_conn->dst->ss_family == AF_INET) {
+				((struct sockaddr_in *)s->target_addr)->sin_addr = ((struct sockaddr_in *)cli_conn->dst)->sin_addr;
+			} else if (cli_conn->dst->ss_family == AF_INET6) {
+				((struct sockaddr_in6 *)s->target_addr)->sin6_addr = ((struct sockaddr_in6 *)cli_conn->dst)->sin6_addr;
 			}
 		}
 
@@ -905,26 +855,25 @@ int assign_server_address(struct stream *s, struct connection *srv_conn)
 		if ((__objt_server(s->target)->flags & SRV_F_MAPPORTS) && cli_conn) {
 			int base_port;
 
-			conn_get_to_addr(cli_conn);
+			if (conn_get_dst(cli_conn)) {
+				/* First, retrieve the port from the incoming connection */
+				base_port = get_host_port(cli_conn->dst);
 
-			/* First, retrieve the port from the incoming connection */
-			base_port = get_host_port(&cli_conn->addr.to);
-
-			/* Second, assign the outgoing connection's port */
-			base_port += get_host_port(&srv_conn->addr.to);
-			set_host_port(&srv_conn->addr.to, base_port);
+				/* Second, assign the outgoing connection's port */
+				base_port += get_host_port(s->target_addr);
+				set_host_port(s->target_addr, base_port);
+			}
 		}
 	}
 	else if (s->be->options & PR_O_DISPATCH) {
 		/* connect to the defined dispatch addr */
-		srv_conn->addr.to = s->be->dispatch_addr;
+		*s->target_addr = s->be->dispatch_addr;
 	}
 	else if ((s->be->options & PR_O_TRANSP) && cli_conn) {
 		/* in transparent mode, use the original dest addr if no dispatch specified */
-		conn_get_to_addr(cli_conn);
-
-		if (cli_conn->addr.to.ss_family == AF_INET || cli_conn->addr.to.ss_family == AF_INET6)
-			srv_conn->addr.to = cli_conn->addr.to;
+		if (conn_get_dst(cli_conn) &&
+		    (cli_conn->dst->ss_family == AF_INET || cli_conn->dst->ss_family == AF_INET6))
+			*s->target_addr = *cli_conn->dst;
 	}
 	else if (s->be->options & PR_O_HTTP_PROXY) {
 		/* If HTTP PROXY option is set, then server is already assigned
@@ -935,9 +884,6 @@ int assign_server_address(struct stream *s, struct connection *srv_conn)
 		return SRV_STATUS_INTERNAL;
 	}
 
-	/* Copy network namespace from client connection */
-	srv_conn->proxy_netns = cli_conn ? cli_conn->proxy_netns : NULL;
-
 	s->flags |= SF_ADDR_SET;
 	return SRV_STATUS_OK;
 }
@@ -1089,51 +1035,42 @@ static void assign_tproxy_address(struct stream *s)
 	else
 		return;
 
+	if (!sockaddr_alloc(&srv_conn->src))
+		return;
+
 	switch (src->opts & CO_SRC_TPROXY_MASK) {
 	case CO_SRC_TPROXY_ADDR:
-		srv_conn->addr.from = src->tproxy_addr;
+		*srv_conn->src = src->tproxy_addr;
 		break;
 	case CO_SRC_TPROXY_CLI:
 	case CO_SRC_TPROXY_CIP:
 		/* FIXME: what can we do if the client connects in IPv6 or unix socket ? */
 		cli_conn = objt_conn(strm_orig(s));
-		if (cli_conn)
-			srv_conn->addr.from = cli_conn->addr.from;
-		else
-			memset(&srv_conn->addr.from, 0, sizeof(srv_conn->addr.from));
+		if (cli_conn && conn_get_src(cli_conn))
+			*srv_conn->src = *cli_conn->src;
+		else {
+			sockaddr_free(&srv_conn->src);
+		}
 		break;
 	case CO_SRC_TPROXY_DYN:
-		if (src->bind_hdr_occ && s->txn) {
+		if (src->bind_hdr_occ && IS_HTX_STRM(s)) {
 			char *vptr;
 			size_t vlen;
 
 			/* bind to the IP in a header */
-			((struct sockaddr_in *)&srv_conn->addr.from)->sin_family = AF_INET;
-			((struct sockaddr_in *)&srv_conn->addr.from)->sin_port = 0;
-			((struct sockaddr_in *)&srv_conn->addr.from)->sin_addr.s_addr = 0;
-			if (!IS_HTX_STRM(s)) {
-				int rewind;
-
-				c_rew(&s->req, rewind = http_hdr_rewind(&s->txn->req));
-				if (http_get_hdr(&s->txn->req, src->bind_hdr_name, src->bind_hdr_len,
-						 &s->txn->hdr_idx, src->bind_hdr_occ, NULL, &vptr, &vlen)) {
-					((struct sockaddr_in *)&srv_conn->addr.from)->sin_addr.s_addr =
-						htonl(inetaddr_host_lim(vptr, vptr + vlen));
-				}
-				c_adv(&s->req, rewind);
-			}
-			else {
-				if (http_get_htx_hdr(htxbuf(&s->req.buf),
-						     ist2(src->bind_hdr_name, src->bind_hdr_len),
-						     src->bind_hdr_occ, NULL, &vptr, &vlen)) {
-					((struct sockaddr_in *)&srv_conn->addr.from)->sin_addr.s_addr =
-						htonl(inetaddr_host_lim(vptr, vptr + vlen));
-				}
+			((struct sockaddr_in *)srv_conn->src)->sin_family = AF_INET;
+			((struct sockaddr_in *)srv_conn->src)->sin_port = 0;
+			((struct sockaddr_in *)srv_conn->src)->sin_addr.s_addr = 0;
+			if (http_get_htx_hdr(htxbuf(&s->req.buf),
+					     ist2(src->bind_hdr_name, src->bind_hdr_len),
+					     src->bind_hdr_occ, NULL, &vptr, &vlen)) {
+				((struct sockaddr_in *)srv_conn->src)->sin_addr.s_addr =
+					htonl(inetaddr_host_lim(vptr, vptr + vlen));
 			}
 		}
 		break;
 	default:
-		memset(&srv_conn->addr.from, 0, sizeof(srv_conn->addr.from));
+		sockaddr_free(&srv_conn->src);
 	}
 #endif
 }
@@ -1204,10 +1141,11 @@ fail:
  */
 int connect_server(struct stream *s)
 {
-	struct connection *cli_conn = NULL;
+	struct connection *cli_conn = objt_conn(strm_orig(s));
 	struct connection *srv_conn = NULL;
 	struct connection *old_conn = NULL;
 	struct conn_stream *srv_cs = NULL;
+	struct sess_srv_list *srv_list;
 	struct server *srv;
 	int reuse = 0;
 	int reuse_orphan = 0;
@@ -1216,57 +1154,44 @@ int connect_server(struct stream *s)
 	int err;
 
 
-	/* Some, such as http_proxy and the LUA, create their connection and
-	 * conn_stream manually, so if we already have a conn_stream, try
-	 * to use it.
+	/* This will catch some corner cases such as lying connections resulting from
+	 * retries or connect timeouts but will rarely trigger.
 	 */
-	srv_cs = objt_cs(s->si[1].end);
-	if (!srv_cs)
-		srv_conn = objt_conn(s->si[1].end);
-	else
-		srv_conn = cs_conn(srv_cs);
+	si_release_endpoint(&s->si[1]);
 
-	if (srv_conn) {
-		if (!srv_conn->target || srv_conn->target == s->target) {
-			srv_conn->flags &= ~(CO_FL_ERROR | CO_FL_SOCK_RD_SH | CO_FL_SOCK_WR_SH);
-			if (srv_cs)
-				srv_cs->flags &= ~(CS_FL_ERROR | CS_FL_EOS);
-			reuse = 1;
-			old_conn = srv_conn;
-		} else {
-			srv_conn = NULL;
-			srv_cs = NULL;
-			si_release_endpoint(&s->si[1]);
-		}
-	}
-
-	if (!old_conn) {
-		struct sess_srv_list *srv_list;
-		list_for_each_entry(srv_list, &s->sess->srv_list, srv_list) {
-			if (srv_list->target == s->target) {
-				list_for_each_entry(srv_conn, &srv_list->conn_list,
-				    session_list) {
-					if (conn_xprt_ready(srv_conn) &&
-					    srv_conn->mux && (srv_conn->mux->avail_streams(srv_conn) > 0)) {
-						reuse = 1;
-						break;
-					}
+	/* first, search for a matching connection in the session's idle conns */
+	list_for_each_entry(srv_list, &s->sess->srv_list, srv_list) {
+		if (srv_list->target == s->target) {
+			list_for_each_entry(srv_conn, &srv_list->conn_list, session_list) {
+				if (conn_xprt_ready(srv_conn) &&
+				    srv_conn->mux && (srv_conn->mux->avail_streams(srv_conn) > 0)) {
+					reuse = 1;
+					break;
 				}
-				break;
 			}
-		}
-		if (reuse == 0) {
-			srv_conn = NULL;
-			if (!LIST_ISEMPTY(&s->sess->srv_list)) {
-				srv_list = LIST_ELEM(s->sess->srv_list.n,
-					struct sess_srv_list *, srv_list);
-				if (!LIST_ISEMPTY(&srv_list->conn_list))
-					srv_conn = LIST_ELEM(srv_list->conn_list.n,
-						struct connection *, session_list);
-			}
-
+			break;
 		}
 	}
+
+	if (!reuse) {
+		/* no connection was found in our session's list. Pick any
+		 * random one that we could trade against another one.
+		 */
+		srv_conn = NULL;
+		if (!LIST_ISEMPTY(&s->sess->srv_list)) {
+			srv_list = LIST_ELEM(s->sess->srv_list.n, struct sess_srv_list *, srv_list);
+			if (!LIST_ISEMPTY(&srv_list->conn_list))
+				srv_conn = LIST_ELEM(srv_list->conn_list.n, struct connection *, session_list);
+		}
+
+	}
+	/* OK at this point we have this :
+	 *   - srv_conn points to an existing connection or NULL
+	 *   - if reuse is set, srv_conn holds a valid connection, otherwise it
+	 *     points to any of our old connections we may want to trade against
+	 *     another one
+	 */
+
 	old_conn = srv_conn;
 
 	srv = objt_server(s->target);
@@ -1287,6 +1212,9 @@ int connect_server(struct stream *s)
 		 *  ----+-----+-----+    ----+-----+-----+   ----+-----+-----+
 		 *  idle|  -  |  1  |    idle|  -  |  1  |   idle|  2  |  1  |
 		 *  ----+-----+-----+    ----+-----+-----+   ----+-----+-----+
+		 *
+		 * Idle conns are necessarily looked up on the same thread so
+		 * that there is no concurrency issues.
 		 */
 		if (srv->idle_conns && !LIST_ISEMPTY(&srv->idle_conns[tid]) &&
 		    ((s->be->options & PR_O_REUSE_MASK) != PR_O_REUSE_NEVR &&
@@ -1301,11 +1229,11 @@ int connect_server(struct stream *s)
 		else if (srv->idle_conns && !LIST_ISEMPTY(&srv->idle_conns[tid]) &&
 			 (s->be->options & PR_O_REUSE_MASK) == PR_O_REUSE_ALWS) {
 			srv_conn = LIST_ELEM(srv->idle_conns[tid].n, struct connection *, list);
-		} else if (srv->idle_orphan_conns && !LIST_ISEMPTY(&srv->idle_orphan_conns[tid]) &&
+		} else if (srv->idle_orphan_conns && !MT_LIST_ISEMPTY(&srv->idle_orphan_conns[tid]) &&
 		    (((s->be->options & PR_O_REUSE_MASK) == PR_O_REUSE_ALWS) ||
 		    (((s->be->options & PR_O_REUSE_MASK) != PR_O_REUSE_NEVR) &&
 		     s->txn && (s->txn->flags & TX_NOT_FIRST)))) {
-			srv_conn = LIST_POP_LOCKED(&srv->idle_orphan_conns[tid],
+			srv_conn = MT_LIST_POP(&srv->idle_orphan_conns[tid],
 			                           struct connection *, list);
 			if (srv_conn)
 				reuse_orphan = 1;
@@ -1321,6 +1249,10 @@ int connect_server(struct stream *s)
 			reuse = 1;
 	}
 
+
+	/* here reuse might have been set above, indicating srv_conn finally
+	 * is OK.
+	 */
 	if (reuse) {
 		/* Disable connection reuse if a dynamic source is used.
 		 * As long as we don't share connections between servers,
@@ -1336,6 +1268,7 @@ int connect_server(struct stream *s)
 				reuse = 0;
 		}
 	}
+
 	if (((!reuse || (srv_conn && !(srv_conn->flags & CO_FL_CONNECTED)))
 	    && ha_used_fds > global.tune.pool_high_count) && srv && srv->idle_orphan_conns) {
 		struct connection *tokill_conn;
@@ -1344,7 +1277,7 @@ int connect_server(struct stream *s)
 		 * acceptable, attempt to kill an idling connection
 		 */
 		/* First, try from our own idle list */
-		tokill_conn = LIST_POP_LOCKED(&srv->idle_orphan_conns[tid],
+		tokill_conn = MT_LIST_POP(&srv->idle_orphan_conns[tid],
 		    struct connection *, list);
 		if (tokill_conn)
 			tokill_conn->mux->destroy(tokill_conn->ctx);
@@ -1362,19 +1295,16 @@ int connect_server(struct stream *s)
 				// see it possibly larger.
 				ALREADY_CHECKED(i);
 
-				HA_SPIN_LOCK(OTHER_LOCK, &toremove_lock[i]);
-				tokill_conn = LIST_POP_LOCKED(&srv->idle_orphan_conns[i],
+				tokill_conn = MT_LIST_POP(&srv->idle_orphan_conns[i],
 				    struct connection *, list);
 				if (tokill_conn) {
 					/* We got one, put it into the concerned thread's to kill list, and wake it's kill task */
 
-					LIST_ADDQ_LOCKED(&toremove_connections[i],
-					    &tokill_conn->list);
+					MT_LIST_ADDQ(&toremove_connections[i],
+					    (struct mt_list *)&tokill_conn->list);
 					task_wakeup(idle_conn_cleanup[i], TASK_WOKEN_OTHER);
-					HA_SPIN_UNLOCK(OTHER_LOCK, &toremove_lock[i]);
 					break;
 				}
-				HA_SPIN_UNLOCK(OTHER_LOCK, &toremove_lock[i]);
 			}
 		}
 
@@ -1401,7 +1331,7 @@ int connect_server(struct stream *s)
 	}
 
 	/* We're about to use another connection, let the mux know we're
-	 * done with this one
+	 * done with this one.
 	 */
 	if (old_conn != srv_conn && old_conn && reuse && !reuse_orphan) {
 		struct session *sess = srv_conn->owner;
@@ -1424,10 +1354,7 @@ int connect_server(struct stream *s)
 	}
 
 	if (reuse) {
-		/* We already created a cs earlier when using http_proxy, so
-		 * only create a new one if we don't have one already.
-		 */
-		if (!srv_cs && srv_conn->mux) {
+		if (srv_conn->mux) {
 			int avail = srv_conn->mux->avail_streams(srv_conn);
 
 			if (avail <= 1) {
@@ -1454,7 +1381,6 @@ int connect_server(struct stream *s)
 
 	/* no reuse or failed to reuse the connection above, pick a new one */
 	if (!srv_conn) {
-		s->flags &= ~SF_ADDR_SET;
 		srv_conn = conn_new();
 		if (srv_conn)
 			srv_conn->target = s->target;
@@ -1479,41 +1405,38 @@ int connect_server(struct stream *s)
 		}
 	}
 
-	if (!srv_conn) {
-		if (srv_conn)
-			conn_free(srv_conn);
+	if (!srv_conn || !sockaddr_alloc(&srv_conn->dst))
 		return SF_ERR_RESOURCE;
-	}
 
 	if (!(s->flags & SF_ADDR_SET)) {
-		err = assign_server_address(s, srv_conn);
-		if (err != SRV_STATUS_OK) {
-			conn_free(srv_conn);
+		err = assign_server_address(s);
+		if (err != SRV_STATUS_OK)
 			return SF_ERR_INTERNAL;
-		}
 	}
 
+	/* copy the target address into the connection */
+	*srv_conn->dst = *s->target_addr;
+
+	/* Copy network namespace from client connection */
+	srv_conn->proxy_netns = cli_conn ? cli_conn->proxy_netns : NULL;
+
 	if (!conn_xprt_ready(srv_conn) && !srv_conn->mux) {
 		/* set the correct protocol on the output stream interface */
 		if (srv)
-			conn_prepare(srv_conn, protocol_by_family(srv_conn->addr.to.ss_family), srv->xprt);
+			conn_prepare(srv_conn, protocol_by_family(srv_conn->dst->ss_family), srv->xprt);
 		else if (obj_type(s->target) == OBJ_TYPE_PROXY) {
 			/* proxies exclusively run on raw_sock right now */
-			conn_prepare(srv_conn, protocol_by_family(srv_conn->addr.to.ss_family), xprt_get(XPRT_RAW));
-			if (!(srv_conn->ctrl)) {
-				conn_free(srv_conn);
+			conn_prepare(srv_conn, protocol_by_family(srv_conn->dst->ss_family), xprt_get(XPRT_RAW));
+			if (!(srv_conn->ctrl))
 				return SF_ERR_INTERNAL;
-			}
 		}
-		else {
-			conn_free(srv_conn);
+		else
 			return SF_ERR_INTERNAL;  /* how did we get there ? */
-		}
 
 #if defined(USE_OPENSSL) && defined(TLSEXT_TYPE_application_layer_protocol_negotiation)
 		if (!srv ||
-		    (srv->use_ssl != 1 || (!(srv->ssl_ctx.alpn_str) && !(srv->ssl_ctx.npn_str)) ||
-		     srv->mux_proto || s->be->mode != PR_MODE_HTTP))
+		    ((!(srv->ssl_ctx.alpn_str) && !(srv->ssl_ctx.npn_str)) ||
+		    srv->mux_proto || s->be->mode != PR_MODE_HTTP))
 #endif
 		{
 			srv_cs = objt_cs(s->si[1].end);
@@ -1539,18 +1462,15 @@ int connect_server(struct stream *s)
 		}
 
 #endif
-
-
 		/* process the case where the server requires the PROXY protocol to be sent */
 		srv_conn->send_proxy_ofs = 0;
-		cli_conn = objt_conn(strm_orig(s));
 
 		if (srv && srv->pp_opts) {
 			srv_conn->flags |= CO_FL_PRIVATE;
 			srv_conn->flags |= CO_FL_SEND_PROXY;
 			srv_conn->send_proxy_ofs = 1; /* must compute size */
 			if (cli_conn)
-				conn_get_to_addr(cli_conn);
+				conn_get_dst(cli_conn);
 		}
 
 		assign_tproxy_address(s);
@@ -1560,20 +1480,16 @@ int connect_server(struct stream *s)
 			srv_conn->flags |= CO_FL_SOCKS4;
 		}
 	}
-	else if (!srv_conn->mux) {
-		/* No mux? We possibly asked for ALPN during a first failed
-		 * attempt and are trying to start again from this connection,
-		 * thus we have nothing to do.
-		 */
-	}
 	else if (!conn_xprt_ready(srv_conn)) {
 		if (srv_conn->mux->reset)
 			srv_conn->mux->reset(srv_conn);
 	}
 	else {
-		s->flags |= SF_SRV_REUSED;
-		if (!(srv_conn->mux->ctl(srv_conn, MUX_STATUS, NULL) & MUX_STATUS_READY))
-			s->flags |= SF_SRV_REUSED_ANTICIPATED;
+		/* Only consider we're doing reuse if the connection was
+		 * ready.
+		 */
+		if (srv_conn->mux->ctl(srv_conn, MUX_STATUS, NULL) & MUX_STATUS_READY)
+			s->flags |= SF_SRV_REUSED;
 	}
 
 	/* flag for logging source ip/port */
@@ -1653,44 +1569,15 @@ int connect_server(struct stream *s)
 		s->flags |= SF_CURR_SESS;
 		count = _HA_ATOMIC_ADD(&srv->cur_sess, 1);
 		HA_ATOMIC_UPDATE_MAX(&srv->counters.cur_sess_max, count);
-		if (s->be->lbprm.server_take_conn) {
-			HA_SPIN_LOCK(SERVER_LOCK, &srv->lock);
+		if (s->be->lbprm.server_take_conn)
 			s->be->lbprm.server_take_conn(srv);
-			HA_SPIN_UNLOCK(SERVER_LOCK, &srv->lock);
-		}
 
 #ifdef USE_OPENSSL
-		if (!(s->flags & SF_SRV_REUSED) && srv->ssl_ctx.sni) {
+		if (srv->ssl_ctx.sni) {
 			struct sample *smp;
-			int rewind;
-
-			if (!IS_HTX_STRM(s)) {
-				/* Tricky case : we have already scheduled the pending
-				 * HTTP request or TCP data for leaving. So in HTTP we
-				 * rewind exactly the headers, otherwise we rewind the
-				 * output data.
-				 */
-				rewind = s->txn ? http_hdr_rewind(&s->txn->req) : co_data(&s->req);
-				c_rew(&s->req, rewind);
-
-				smp = sample_fetch_as_type(s->be, s->sess, s, SMP_OPT_DIR_REQ | SMP_OPT_FINAL,
-							   srv->ssl_ctx.sni, SMP_T_STR);
-
-				/* restore the pointers */
-				c_adv(&s->req, rewind);
-			}
-			else {
-				/* rewind the output data. */
-				rewind = co_data(&s->req);
-				c_rew(&s->req, rewind);
-
-				smp = sample_fetch_as_type(s->be, s->sess, s, SMP_OPT_DIR_REQ | SMP_OPT_FINAL,
-							   srv->ssl_ctx.sni, SMP_T_STR);
-
-				/* restore the pointers */
-				c_adv(&s->req, rewind);
-			}
 
+			smp = sample_fetch_as_type(s->be, s->sess, s, SMP_OPT_DIR_REQ | SMP_OPT_FINAL,
+						   srv->ssl_ctx.sni, SMP_T_STR);
 			if (smp_make_safe(smp)) {
 				ssl_sock_set_servername(srv_conn,
 							smp->data.u.str.area);
@@ -1740,6 +1627,7 @@ int srv_redispatch_connect(struct stream *s)
 		if (((s->flags & (SF_DIRECT|SF_FORCE_PRST)) == SF_DIRECT) &&
 		    (s->be->options & PR_O_REDISP)) {
 			s->flags &= ~(SF_DIRECT | SF_ASSIGNED | SF_ADDR_SET);
+			sockaddr_free(&s->target_addr);
 			goto redispatch;
 		}
 
@@ -1782,7 +1670,7 @@ int srv_redispatch_connect(struct stream *s)
 
 		/* release other streams waiting for this server */
 		if (may_dequeue_tasks(srv, s->be))
-			process_srv_queue(srv, 0);
+			process_srv_queue(srv);
 		return 1;
 	}
 	/* if we get here, it's because we got SRV_STATUS_OK, which also
@@ -1821,14 +1709,7 @@ int tcp_persist_rdp_cookie(struct stream *s, struct channel *req, int an_bit)
 	uint32_t addr;
 	char *p;
 
-	DPRINTF(stderr,"[%u] %s: stream=%p b=%p, exp(r,w)=%u,%u bf=%08x bh=%lu analysers=%02x\n",
-		now_ms, __FUNCTION__,
-		s,
-		req,
-		req->rex, req->wex,
-		req->flags,
-		ci_data(req),
-		req->analysers);
+	DBG_TRACE_ENTER(STRM_EV_STRM_ANA|STRM_EV_TCP_ANA, s);
 
 	if (s->flags & SF_ASSIGNED)
 		goto no_cookie;
@@ -1871,6 +1752,7 @@ int tcp_persist_rdp_cookie(struct stream *s, struct channel *req, int an_bit)
 no_cookie:
 	req->analysers &= ~an_bit;
 	req->analyse_exp = TICK_ETERNITY;
+	DBG_TRACE_LEAVE(STRM_EV_STRM_ANA|STRM_EV_TCP_ANA, s);
 	return 1;
 }
 
@@ -2214,6 +2096,26 @@ smp_fetch_srv_id(const struct arg *args, struct sample *smp, const char *kw, voi
 	return 1;
 }
 
+/* set string to the name of the server */
+static int
+smp_fetch_srv_name(const struct arg *args, struct sample *smp, const char *kw, void *private)
+{
+	if (!smp->strm)
+		return 0;
+
+	if (!objt_server(smp->strm->target))
+		return 0;
+
+	smp->data.u.str.area = (char *)__objt_server(smp->strm->target)->id;
+	if (!smp->data.u.str.area)
+	        return 0;
+
+	smp->data.type = SMP_T_STR;
+	smp->data.u.str.data = strlen(smp->data.u.str.area);
+
+	return 1;
+}
+
 /* set temp integer to the number of connections per second reaching the backend.
  * Accepts exactly 1 argument. Argument is a backend, other types will lead to
  * undefined behaviour.
@@ -2405,6 +2307,38 @@ static int sample_conv_nbsrv(const struct arg *args, struct sample *smp, void *p
 	return 1;
 }
 
+static int
+sample_conv_srv_queue(const struct arg *args, struct sample *smp, void *private)
+{
+	struct proxy *px;
+	struct server *srv;
+	char *bksep;
+
+	if (!smp_make_safe(smp))
+		return 0;
+
+	bksep = strchr(smp->data.u.str.area, '/');
+
+	if (bksep) {
+		*bksep = '\0';
+		px = proxy_find_by_name(smp->data.u.str.area, PR_CAP_BE, 0);
+		if (!px)
+			return 0;
+		smp->data.u.str.area = bksep + 1;
+	} else {
+		if (!(smp->px->cap & PR_CAP_BE))
+			return 0;
+		px = smp->px;
+	}
+
+	srv = server_find_by_name(px, smp->data.u.str.area);
+	if (!srv)
+		return 0;
+
+	smp->data.type = SMP_T_SINT;
+	smp->data.u.sint = srv->nbpend;
+	return 1;
+}
 
 /* Note: must not be declared <const> as its list will be overwritten.
  * Please take care of keeping this list alphabetically sorted.
@@ -2423,6 +2357,7 @@ static struct sample_fetch_kw_list smp_kws = {ILH, {
 	{ "srv_conn_free", smp_fetch_srv_conn_free,  ARG1(1,SRV), NULL, SMP_T_SINT, SMP_USE_INTRN, },
 	{ "srv_id",        smp_fetch_srv_id,         0,           NULL, SMP_T_SINT, SMP_USE_SERVR, },
 	{ "srv_is_up",     smp_fetch_srv_is_up,      ARG1(1,SRV), NULL, SMP_T_BOOL, SMP_USE_INTRN, },
+	{ "srv_name",      smp_fetch_srv_name,       0,           NULL, SMP_T_STR,  SMP_USE_SERVR, },
 	{ "srv_queue",     smp_fetch_srv_queue,      ARG1(1,SRV), NULL, SMP_T_SINT, SMP_USE_INTRN, },
 	{ "srv_sess_rate", smp_fetch_srv_sess_rate,  ARG1(1,SRV), NULL, SMP_T_SINT, SMP_USE_INTRN, },
 	{ /* END */ },
@@ -2432,7 +2367,8 @@ INITCALL1(STG_REGISTER, sample_register_fetches, &smp_kws);
 
 /* Note: must not be declared <const> as its list will be overwritten */
 static struct sample_conv_kw_list sample_conv_kws = {ILH, {
-	{ "nbsrv", sample_conv_nbsrv, 0, NULL, SMP_T_STR, SMP_T_SINT },
+	{ "nbsrv",     sample_conv_nbsrv,     0, NULL, SMP_T_STR, SMP_T_SINT },
+	{ "srv_queue", sample_conv_srv_queue, 0, NULL, SMP_T_STR, SMP_T_SINT },
 	{ /* END */ },
 }};
 
