@@ -12,7 +12,6 @@
 #include <unistd.h>
 
 #include <common/cfgparse.h>
-#include <common/buf.h>
 #include <proto/compression.h>
 
 /*
@@ -921,13 +920,7 @@ int cfg_parse_global(const char *file, int linenum, char **args, int kwm)
 			goto out;
 		}
 		chunk_destroy(&global.log_tag);
-		chunk_initlen(&global.log_tag, strdup(args[1]), strlen(args[1]), strlen(args[1]));
-		if (b_orig(&global.log_tag) == NULL) {
-			chunk_destroy(&global.log_tag);
-			ha_alert("parsing [%s:%d]: cannot allocate memory for '%s'.\n", file, linenum, args[0]);
-			err_code |= ERR_ALERT | ERR_FATAL;
-			goto out;
-		}
+		chunk_initstr(&global.log_tag, strdup(args[1]));
 	}
 	else if (!strcmp(args[0], "spread-checks")) {  /* random time between checks (0-50) */
 		if (alertif_too_many_args(1, file, linenum, args, &err_code))
@@ -1065,34 +1058,22 @@ int cfg_parse_global(const char *file, int linenum, char **args, int kwm)
 				}
 			}
 		} else {
-			/* Mapping at the thread level.
-			 * Either proc and/or thread must be 1 and only 1. All
-			 * other combinations are silently ignored.
+			/* Mapping at the thread level. All threads are retained
+			 * for process 1, and only thread 1 is retained for other
+			 * processes.
 			 */
 			if (thread == 0x1) {
-				int val;
-
 				/* first thread, iterate on processes. E.g. cpu-map 1-4/1 0-3 */
 				for (i = n = 0; i < MAX_PROCS; i++) {
 					/* No mapping for this process */
 					if (!(proc & (1UL << i)))
 						continue;
-
-					if (!autoinc) {
-						val = cpus;
-					}
+					if (!autoinc)
+						global.cpu_map.proc_t1[i] = cpus;
 					else {
 						n += my_ffsl(cpus >> n);
-						val = 1UL << (n - 1);
+						global.cpu_map.proc_t1[i] = (1UL << (n-1));
 					}
-
-					/* For first process, thread[0] is used.
-					 * Use proc_t1[N] for all others
-					 */
-					if (!i)
-						global.cpu_map.thread[0] = val;
-					else
-						global.cpu_map.proc_t1[i] = val;
 				}
 			}
 
@@ -1191,6 +1172,14 @@ int cfg_parse_global(const char *file, int linenum, char **args, int kwm)
 				env++;
 		}
 	}
+	else if (!strcmp(args[0], "strict-limits")) { /* "no strict-limits" or "strict-limits" */
+		if (alertif_too_many_args(0, file, linenum, args, &err_code))
+			goto out;
+		if (kwm == KWM_NO)
+			global.tune.options &= ~GTUNE_STRICT_LIMITS;
+		else
+			global.tune.options |= GTUNE_STRICT_LIMITS;
+	}
 	else {
 		struct cfg_kw_list *kwl;
 		int index;
