@@ -96,7 +96,7 @@ int co_inject(struct channel *chn, const char *msg, int len)
 	if (len > max)
 		return max;
 
-	memcpy(co_tail(chn), msg, len);
+	memcpy(ci_tail(chn), msg, len);
 	b_add(&chn->buf, len);
 	c_adv(chn, len);
 	chn->total += len;
@@ -153,9 +153,8 @@ int ci_putblk(struct channel *chn, const char *blk, int len)
 	max = channel_recv_limit(chn);
 	if (unlikely(len > max - c_data(chn))) {
 		/* we can't write this chunk right now because the buffer is
-		 * almost full or because the block is too large. Returns
-		 * -3 if block is too large for this buffer. Or -1 if the
-		 * room left is not large enough.
+		 * almost full or because the block is too large. Return the
+		 * available space or -2 if impossible.
 		 */
 		if (len > max)
 			return -3;
@@ -177,90 +176,6 @@ int ci_putblk(struct channel *chn, const char *blk, int len)
 	return len;
 }
 
-/* Locates the longest part of the channel's output buffer that is composed
- * exclusively of characters not in the <delim> set, and delimited by one of
- * these characters, and returns the initial part and the first of such
- * delimiters. A single escape character in <escape> may be specified so that
- * when not 0 and found, the character that follows it is never taken as a
- * delimiter. Note that <delim> cannot contain the zero byte, hence this
- * function is not usable with byte zero as a delimiter.
- *
- * Return values :
- *   >0 : number of bytes read. Includes the sep if present before len or end.
- *   =0 : no sep before end found. <str> is left undefined.
- *   <0 : no more bytes readable because output is shut.
- * The channel status is not changed. The caller must call co_skip() to
- * update it. One of the delimiters is waited for as long as neither the buffer
- * nor the output are full. If either of them is full, the string may be
- * returned as is, without the delimiter.
- */
-int co_getdelim(const struct channel *chn, char *str, int len, const char *delim, char escape)
-{
-	unsigned char delim_map[256 / 8];
-	int found, escaped;
-	uint pos, bit;
-	int ret, max;
-	unsigned char b;
-	char *p;
-
-	ret = 0;
-	max = len;
-
-	/* closed or empty + imminent close = -1; empty = 0 */
-	if (unlikely((chn->flags & CF_SHUTW) || channel_is_empty(chn))) {
-		if (chn->flags & (CF_SHUTW|CF_SHUTW_NOW))
-			ret = -1;
-		goto out;
-	}
-
-	p = co_head(chn);
-
-	if (max > co_data(chn)) {
-		max = co_data(chn);
-		str[max-1] = 0;
-	}
-
-	/* create the byte map */
-	memset(delim_map, 0, sizeof(delim_map));
-	while ((b = *delim)) {
-		pos = b >> 3;
-		bit = b &  7;
-		delim_map[pos] |= 1 << bit;
-		delim++;
-	}
-
-	found = escaped = 0;
-	while (max) {
-		*str++ = b = *p;
-		ret++;
-		max--;
-
-		if (escape && (escaped || *p == escape)) {
-			escaped = !escaped;
-			goto skip;
-		}
-
-		pos = b >> 3;
-		bit = b &  7;
-		if (delim_map[pos] & (1 << bit)) {
-			found = 1;
-			break;
-		}
-	skip:
-		p = b_next(&chn->buf, p);
-	}
-
-	if (ret > 0 && ret < len &&
-	    (ret < co_data(chn) || channel_may_recv(chn)) &&
-	    !found &&
-	    !(chn->flags & (CF_SHUTW|CF_SHUTW_NOW)))
-		ret = 0;
- out:
-	if (max)
-		*str = 0;
-	return ret;
-}
-
 /* Gets one text line out of a channel's buffer from a stream interface.
  * Return values :
  *   >0 : number of bytes read. Includes the \n if present before len or end.
@@ -345,7 +260,7 @@ int co_getblk(const struct channel *chn, char *blk, int len, int offset)
 int co_getblk_nc(const struct channel *chn, const char **blk1, size_t *len1, const char **blk2, size_t *len2)
 {
 	if (unlikely(co_data(chn) == 0)) {
-		if (chn->flags & (CF_SHUTW|CF_SHUTW_NOW))
+		if (chn->flags & CF_SHUTW)
 			return -1;
 		return 0;
 	}
@@ -387,7 +302,7 @@ int co_getline_nc(const struct channel *chn,
 		}
 	}
 
-	if (chn->flags & (CF_SHUTW|CF_SHUTW_NOW)) {
+	if (chn->flags & CF_SHUTW) {
 		/* If we have found no LF and the buffer is shut, then
 		 * the resulting string is made of the concatenation of
 		 * the pending blocks (1 or 2).
