@@ -26,10 +26,9 @@
 #include <types/stream.h>
 
 #include <proto/filters.h>
-#include <proto/hdr_idx.h>
 #include <proto/http_htx.h>
 #include <proto/log.h>
-#include <proto/proto_http.h>
+#include <proto/http_ana.h>
 #include <proto/stream.h>
 
 const char *trace_flt_id = "trace filter";
@@ -44,12 +43,12 @@ struct trace_config {
 	int           hexdump;
 };
 
-#define TRACE(conf, fmt, ...)						\
+#define FLT_TRACE(conf, fmt, ...)						\
 	fprintf(stderr, "%d.%06d [%-20s] " fmt "\n",			\
 		(int)now.tv_sec, (int)now.tv_usec, (conf)->name,	\
 		##__VA_ARGS__)
 
-#define STRM_TRACE(conf, strm, fmt, ...)						\
+#define FLT_STRM_TRACE(conf, strm, fmt, ...)						\
 	fprintf(stderr, "%d.%06d [%-20s] [strm %p(%x) 0x%08x 0x%08x] " fmt "\n",	\
 		(int)now.tv_sec, (int)now.tv_usec, (conf)->name,			\
 		strm, (strm ? ((struct stream *)strm)->uniq_id : ~0U),			\
@@ -68,9 +67,7 @@ proxy_mode(const struct stream *s)
 {
 	struct proxy *px = (s->flags & SF_BE_ASSIGNED ? s->be : strm_fe(s));
 
-	return ((px->mode == PR_MODE_HTTP)
-		? (IS_HTX_STRM(s) ? "HTX" : "HTTP")
-		: "TCP");
+	return ((px->mode == PR_MODE_HTTP) ? "HTTP" : "TCP");
 }
 
 static const char *
@@ -106,24 +103,24 @@ trace_hexdump(struct ist ist)
                 if (i % 16 == 15) {
                         fprintf(stderr, "  |");
                         for(j = i - 15; j <= i && j < ist.len; j++)
-				fprintf(stderr, "%c", (isprint((unsigned char)*(ist.ptr+j)) ? *(ist.ptr+j) : '.'));
+				fprintf(stderr, "%c", (isprint(*(ist.ptr+j)) ? *(ist.ptr+j) : '.'));
                         fprintf(stderr, "|\n");
                 }
         }
 }
 
 static void
-trace_raw_hexdump(struct buffer *buf, int len, int out)
+trace_raw_hexdump(struct buffer *buf, unsigned int offset, unsigned int len)
 {
 	unsigned char p[len];
 	int block1, block2;
 
 	block1 = len;
-	if (block1 > b_contig_data(buf, out))
-		block1 = b_contig_data(buf, out);
+	if (block1 > b_contig_data(buf, offset))
+		block1 = b_contig_data(buf, offset);
 	block2 = len - block1;
 
-	memcpy(p, b_head(buf), block1);
+	memcpy(p, b_peek(buf, offset), block1);
 	memcpy(p+block1, b_orig(buf), block2);
 	trace_hexdump(ist2(p, len));
 }
@@ -156,6 +153,31 @@ trace_htx_hexdump(struct htx *htx, unsigned int offset, unsigned int len)
 	}
 }
 
+static unsigned int
+trace_get_htx_datalen(struct htx *htx, unsigned int offset, unsigned int len)
+{
+	struct htx_blk *blk;
+	uint32_t sz, data = 0;
+
+	for (blk = htx_get_first_blk(htx); blk; blk = htx_get_next_blk(htx, blk)) {
+		if (htx_get_blk_type(blk) != HTX_BLK_DATA)
+			break;
+
+		sz = htx_get_blksz(blk);
+		if (offset >= sz) {
+			offset -= sz;
+			continue;
+		}
+		data  += sz - offset;
+		offset = 0;
+		if (data > len) {
+			data = len;
+			break;
+		}
+	}
+	return data;
+}
+
 /***************************************************************************
  * Hooks that manage the filter lifecycle (init/check/deinit)
  **************************************************************************/
@@ -173,7 +195,7 @@ trace_init(struct proxy *px, struct flt_conf *fconf)
 	fconf->flags |= FLT_CFG_FL_HTX;
 	fconf->conf = conf;
 
-	TRACE(conf, "filter initialized [read random=%s - fwd random=%s - hexdump=%s]",
+	FLT_TRACE(conf, "filter initialized [read random=%s - fwd random=%s - hexdump=%s]",
 	      (conf->rand_parsing ? "true" : "false"),
 	      (conf->rand_forwarding ? "true" : "false"),
 	      (conf->hexdump ? "true" : "false"));
@@ -187,7 +209,7 @@ trace_deinit(struct proxy *px, struct flt_conf *fconf)
 	struct trace_config *conf = fconf->conf;
 
 	if (conf) {
-		TRACE(conf, "filter deinitialized");
+		FLT_TRACE(conf, "filter deinitialized");
 		free(conf->name);
 		free(conf);
 	}
@@ -208,7 +230,7 @@ trace_init_per_thread(struct proxy *px, struct flt_conf *fconf)
 {
 	struct trace_config *conf = fconf->conf;
 
-	TRACE(conf, "filter initialized for thread tid %u", tid);
+	FLT_TRACE(conf, "filter initialized for thread tid %u", tid);
 	return 0;
 }
 
@@ -219,7 +241,7 @@ trace_deinit_per_thread(struct proxy *px, struct flt_conf *fconf)
 	struct trace_config *conf = fconf->conf;
 
 	if (conf)
-		TRACE(conf, "filter deinitialized for thread tid %u", tid);
+		FLT_TRACE(conf, "filter deinitialized for thread tid %u", tid);
 }
 
 /**************************************************************************
@@ -231,7 +253,7 @@ trace_attach(struct stream *s, struct filter *filter)
 {
 	struct trace_config *conf = FLT_CONF(filter);
 
-	STRM_TRACE(conf, s, "%-25s: filter-type=%s",
+	FLT_STRM_TRACE(conf, s, "%-25s: filter-type=%s",
 		   __FUNCTION__, filter_type(filter));
 
 	return 1;
@@ -244,7 +266,7 @@ trace_detach(struct stream *s, struct filter *filter)
 {
 	struct trace_config *conf = FLT_CONF(filter);
 
-	STRM_TRACE(conf, s, "%-25s: filter-type=%s",
+	FLT_STRM_TRACE(conf, s, "%-25s: filter-type=%s",
 		   __FUNCTION__, filter_type(filter));
 }
 
@@ -254,7 +276,7 @@ trace_stream_start(struct stream *s, struct filter *filter)
 {
 	struct trace_config *conf = FLT_CONF(filter);
 
-	STRM_TRACE(conf, s, "%-25s",
+	FLT_STRM_TRACE(conf, s, "%-25s",
 		   __FUNCTION__);
 	return 0;
 }
@@ -267,7 +289,7 @@ trace_stream_set_backend(struct stream *s, struct filter *filter,
 {
 	struct trace_config *conf = FLT_CONF(filter);
 
-	STRM_TRACE(conf, s, "%-25s: backend=%s",
+	FLT_STRM_TRACE(conf, s, "%-25s: backend=%s",
 		   __FUNCTION__, be->id);
 	return 0;
 }
@@ -278,7 +300,7 @@ trace_stream_stop(struct stream *s, struct filter *filter)
 {
 	struct trace_config *conf = FLT_CONF(filter);
 
-	STRM_TRACE(conf, s, "%-25s",
+	FLT_STRM_TRACE(conf, s, "%-25s",
 		   __FUNCTION__);
 }
 
@@ -288,7 +310,7 @@ trace_check_timeouts(struct stream *s, struct filter *filter)
 {
 	struct trace_config *conf = FLT_CONF(filter);
 
-	STRM_TRACE(conf, s, "%-25s",
+	FLT_STRM_TRACE(conf, s, "%-25s",
 		   __FUNCTION__);
 }
 
@@ -302,7 +324,7 @@ trace_chn_start_analyze(struct stream *s, struct filter *filter,
 {
 	struct trace_config *conf = FLT_CONF(filter);
 
-	STRM_TRACE(conf, s, "%-25s: channel=%-10s - mode=%-5s (%s)",
+	FLT_STRM_TRACE(conf, s, "%-25s: channel=%-10s - mode=%-5s (%s)",
 		   __FUNCTION__,
 		   channel_label(chn), proxy_mode(s), stream_pos(s));
 	filter->pre_analyzers  |= (AN_REQ_ALL | AN_RES_ALL);
@@ -378,7 +400,7 @@ trace_chn_analyze(struct stream *s, struct filter *filter,
 			ana = "unknown";
 	}
 
-	STRM_TRACE(conf, s, "%-25s: channel=%-10s - mode=%-5s (%s) - "
+	FLT_STRM_TRACE(conf, s, "%-25s: channel=%-10s - mode=%-5s (%s) - "
 		   "analyzer=%s - step=%s",
 		   __FUNCTION__,
 		   channel_label(chn), proxy_mode(s), stream_pos(s),
@@ -393,7 +415,7 @@ trace_chn_end_analyze(struct stream *s, struct filter *filter,
 {
 	struct trace_config *conf = FLT_CONF(filter);
 
-	STRM_TRACE(conf, s, "%-25s: channel=%-10s - mode=%-5s (%s)",
+	FLT_STRM_TRACE(conf, s, "%-25s: channel=%-10s - mode=%-5s (%s)",
 		   __FUNCTION__,
 		   channel_label(chn), proxy_mode(s), stream_pos(s));
 	return 1;
@@ -407,52 +429,31 @@ trace_http_headers(struct stream *s, struct filter *filter,
 		   struct http_msg *msg)
 {
 	struct trace_config *conf = FLT_CONF(filter);
+	struct htx *htx = htxbuf(&msg->chn->buf);
+	struct htx_sl *sl = http_get_stline(htx);
+	int32_t pos;
 
-	STRM_TRACE(conf, s, "%-25s: channel=%-10s - mode=%-5s (%s)",
+	FLT_STRM_TRACE(conf, s, "%-25s: channel=%-10s - mode=%-5s (%s)\t%.*s %.*s %.*s",
 		   __FUNCTION__,
-		   channel_label(msg->chn), proxy_mode(s), stream_pos(s));
+		   channel_label(msg->chn), proxy_mode(s), stream_pos(s),
+		   HTX_SL_P1_LEN(sl), HTX_SL_P1_PTR(sl),
+		   HTX_SL_P2_LEN(sl), HTX_SL_P2_PTR(sl),
+		   HTX_SL_P3_LEN(sl), HTX_SL_P3_PTR(sl));
 
-	if (IS_HTX_STRM(s)) {
-		struct htx *htx = htxbuf(&msg->chn->buf);
-		struct htx_sl *sl = http_get_stline(htx);
-		int32_t pos;
+	for (pos = htx_get_first(htx); pos != -1; pos = htx_get_next(htx, pos)) {
+		struct htx_blk *blk = htx_get_blk(htx, pos);
+		enum htx_blk_type type = htx_get_blk_type(blk);
+		struct ist n, v;
 
-		STRM_TRACE(conf, s, "\t%.*s %.*s %.*s",
-			   HTX_SL_P1_LEN(sl), HTX_SL_P1_PTR(sl),
-			   HTX_SL_P2_LEN(sl), HTX_SL_P2_PTR(sl),
-			   HTX_SL_P3_LEN(sl), HTX_SL_P3_PTR(sl));
+		if (type == HTX_BLK_EOH)
+			break;
+		if (type != HTX_BLK_HDR)
+			continue;
 
-		for (pos = htx_get_first(htx); pos != -1; pos = htx_get_next(htx, pos)) {
-			struct htx_blk *blk = htx_get_blk(htx, pos);
-			enum htx_blk_type type = htx_get_blk_type(blk);
-			struct ist n, v;
-
-			if (type == HTX_BLK_EOH)
-				break;
-			if (type != HTX_BLK_HDR)
-				continue;
-
-			n = htx_get_blk_name(htx, blk);
-			v = htx_get_blk_value(htx, blk);
-			STRM_TRACE(conf, s, "\t%.*s: %.*s",
-				   (int)n.len, n.ptr, (int)v.len, v.ptr);
-		}
-	}
-	else {
-		struct hdr_idx      *hdr_idx;
-		char                *cur_hdr;
-		int                  cur_idx;
-
-		STRM_TRACE(conf, s, "\t%.*s", MIN(msg->sl.rq.l, 74), ci_head(msg->chn));
-		hdr_idx = &s->txn->hdr_idx;
-		cur_idx = hdr_idx_first_idx(hdr_idx);
-		cur_hdr = ci_head(msg->chn) + hdr_idx_first_pos(hdr_idx);
-		while (cur_idx) {
-			STRM_TRACE(conf, s, "\t%.*s",
-				   MIN(hdr_idx->v[cur_idx].len, 74), cur_hdr);
-			cur_hdr += hdr_idx->v[cur_idx].len + hdr_idx->v[cur_idx].cr + 1;
-			cur_idx = hdr_idx->v[cur_idx].next;
-		}
+		n = htx_get_blk_name(htx, blk);
+		v = htx_get_blk_value(htx, blk);
+		FLT_STRM_TRACE(conf, s, "\t%.*s: %.*s",
+			   (int)n.len, n.ptr, (int)v.len, v.ptr);
 	}
 	return 1;
 }
@@ -465,89 +466,36 @@ trace_http_payload(struct stream *s, struct filter *filter, struct http_msg *msg
 	int ret = len;
 
 	if (ret && conf->rand_forwarding) {
-		struct htx *htx = htxbuf(&msg->chn->buf);
-		struct htx_blk *blk;
-		uint32_t sz, data = 0;
-		unsigned int off = offset;
+		unsigned int data = trace_get_htx_datalen(htxbuf(&msg->chn->buf), offset, len);
 
-		for (blk = htx_get_first_blk(htx); blk; blk = htx_get_next_blk(htx, blk)) {
-			if (htx_get_blk_type(blk) != HTX_BLK_DATA)
-				break;
-
-			sz = htx_get_blksz(blk);
-			if (off >= sz) {
-				off -= sz;
-				continue;
-			}
-			data  += sz - off;
-			off = 0;
-			if (data > len) {
-				data = len;
-				break;
-			}
-		}
-		if (data)  {
-			ret = ha_random() % (ret+1);
+		if (data) {
+			ret = random() % (ret+1);
 			if (!ret || ret >= data)
 				ret = len;
 		}
 	}
 
-	STRM_TRACE(conf, s, "%-25s: channel=%-10s - mode=%-5s (%s) - "
+	FLT_STRM_TRACE(conf, s, "%-25s: channel=%-10s - mode=%-5s (%s) - "
 		   "offset=%u - len=%u - forward=%d",
 		   __FUNCTION__,
 		   channel_label(msg->chn), proxy_mode(s), stream_pos(s),
 		   offset, len, ret);
 
 	 if (conf->hexdump)
-		 trace_htx_hexdump(htxbuf(&msg->chn->buf), offset, len);
+		 trace_htx_hexdump(htxbuf(&msg->chn->buf), offset, ret);
 
 	 if (ret != len)
 		 task_wakeup(s->task, TASK_WOKEN_MSG);
 	return ret;
 }
 
-static int
-trace_http_data(struct stream *s, struct filter *filter,
-		      struct http_msg *msg)
-{
-	struct trace_config *conf = FLT_CONF(filter);
-	int avail = MIN(msg->chunk_len + msg->next, ci_data(msg->chn)) - FLT_NXT(filter, msg->chn);
-	int ret   = avail;
-
-	if (ret && conf->rand_parsing)
-		ret = ha_random() % (ret+1);
-
-	STRM_TRACE(conf, s, "%-25s: channel=%-10s - mode=%-5s (%s) - "
-		   "chunk_len=%llu - next=%u - fwd=%u - avail=%d - consume=%d",
-		   __FUNCTION__,
-		   channel_label(msg->chn), proxy_mode(s), stream_pos(s),
-		   msg->chunk_len, FLT_NXT(filter, msg->chn),
-		   FLT_FWD(filter, msg->chn), avail, ret);
-	if (ret != avail)
-		task_wakeup(s->task, TASK_WOKEN_MSG);
-	return ret;
-}
-
-static int
-trace_http_chunk_trailers(struct stream *s, struct filter *filter,
-			  struct http_msg *msg)
-{
-	struct trace_config *conf = FLT_CONF(filter);
-
-	STRM_TRACE(conf, s, "%-25s: channel=%-10s - mode=%-5s (%s)",
-		   __FUNCTION__,
-		   channel_label(msg->chn), proxy_mode(s), stream_pos(s));
-	return 1;
-}
-
 static int
 trace_http_end(struct stream *s, struct filter *filter,
 	       struct http_msg *msg)
 {
 	struct trace_config *conf = FLT_CONF(filter);
 
-	STRM_TRACE(conf, s, "%-25s: channel=%-10s - mode=%-5s (%s)",
+	FLT_STRM_TRACE(conf, s, "%-25s: channel=%-10s - mode=%-5s (%s)",
 		   __FUNCTION__,
 		   channel_label(msg->chn), proxy_mode(s), stream_pos(s));
 	return 1;
@@ -559,7 +507,7 @@ trace_http_reset(struct stream *s, struct filter *filter,
 {
 	struct trace_config *conf = FLT_CONF(filter);
 
-	STRM_TRACE(conf, s, "%-25s: channel=%-10s - mode=%-5s (%s)",
+	FLT_STRM_TRACE(conf, s, "%-25s: channel=%-10s - mode=%-5s (%s)",
 		   __FUNCTION__,
 		   channel_label(msg->chn), proxy_mode(s), stream_pos(s));
 }
@@ -570,87 +518,59 @@ trace_http_reply(struct stream *s, struct filter *filter, short status,
 {
 	struct trace_config *conf = FLT_CONF(filter);
 
-	STRM_TRACE(conf, s, "%-25s: channel=%-10s - mode=%-5s (%s)",
+	FLT_STRM_TRACE(conf, s, "%-25s: channel=%-10s - mode=%-5s (%s)",
 		   __FUNCTION__, "-", proxy_mode(s), stream_pos(s));
 }
 
-static int
-trace_http_forward_data(struct stream *s, struct filter *filter,
-			struct http_msg *msg, unsigned int len)
-{
-	struct trace_config *conf = FLT_CONF(filter);
-	int                  ret  = len;
-
-	if (ret && conf->rand_forwarding)
-		ret = ha_random() % (ret+1);
-
-	STRM_TRACE(conf, s, "%-25s: channel=%-10s - mode=%-5s (%s) - "
-		   "len=%u - nxt=%u - fwd=%u - forward=%d",
-		   __FUNCTION__,
-		   channel_label(msg->chn), proxy_mode(s), stream_pos(s), len,
-		   FLT_NXT(filter, msg->chn), FLT_FWD(filter, msg->chn), ret);
-
-	if (conf->hexdump) {
-		c_adv(msg->chn, FLT_FWD(filter, msg->chn));
-		trace_raw_hexdump(&msg->chn->buf, ret, co_data(msg->chn));
-		c_rew(msg->chn, FLT_FWD(filter, msg->chn));
-	}
-
-	if ((ret != len) ||
-	    (FLT_NXT(filter, msg->chn) != FLT_FWD(filter, msg->chn) + ret))
-		task_wakeup(s->task, TASK_WOKEN_MSG);
-	return ret;
-}
-
 /**************************************************************************
  * Hooks to filter TCP data
  *************************************************************************/
 static int
-trace_tcp_data(struct stream *s, struct filter *filter, struct channel *chn)
+trace_tcp_payload(struct stream *s, struct filter *filter, struct channel *chn,
+		  unsigned int offset, unsigned int len)
 {
 	struct trace_config *conf = FLT_CONF(filter);
-	int                  avail = ci_data(chn) - FLT_NXT(filter, chn);
-	int                  ret  = avail;
+	int ret = len;
 
-	if (ret && conf->rand_parsing)
-		ret = ha_random() % (ret+1);
+	if (s->flags & SF_HTX) {
+		if (ret && conf->rand_forwarding) {
+			unsigned int data = trace_get_htx_datalen(htxbuf(&chn->buf), offset, len);
 
-	STRM_TRACE(conf, s, "%-25s: channel=%-10s - mode=%-5s (%s) - next=%u - avail=%u - consume=%d",
-		   __FUNCTION__,
-		   channel_label(chn), proxy_mode(s), stream_pos(s),
-		   FLT_NXT(filter, chn), avail, ret);
+			if (data) {
+				ret = random() % (ret+1);
+				if (!ret || ret >= data)
+					ret = len;
+			}
+		}
 
-	if (ret != avail)
-		task_wakeup(s->task, TASK_WOKEN_MSG);
-	return ret;
-}
+		FLT_STRM_TRACE(conf, s, "%-25s: channel=%-10s - mode=%-5s (%s) - "
+			       "offset=%u - len=%u - forward=%d",
+			       __FUNCTION__,
+			       channel_label(chn), proxy_mode(s), stream_pos(s),
+			       offset, len, ret);
 
-static int
-trace_tcp_forward_data(struct stream *s, struct filter *filter, struct channel *chn,
-		 unsigned int len)
-{
-	struct trace_config *conf = FLT_CONF(filter);
-	int                  ret  = len;
+		if (conf->hexdump)
+			trace_htx_hexdump(htxbuf(&chn->buf), offset, ret);
+	}
+	else {
 
-	if (ret && conf->rand_forwarding)
-		ret = ha_random() % (ret+1);
+		if (ret && conf->rand_forwarding)
+			ret = random() % (ret+1);
 
-	STRM_TRACE(conf, s, "%-25s: channel=%-10s - mode=%-5s (%s) - len=%u - fwd=%u - forward=%d",
-		   __FUNCTION__,
-		   channel_label(chn), proxy_mode(s), stream_pos(s), len,
-		   FLT_FWD(filter, chn), ret);
+		FLT_STRM_TRACE(conf, s, "%-25s: channel=%-10s - mode=%-5s (%s) - "
+			       "offset=%u - len=%u - forward=%d",
+			       __FUNCTION__,
+			       channel_label(chn), proxy_mode(s), stream_pos(s),
+			       offset, len, ret);
 
-	if (conf->hexdump) {
-		c_adv(chn, FLT_FWD(filter, chn));
-		trace_raw_hexdump(&chn->buf, ret, co_data(chn));
-		c_rew(chn, FLT_FWD(filter, chn));
+		if (conf->hexdump)
+			trace_raw_hexdump(&chn->buf, offset, ret);
 	}
 
-	if (ret != len)
-		task_wakeup(s->task, TASK_WOKEN_MSG);
+	 if (ret != len)
+		 task_wakeup(s->task, TASK_WOKEN_MSG);
 	return ret;
 }
-
 /********************************************************************
  * Functions that manage the filter initialization
  ********************************************************************/
@@ -679,18 +599,12 @@ struct flt_ops trace_ops = {
 	/* Filter HTTP requests and responses */
 	.http_headers        = trace_http_headers,
 	.http_payload        = trace_http_payload,
-
-	.http_data           = trace_http_data,
-	.http_chunk_trailers = trace_http_chunk_trailers,
 	.http_end            = trace_http_end,
-
 	.http_reset          = trace_http_reset,
 	.http_reply          = trace_http_reply,
-	.http_forward_data   = trace_http_forward_data,
 
 	/* Filter TCP data */
-	.tcp_data         = trace_tcp_data,
-	.tcp_forward_data = trace_tcp_forward_data,
+	.tcp_payload        = trace_tcp_payload,
 };
 
 /* Return -1 on error, else 0 */
