@@ -16,7 +16,6 @@
 #include <common/http-hdr.h>
 
 #include <proto/channel.h>
-#include <proto/hdr_idx.h>
 
 /* Parse the Content-Length header field of an HTTP/1 request. The function
  * checks all possible occurrences of a comma-delimited value, and verifies
@@ -32,20 +31,13 @@ int h1_parse_cont_len_header(struct h1m *h1m, struct ist *value)
 	int not_first = !!(h1m->flags & H1_MF_CLEN);
 	struct ist word;
 
-	word.ptr = value->ptr;
+	word.ptr = value->ptr - 1; // -1 for next loop's pre-increment
 	e = value->ptr + value->len;
 
-	while (1) {
-		if (word.ptr >= e) {
-			/* empty header or empty value */
-			goto fail;
-		}
-
-		/* skip leading delimiter and blanks */
-		if (unlikely(HTTP_IS_LWS(*word.ptr))) {
-			word.ptr++;
+	while (++word.ptr < e) {
+		/* skip leading delimitor and blanks */
+		if (unlikely(HTTP_IS_LWS(*word.ptr)))
 			continue;
-		}
 
 		/* digits only now */
 		for (cl = 0, n = word.ptr; n < e; n++) {
@@ -56,14 +48,6 @@ int h1_parse_cont_len_header(struct h1m *h1m, struct ist *value)
 					goto fail;
 				break;
 			}
-
-			if (unlikely(!cl && n > word.ptr)) {
-				/* There was a leading zero before this digit,
-				 * let's trim it.
-				 */
-				word.ptr = n;
-			}
-
 			if (unlikely(cl > ULLONG_MAX / 10ULL))
 				goto fail; /* multiply overflow */
 			cl = cl * 10ULL;
@@ -92,13 +76,6 @@ int h1_parse_cont_len_header(struct h1m *h1m, struct ist *value)
 		h1m->flags |= H1_MF_CLEN;
 		h1m->curr_len = h1m->body_len = cl;
 		*value = word;
-
-		/* Now either n==e and we're done, or n points to the comma,
-		 * and we skip it and continue.
-		 */
-		if (n++ == e)
-			break;
-
 		word.ptr = n;
 	}
 	/* here we've reached the end with a single value or a series of
@@ -305,6 +282,7 @@ int h1_headers_to_hdr_list(char *start, const char *stop,
 	union h1_sl sl;
 	int skip_update;
 	int restarting;
+	int host_idx;
 	struct ist n, v;       /* header name and value during parsing */
 
 	skip = 0; // do it only once to keep track of the leading CRLF.
@@ -313,6 +291,7 @@ int h1_headers_to_hdr_list(char *start, const char *stop,
 	hdr_count = sol = col = eol = sov = 0;
 	sl.st.status = 0;
 	skip_update = restarting = 0;
+	host_idx = -1;
 
 	if (h1m->flags & H1_MF_HDRS_ONLY) {
 		state = H1_MSG_HDR_FIRST;
@@ -412,11 +391,11 @@ int h1_headers_to_hdr_list(char *start, const char *stop,
     defined(__ARM_ARCH_7A__)
 		/* speedup: skip bytes not between 0x21 and 0x7e inclusive */
 		while (ptr <= end - sizeof(int)) {
-			int x = *(int *)ptr - 0x24242424;
+			int x = *(int *)ptr - 0x21212121;
 			if (x & 0x80808080)
 				break;
 
-			x -= 0x5b5b5b5b;
+			x -= 0x5e5e5e5e;
 			if (!(x & 0x80808080))
 				break;
 
@@ -428,15 +407,8 @@ int h1_headers_to_hdr_list(char *start, const char *stop,
 			goto http_msg_ood;
 		}
 	http_msg_rquri2:
-		if (likely((unsigned char)(*ptr - 33) <= 93)) { /* 33 to 126 included */
-			if (*ptr == '#') {
-				if (h1m->err_pos < -1) /* PR_O2_REQBUG_OK not set */
-					goto invalid_char;
-				if (h1m->err_pos == -1) /* PR_O2_REQBUG_OK set: just log */
-					h1m->err_pos = ptr - start + skip;
-			}
+		if (likely((unsigned char)(*ptr - 33) <= 93)) /* 33 to 126 included */
 			EAT_AND_JUMP_OR_RETURN(ptr, end, http_msg_rquri2, http_msg_ood, state, H1_MSG_RQURI);
-		}
 
 		if (likely(HTTP_IS_SPHT(*ptr))) {
 			sl.rq.u.len = ptr - sl.rq.u.ptr;
@@ -701,10 +673,6 @@ int h1_headers_to_hdr_list(char *start, const char *stop,
 
 		if (likely(*ptr == ':')) {
 			col = ptr - start;
-			if (col <= sol) {
-				state = H1_MSG_HDR_NAME;
-				goto http_msg_invalid;
-			}
 			EAT_AND_JUMP_OR_RETURN(ptr, end, http_msg_hdr_l1_sp, http_msg_ood, state, H1_MSG_HDR_L1_SP);
 		}
 
@@ -851,7 +819,6 @@ int h1_headers_to_hdr_list(char *start, const char *stop,
 
 					if (ret < 0) {
 						state = H1_MSG_HDR_L2_LWS;
-						ptr = v.ptr; /* Set ptr on the error */
 						goto http_msg_invalid;
 					}
 					else if (ret == 0) {
@@ -866,6 +833,30 @@ int h1_headers_to_hdr_list(char *start, const char *stop,
 						break;
 					}
 				}
+				else if (isteqi(n, ist("host"))) {
+					if (host_idx == -1) {
+						struct ist authority;
+
+						authority = http_get_authority(sl.rq.u, 1);
+						if (authority.len && !isteqi(v, authority)) {
+							if (h1m->err_pos < -1) {
+								state = H1_MSG_HDR_L2_LWS;
+								goto http_msg_invalid;
+							}
+							if (h1m->err_pos == -1) /* capture the error pointer */
+								h1m->err_pos = ptr - start + skip; /* >= 0 now */
+						}
+						host_idx = hdr_count;
+					}
+					else {
+						if (!isteqi(v, hdr[host_idx].v)) {
+							state = H1_MSG_HDR_L2_LWS;
+							goto http_msg_invalid;
+						}
+						/* if the same host, skip it */
+						break;
+					}
+				}
 
 				http_set_hdr(&hdr[hdr_count++], n, v);
 			} while (0);
