@@ -51,7 +51,7 @@ struct xfer_sock_list *xfer_sock_list = NULL;
 
 /* there is one listener queue per thread so that a thread unblocking the
  * global queue can wake up listeners bound only to foreing threads by
- * moving them to the remote queues and waking up the associated task.
+ * moving them to the remote queues and waking up the associated tasklet.
  */
 static struct work_list *local_listener_queue;
 
@@ -194,27 +194,27 @@ static struct task *accept_queue_process(struct task *t, void *context, unsigned
 
 	/* ran out of budget ? Let's come here ASAP */
 	if (!max_accept)
-		task_wakeup(t, TASK_WOKEN_IO);
+		tasklet_wakeup(ring->tasklet);
 
-	return t;
+	return NULL;
 }
 
 /* Initializes the accept-queues. Returns 0 on success, otherwise ERR_* flags */
 static int accept_queue_init()
 {
-	struct task *t;
+	struct tasklet *t;
 	int i;
 
 	for (i = 0; i < global.nbthread; i++) {
-		t = task_new(1UL << i);
+		t = tasklet_new();
 		if (!t) {
 			ha_alert("Out of memory while initializing accept queue for thread %d\n", i);
 			return ERR_FATAL|ERR_ABORT;
 		}
-		t->nice = -1024;
+		t->tid = i;
 		t->process = accept_queue_process;
 		t->context = &accept_queue_rings[i];
-		accept_queue_rings[i].task = t;
+		accept_queue_rings[i].tasklet = t;
 	}
 	return 0;
 }
@@ -273,7 +273,7 @@ static void disable_listener(struct listener *listener)
 		goto end;
 	if (listener->state == LI_READY)
 		fd_stop_recv(listener->fd);
-	LIST_DEL_LOCKED(&listener->wait_queue);
+	MT_LIST_DEL(&listener->wait_queue);
 	listener->state = LI_LISTEN;
   end:
 	HA_SPIN_UNLOCK(LISTENER_LOCK, &listener->lock);
@@ -295,10 +295,6 @@ int pause_listener(struct listener *l)
 	if (l->state <= LI_ZOMBIE)
 		goto end;
 
-	if ((global.mode & (MODE_DAEMON | MODE_MWORKER)) &&
-	    !(proc_mask(l->bind_conf->bind_proc) & pid_bit))
-		goto end;
-
 	if (l->proto->pause) {
 		/* Returns < 0 in case of failure, 0 if the listener
 		 * was totally stopped, or > 0 if correctly paused.
@@ -313,7 +309,7 @@ int pause_listener(struct listener *l)
 			goto end;
 	}
 
-	LIST_DEL_LOCKED(&l->wait_queue);
+	MT_LIST_DEL(&l->wait_queue);
 
 	fd_stop_recv(l->fd);
 	l->state = LI_PAUSED;
@@ -341,7 +337,7 @@ int resume_listener(struct listener *l)
 	/* check that another thread didn't to the job in parallel (e.g. at the
 	 * end of listen_accept() while we'd come from dequeue_all_listeners().
 	 */
-	if (LIST_ADDED(&l->wait_queue))
+	if (MT_LIST_ADDED(&l->wait_queue))
 		goto end;
 
 	if ((global.mode & (MODE_DAEMON | MODE_MWORKER)) &&
@@ -379,7 +375,7 @@ int resume_listener(struct listener *l)
 	if (l->state == LI_READY)
 		goto end;
 
-	LIST_DEL_LOCKED(&l->wait_queue);
+	MT_LIST_DEL(&l->wait_queue);
 
 	if (l->maxconn && l->nbconn >= l->maxconn) {
 		l->state = LI_FULL;
@@ -390,7 +386,7 @@ int resume_listener(struct listener *l)
 		/* we're not allowed to touch this listener's FD, let's requeue
 		 * the listener into one of its owning thread's queue instead.
 		 */
-		int first_thread = my_flsl(thread_mask(l->bind_conf->bind_thread) & all_threads_mask) - 1;
+		int first_thread = my_flsl(thread_mask(l->bind_conf->bind_thread)) - 1;
 		work_list_add(&local_listener_queue[first_thread], &l->wait_queue);
 		goto end;
 	}
@@ -409,7 +405,7 @@ static void listener_full(struct listener *l)
 {
 	HA_SPIN_LOCK(LISTENER_LOCK, &l->lock);
 	if (l->state >= LI_READY) {
-		LIST_DEL_LOCKED(&l->wait_queue);
+		MT_LIST_DEL(&l->wait_queue);
 		if (l->state != LI_FULL) {
 			fd_stop_recv(l->fd);
 			l->state = LI_FULL;
@@ -421,11 +417,11 @@ static void listener_full(struct listener *l)
 /* Marks a ready listener as limited so that we only try to re-enable it when
  * resources are free again. It will be queued into the specified queue.
  */
-static void limit_listener(struct listener *l, struct list *list)
+static void limit_listener(struct listener *l, struct mt_list *list)
 {
 	HA_SPIN_LOCK(LISTENER_LOCK, &l->lock);
 	if (l->state == LI_READY) {
-		LIST_ADDQ_LOCKED(list, &l->wait_queue);
+		MT_LIST_ADDQ(list, &l->wait_queue);
 		fd_stop_recv(l->fd);
 		l->state = LI_LIMITED;
 	}
@@ -468,11 +464,11 @@ int disable_all_listeners(struct protocol *proto)
 }
 
 /* Dequeues all of the listeners waiting for a resource in wait queue <queue>. */
-void dequeue_all_listeners(struct list *list)
+void dequeue_all_listeners(struct mt_list *list)
 {
 	struct listener *listener;
 
-	while ((listener = LIST_POP_LOCKED(list, struct listener *, wait_queue))) {
+	while ((listener = MT_LIST_POP(list, struct listener *, wait_queue))) {
 		/* This cannot fail because the listeners are by definition in
 		 * the LI_LIMITED state.
 		 */
@@ -488,7 +484,7 @@ void do_unbind_listener(struct listener *listener, int do_close)
 	if (listener->state == LI_READY && fd_updt)
 		fd_stop_recv(listener->fd);
 
-	LIST_DEL_LOCKED(&listener->wait_queue);
+	MT_LIST_DEL(&listener->wait_queue);
 
 	if (listener->state >= LI_PAUSED) {
 		if (do_close) {
@@ -573,7 +569,7 @@ int create_listeners(struct bind_conf *bc, const struct sockaddr_storage *ss,
 
 		l->fd = fd;
 		memcpy(&l->addr, ss, sizeof(*ss));
-		LIST_INIT(&l->wait_queue);
+		MT_LIST_INIT(&l->wait_queue);
 		l->state = LI_INIT;
 
 		proto->add(l, port);
@@ -766,9 +762,7 @@ void listener_accept(int fd)
 					 */
 					next_actconn = 0;
 					limit_listener(l, &global_listener_queue);
-					HA_RWLOCK_RDLOCK(LISTENER_LOCK, &global_listener_rwlock);
 					task_schedule(global_listener_queue_task, tick_add(now_ms, 1000)); /* try again in 1 second */
-					HA_RWLOCK_RDUNLOCK(LISTENER_LOCK, &global_listener_rwlock);
 					goto end;
 				}
 				next_actconn = count + 1;
@@ -799,15 +793,16 @@ void listener_accept(int fd)
 		if (unlikely(cfd == -1)) {
 			switch (errno) {
 			case EAGAIN:
-				if (fdtab[fd].ev & (FD_POLL_HUP|FD_POLL_ERR)) {
+				if (fdtab[fd].ev & FD_POLL_HUP) {
 					/* the listening socket might have been disabled in a shared
 					 * process and we're a collateral victim. We'll just pause for
 					 * a while in case it comes back. In the mean time, we need to
 					 * clear this sticky flag.
 					 */
-					_HA_ATOMIC_AND(&fdtab[fd].ev, ~(FD_POLL_HUP|FD_POLL_ERR));
+					fdtab[fd].ev &= ~FD_POLL_HUP;
 					goto transient_error;
 				}
+				fd_cant_recv(fd);
 				goto end; /* nothing more to accept */
 			case EINVAL:
 				/* might be trying to accept on a shut fd (eg: soft stop) */
@@ -841,8 +836,7 @@ void listener_accept(int fd)
 				goto transient_error;
 			default:
 				/* unexpected result, let's give up and let other tasks run */
-				max_accept = 0;
-				goto end;
+				goto stop;
 			}
 		}
 
@@ -872,9 +866,7 @@ void listener_accept(int fd)
 				 p->id);
 			close(cfd);
 			limit_listener(l, &global_listener_queue);
-			HA_RWLOCK_RDLOCK(LISTENER_LOCK, &global_listener_rwlock);
 			task_schedule(global_listener_queue_task, tick_add(now_ms, 1000)); /* try again in 1 second */
-			HA_RWLOCK_RDUNLOCK(LISTENER_LOCK, &global_listener_rwlock);
 			goto end;
 		}
 
@@ -889,7 +881,7 @@ void listener_accept(int fd)
 
 #if defined(USE_THREAD)
 		mask = thread_mask(l->bind_conf->bind_thread) & all_threads_mask;
-		if (atleast2(mask) && (global.tune.options & GTUNE_LISTENER_MQ) && !stopping) {
+		if (atleast2(mask) && (global.tune.options & GTUNE_LISTENER_MQ)) {
 			struct accept_queue_ring *ring;
 			unsigned int t, t0, t1, t2;
 
@@ -991,7 +983,7 @@ void listener_accept(int fd)
 			ring = &accept_queue_rings[t];
 			if (accept_queue_push_mp(ring, cfd, l, &addr, laddr)) {
 				_HA_ATOMIC_ADD(&activity[t].accq_pushed, 1);
-				task_wakeup(ring->task, TASK_WOKEN_IO);
+				tasklet_wakeup(ring->tasklet);
 				continue;
 			}
 			/* If the ring is full we do a synchronous accept on
@@ -1029,22 +1021,20 @@ void listener_accept(int fd)
 		}
 #endif
 
-		ti->flags &= ~TI_FL_STUCK; // this thread is still running
 	} /* end of for (max_accept--) */
 
 	/* we've exhausted max_accept, so there is no need to poll again */
+ stop:
+	fd_done_recv(fd);
 	goto end;
 
  transient_error:
-	/* pause the listener for up to 100 ms */
+	/* pause the listener and try again in 100 ms */
 	expire = tick_add(now_ms, 100);
 
  wait_expire:
-	/* switch the listener to LI_LIMITED and wait until up to <expire> in the global queue */
 	limit_listener(l, &global_listener_queue);
-	HA_RWLOCK_RDLOCK(LISTENER_LOCK, &global_listener_rwlock);
 	task_schedule(global_listener_queue_task, tick_first(expire, global_listener_queue_task->expire));
-	HA_RWLOCK_RDUNLOCK(LISTENER_LOCK, &global_listener_rwlock);
  end:
 	if (next_conn)
 		_HA_ATOMIC_SUB(&l->nbconn, 1);
@@ -1056,40 +1046,18 @@ void listener_accept(int fd)
 		_HA_ATOMIC_SUB(&actconn, 1);
 
 	if ((l->state == LI_FULL && (!l->maxconn || l->nbconn < l->maxconn)) ||
-	    (l->state == LI_LIMITED &&
-	     ((!p || p->feconn < p->maxconn) && (actconn < global.maxconn) &&
-	      (!tick_isset(global_listener_queue_task->expire) ||
-	       tick_is_expired(global_listener_queue_task->expire, now_ms))))) {
+	    (l->state == LI_LIMITED && ((!p || p->feconn < p->maxconn) && (actconn < global.maxconn)))) {
 		/* at least one thread has to this when quitting */
 		resume_listener(l);
 
 		/* Dequeues all of the listeners waiting for a resource */
-		if (!LIST_ISEMPTY(&global_listener_queue))
+		if (!MT_LIST_ISEMPTY(&global_listener_queue))
 			dequeue_all_listeners(&global_listener_queue);
 
-		if (p && !LIST_ISEMPTY(&p->listener_queue) &&
+		if (p && !MT_LIST_ISEMPTY(&p->listener_queue) &&
 		    (!p->fe_sps_lim || freq_ctr_remain(&p->fe_sess_per_sec, p->fe_sps_lim, 0) > 0))
 			dequeue_all_listeners(&p->listener_queue);
 	}
-
-	/* Now it's getting tricky. The listener was supposed to be in LI_READY
-	 * state but in the mean time we might have changed it to LI_FULL or
-	 * LI_LIMITED, and another thread might also have turned it to
-	 * LI_PAUSED, LI_LISTEN or even LI_INI when stopping a proxy. We must
-	 * be certain to keep the FD enabled when in the READY state but we
-	 * must also stop it for other states that we might have switched to
-	 * while others re-enabled polling.
-	 */
-	HA_SPIN_LOCK(LISTENER_LOCK, &l->lock);
-	if (l->state == LI_READY) {
-		if (max_accept > 0)
-			fd_cant_recv(fd);
-		else
-			fd_done_recv(fd);
-	} else if (l->state > LI_ASSIGNED) {
-		fd_stop_recv(l->fd);
-	}
-	HA_SPIN_UNLOCK(LISTENER_LOCK, &l->lock);
 }
 
 /* Notify the listener that a connection initiated from it was released. This
@@ -1111,10 +1079,10 @@ void listener_release(struct listener *l)
 		resume_listener(l);
 
 	/* Dequeues all of the listeners waiting for a resource */
-	if (!LIST_ISEMPTY(&global_listener_queue))
+	if (!MT_LIST_ISEMPTY(&global_listener_queue))
 		dequeue_all_listeners(&global_listener_queue);
 
-	if (!LIST_ISEMPTY(&fe->listener_queue) &&
+	if (!MT_LIST_ISEMPTY(&fe->listener_queue) &&
 	    (!fe->fe_sps_lim || freq_ctr_remain(&fe->fe_sess_per_sec, fe->fe_sps_lim, 0) > 0))
 		dequeue_all_listeners(&fe->listener_queue);
 }
@@ -1125,7 +1093,7 @@ static struct task *listener_queue_process(struct task *t, void *context, unsign
 	struct work_list *wl = context;
 	struct listener *l;
 
-	while ((l = LIST_POP_LOCKED(&wl->head, struct listener *, wait_queue))) {
+	while ((l = MT_LIST_POP(&wl->head, struct listener *, wait_queue))) {
 		/* The listeners are still in the LI_LIMITED state */
 		resume_listener(l);
 	}
@@ -1237,18 +1205,6 @@ smp_fetch_so_id(const struct arg *args, struct sample *smp, const char *kw, void
 	smp->data.u.sint = smp->sess->listener->luid;
 	return 1;
 }
-static int
-smp_fetch_so_name(const struct arg *args, struct sample *smp, const char *kw, void *private)
-{
-	smp->data.u.str.area = smp->sess->listener->name;
-	if (!smp->data.u.str.area)
-		return 0;
-
-	smp->data.type = SMP_T_STR;
-	smp->flags = SMP_F_CONST;
-	smp->data.u.str.data = strlen(smp->data.u.str.area);
-	return 1;
-}
 
 /* parse the "accept-proxy" bind keyword */
 static int bind_parse_accept_proxy(char **args, int cur_arg, struct proxy *px, struct bind_conf *conf, char **err)
@@ -1485,7 +1441,6 @@ static int cfg_parse_tune_listener_mq(char **args, int section_type, struct prox
 static struct sample_fetch_kw_list smp_kws = {ILH, {
 	{ "dst_conn", smp_fetch_dconn, 0, NULL, SMP_T_SINT, SMP_USE_FTEND, },
 	{ "so_id",    smp_fetch_so_id, 0, NULL, SMP_T_SINT, SMP_USE_FTEND, },
-	{ "so_name",  smp_fetch_so_name, 0, NULL, SMP_T_STR, SMP_USE_FTEND, },
 	{ /* END */ },
 }};
 
