@@ -36,9 +36,12 @@
 
 #include <proto/applet.h>
 #include <proto/cli.h>
+#include <proto/fd.h>
 #include <proto/frontend.h>
 #include <proto/log.h>
+#include <proto/ring.h>
 #include <proto/sample.h>
+#include <proto/sink.h>
 #include <proto/ssl_sock.h>
 #include <proto/stream.h>
 #include <proto/stream_interface.h>
@@ -245,7 +248,7 @@ THREAD_LOCAL char *logline_rfc5424 = NULL;
 
 /* A global buffer used to store all startup alerts/warnings. It will then be
  * retrieve on the CLI. */
-static char *startup_logs = NULL;
+static struct ring *startup_logs = NULL;
 
 struct logformat_var_args {
 	char *name;
@@ -517,7 +520,7 @@ int add_sample_to_logformat_list(char *text, char *arg, int arg_len, struct prox
 		goto error_free;
 	}
 
-	/* check if we need to allocate an hdr_idx struct for HTTP parsing */
+	/* check if we need to allocate an http_txn struct for HTTP parsing */
 	/* Note, we may also need to set curpx->to_log with certain fetches */
 	curpx->http_needed |= !!(expr->fetch->use & SMP_USE_HTTP_ANY);
 
@@ -615,7 +618,7 @@ int parse_logformat_string(const char *fmt, struct proxy *curproxy, struct list
 				sp = str - 1; /* send both the '%' and the current char */
 				memprintf(err, "unexpected variable name near '%c' at position %d line : '%s'. Maybe you want to write a single '%%', use the syntax '%%%%'",
 				          *str, (int)(str - backfmt), fmt);
-				goto fail;
+				return 0;
 
 			}
 			else
@@ -642,7 +645,7 @@ int parse_logformat_string(const char *fmt, struct proxy *curproxy, struct list
 				break;
 			}
 			memprintf(err, "parse argument modifier without variable name near '%%{%s}'", arg);
-			goto fail;
+			return 0;
 
 		case LF_STEXPR:                        // text immediately following '%['
 			if (*str == ']') {             // end of arg
@@ -675,16 +678,16 @@ int parse_logformat_string(const char *fmt, struct proxy *curproxy, struct list
 			switch (pformat) {
 			case LF_VAR:
 				if (!parse_logformat_var(arg, arg_len, var, var_len, curproxy, list_format, &options, err))
-					goto fail;
+					return 0;
 				break;
 			case LF_STEXPR:
 				if (!add_sample_to_logformat_list(var, arg, arg_len, curproxy, list_format, options, cap, err))
-					goto fail;
+					return 0;
 				break;
 			case LF_TEXT:
 			case LF_SEPARATOR:
 				if (!add_to_logformat_list(sp, str, pformat, list_format, err))
-					goto fail;
+					return 0;
 				break;
 			}
 			sp = str; /* new start of text at every state switch and at every separator */
@@ -693,14 +696,11 @@ int parse_logformat_string(const char *fmt, struct proxy *curproxy, struct list
 
 	if (pformat == LF_STARTVAR || pformat == LF_STARG || pformat == LF_STEXPR) {
 		memprintf(err, "truncated line after '%s'", var ? var : arg ? arg : "%");
-		goto fail;
+		return 0;
 	}
 	free(backfmt);
 
 	return 1;
- fail:
-	free(backfmt);
-	return 0;
 }
 
 /*
@@ -791,7 +791,6 @@ int smp_log_range_cmp(const void *a, const void *b)
  */
 int parse_logsrv(char **args, struct list *logsrvs, int do_del, char **err)
 {
-	struct smp_log_range *smp_rgs = NULL;
 	struct sockaddr_storage *sk;
 	struct logsrv *logsrv = NULL;
 	int port1, port2;
@@ -834,10 +833,6 @@ int parse_logsrv(char **args, struct list *logsrvs, int do_del, char **err)
 			}
 
 			node = malloc(sizeof(*node));
-			if (!node) {
-				memprintf(err, "out of memory error");
-				goto error;
-			}
 			memcpy(node, logsrv, sizeof(struct logsrv));
 			node->ref = logsrv;
 			LIST_INIT(&node->list);
@@ -901,6 +896,7 @@ int parse_logsrv(char **args, struct list *logsrvs, int do_del, char **err)
 	if (strcmp(args[cur_arg], "sample") == 0) {
 		unsigned low, high;
 		char *p, *beg, *end, *smp_sz_str;
+		struct smp_log_range *smp_rgs = NULL;
 		size_t smp_rgs_sz = 0, smp_sz = 0, new_smp_sz;
 
 		p = args[cur_arg+1];
@@ -1005,6 +1001,24 @@ int parse_logsrv(char **args, struct list *logsrvs, int do_del, char **err)
 	}
 
 	/* now, back to the address */
+	logsrv->type = LOG_TARGET_DGRAM;
+	if (strncmp(args[1], "ring@", 5) == 0) {
+		struct sink *sink = sink_find(args[1] + 5);
+
+		if (!sink || sink->type != SINK_TYPE_BUFFER) {
+			memprintf(err, "cannot find ring buffer '%s'", args[1] + 5);
+			goto error;
+		}
+
+		logsrv->addr.ss_family = AF_UNSPEC;
+		logsrv->type = LOG_TARGET_BUFFER;
+		logsrv->ring = sink->ctx.ring;
+		goto done;
+	}
+
+	if (strncmp(args[1], "fd@", 3) == 0)
+		logsrv->type = LOG_TARGET_FD;
+
 	sk = str2sa_range(args[1], NULL, &port1, &port2, err, NULL, NULL, 1);
 	if (!sk)
 		goto error;
@@ -1019,11 +1033,11 @@ int parse_logsrv(char **args, struct list *logsrvs, int do_del, char **err)
 		if (!port1)
 			set_host_port(&logsrv->addr, SYSLOG_PORT);
 	}
+ done:
 	LIST_ADDQ(logsrvs, &logsrv->list);
 	return 1;
 
   error:
-	free(smp_rgs);
 	free(logsrv);
 	return 0;
 }
@@ -1042,9 +1056,21 @@ static void print_message(const char *label, const char *fmt, va_list argp)
 		  label, tm.tm_yday, tm.tm_hour, tm.tm_min, tm.tm_sec, (int)getpid());
 	memvprintf(&msg, fmt, argp);
 
-	if (global.mode & MODE_STARTING &&
-	    (!startup_logs || strlen(startup_logs) + strlen(head) + strlen(msg) < global.tune.bufsize))
-		memprintf(&startup_logs, "%s%s%s", (startup_logs ? startup_logs : ""), head, msg);
+	if (global.mode & MODE_STARTING) {
+		if (unlikely(!startup_logs))
+			startup_logs = ring_new(STARTUP_LOG_SIZE);
+
+		if (likely(startup_logs)) {
+			struct ist m[2];
+
+			m[0] = ist(head);
+			m[1] = ist(msg);
+			/* trim the trailing '\n' */
+			if (m[1].len > 0 && m[1].ptr[m[1].len - 1] == '\n')
+				m[1].len--;
+			ring_write(startup_logs, ~0, 0, 0, m, 2);
+		}
+	}
 
 	fprintf(stderr, "%s%s", head, msg);
 	fflush(stderr);
@@ -1266,21 +1292,17 @@ char *lf_text_len(char *dst, const char *src, size_t len, size_t size, const str
 	}
 
 	if (src && len) {
-		/* escape_string and strlcpy2 will both try to add terminating NULL-byte
-		 * to dst, so we need to make sure that extra byte will fit into dst
-		 * before calling them
-		 */
+		if (++len > size)
+			len = size;
 		if (node->options & LOG_OPT_ESC) {
 			char *ret;
 
-			ret = escape_string(dst, (dst + size - 1), '\\', rfc5424_escape_map, src, src + len);
+			ret = escape_string(dst, dst + len, '\\', rfc5424_escape_map, src);
 			if (ret == NULL || *ret != '\0')
 				return NULL;
 			len = ret - dst;
 		}
 		else {
-			if (++len > size)
-				len = size;
 			len = strlcpy2(dst, src, len);
 		}
 
@@ -1291,7 +1313,6 @@ char *lf_text_len(char *dst, const char *src, size_t len, size_t size, const str
 		if (size < 2)
 			return NULL;
 		*(dst++) = '-';
-		size -= 1;
 	}
 
 	if (node->options & LOG_OPT_QUOTE) {
@@ -1471,7 +1492,8 @@ void send_log(struct proxy *p, int level, const char *format, ...)
 		data_len = global.max_syslog_len;
 	va_end(argp);
 
-	__send_log(p, level, logline, data_len, default_rfc5424_sd_log_format, 2);
+	__send_log((p ? &p->logsrvs : NULL), (p ? &p->log_tag : NULL), level,
+		   logline, data_len, default_rfc5424_sd_log_format, 2);
 }
 
 /*
@@ -1515,30 +1537,19 @@ static inline void __do_send_log(struct logsrv *logsrv, int nblogger, char *pid_
 
 	dataptr = message;
 
-	/* historically some messages used to already contain the trailing LF
-	 * or Zero. Let's remove all trailing LF or Zero
-	 */
-	while (size && ((dataptr[size-1] == '\n' || (dataptr[size-1] == 0))))
-		size--;
-
-	if (logsrv->addr.ss_family == AF_UNSPEC) {
+	if (logsrv->type == LOG_TARGET_FD) {
 		/* the socket's address is a file descriptor */
 		plogfd = (int *)&((struct sockaddr_in *)&logsrv->addr)->sin_addr.s_addr;
-		if (unlikely(!((struct sockaddr_in *)&logsrv->addr)->sin_port)) {
-			/* FD not yet initialized to non-blocking mode.
-			 * DON'T DO IT ON A TERMINAL!
-			 */
-			if (!isatty(*plogfd))
-				fcntl(*plogfd, F_SETFL, O_NONBLOCK);
-			((struct sockaddr_in *)&logsrv->addr)->sin_port = 1;
-		}
+	}
+	else if (logsrv->type == LOG_TARGET_BUFFER) {
+		plogfd = NULL;
 	}
 	else if (logsrv->addr.ss_family == AF_UNIX)
 		plogfd = &logfdunix;
 	else
 		plogfd = &logfdinet;
 
-	if (unlikely(*plogfd < 0)) {
+	if (plogfd && unlikely(*plogfd < 0)) {
 		/* socket not successfully initialized yet */
 		if ((*plogfd = socket(logsrv->addr.ss_family, SOCK_DGRAM,
 							  (logsrv->addr.ss_family == AF_UNIX) ? 0 : IPPROTO_UDP)) < 0) {
@@ -1580,7 +1591,7 @@ static inline void __do_send_log(struct logsrv *logsrv, int nblogger, char *pid_
 		hdr_ptr = hdr;
 		hdr_max = 3;
 		maxlen = logsrv->maxlen - hdr_max;
-		max = MIN(size, maxlen - 1);
+		max = MIN(size, maxlen) - 1;
 		goto send;
 
 	case LOG_FORMAT_RAW:
@@ -1588,7 +1599,7 @@ static inline void __do_send_log(struct logsrv *logsrv, int nblogger, char *pid_
 		hdr_ptr = hdr = "";
 		hdr_max = 0;
 		maxlen = logsrv->maxlen;
-		max = MIN(size, maxlen - 1);
+		max = MIN(size, maxlen) - 1;
 		goto send;
 
 	default:
@@ -1672,61 +1683,49 @@ static inline void __do_send_log(struct logsrv *logsrv, int nblogger, char *pid_
 		goto send;
 	}
 
-	max = MIN(size, maxlen - sd_max - 1);
+	max = MIN(size, maxlen - sd_max) - 1;
 send:
-	iovec[0].iov_base = hdr_ptr;
-	iovec[0].iov_len  = hdr_max;
-	iovec[1].iov_base = tag_str;
-	iovec[1].iov_len  = tag_max;
-	iovec[2].iov_base = pid_sep1;
-	iovec[2].iov_len  = pid_sep1_max;
-	iovec[3].iov_base = pid_str;
-	iovec[3].iov_len  = pid_max;
-	iovec[4].iov_base = pid_sep2;
-	iovec[4].iov_len  = pid_sep2_max;
-	iovec[5].iov_base = sd;
-	iovec[5].iov_len  = sd_max;
-	iovec[6].iov_base = dataptr;
-	iovec[6].iov_len  = max;
-	iovec[7].iov_base = "\n"; /* insert a \n at the end of the message */
-	iovec[7].iov_len  = 1;
-
 	if (logsrv->addr.ss_family == AF_UNSPEC) {
-		int attempts = 0;
-		/* make sure we never interleave writes and we never block. This means
-		 * we prefer to fail on collision than to block. But we don't want to
-		 * lose too many logs so we just perform a few lock attempts then give
-		 * up.
-		 */
+		/* the target is a file descriptor or a ring buffer */
+		struct ist msg[7];
 
-		while (HA_SPIN_TRYLOCK(LOGSRV_LOCK, &logsrv->lock) != 0) {
-			if (++attempts >= 200) {
-				/* so that the caller knows the message couldn't be delivered */
-				sent = -1;
-				errno = EAGAIN;
-				goto leave;
-			}
-			ha_thread_relax();
-		}
+		msg[0].ptr = hdr_ptr;  msg[0].len = hdr_max;
+		msg[1].ptr = tag_str;  msg[1].len = tag_max;
+		msg[2].ptr = pid_sep1; msg[2].len = pid_sep1_max;
+		msg[3].ptr = pid_str;  msg[3].len = pid_max;
+		msg[4].ptr = pid_sep2; msg[4].len = pid_sep2_max;
+		msg[5].ptr = sd;       msg[5].len = sd_max;
+		msg[6].ptr = dataptr;  msg[6].len = max;
 
-		/* the target is a direct file descriptor. While writev() guarantees
-		 * to write everything, it doesn't guarantee that it will not be
-		 * interrupted while doing so. This occasionally results in interleaved
-		 * messages when the output is a tty, hence the lock. There's no real
-		 * performance concern here for such type of output.
-		 */
-
-		sent = writev(*plogfd, iovec, 8);
-		HA_SPIN_UNLOCK(LOGSRV_LOCK, &logsrv->lock);
+		if (logsrv->type == LOG_TARGET_BUFFER)
+			sent = ring_write(logsrv->ring, ~0, NULL, 0, msg, 7);
+		else /* LOG_TARGET_FD */
+			sent = fd_write_frag_line(*plogfd, ~0, NULL, 0, msg, 7, 1);
 	}
 	else {
+		iovec[0].iov_base = hdr_ptr;
+		iovec[0].iov_len  = hdr_max;
+		iovec[1].iov_base = tag_str;
+		iovec[1].iov_len  = tag_max;
+		iovec[2].iov_base = pid_sep1;
+		iovec[2].iov_len  = pid_sep1_max;
+		iovec[3].iov_base = pid_str;
+		iovec[3].iov_len  = pid_max;
+		iovec[4].iov_base = pid_sep2;
+		iovec[4].iov_len  = pid_sep2_max;
+		iovec[5].iov_base = sd;
+		iovec[5].iov_len  = sd_max;
+		iovec[6].iov_base = dataptr;
+		iovec[6].iov_len  = max;
+		iovec[7].iov_base = "\n"; /* insert a \n at the end of the message */
+		iovec[7].iov_len  = 1;
+
 		msghdr.msg_name = (struct sockaddr *)&logsrv->addr;
 		msghdr.msg_namelen = get_addr_len(&logsrv->addr);
 
 		sent = sendmsg(*plogfd, &msghdr, MSG_DONTWAIT | MSG_NOSIGNAL);
 	}
 
-leave:
 	if (sent < 0) {
 		static char once;
 
@@ -1746,30 +1745,24 @@ leave:
  * The arguments <sd> and <sd_size> are used for the structured-data part
  * in RFC5424 formatted syslog messages.
  */
-void __send_log(struct proxy *p, int level, char *message, size_t size, char *sd, size_t sd_size)
+void __send_log(struct list *logsrvs, struct buffer *tag, int level,
+		char *message, size_t size, char *sd, size_t sd_size)
 {
-	struct list *logsrvs = NULL;
 	struct logsrv *logsrv;
 	int nblogger;
 	static THREAD_LOCAL int curr_pid;
 	static THREAD_LOCAL char pidstr[100];
 	static THREAD_LOCAL struct buffer pid;
-	struct buffer *tag = &global.log_tag;
 
-	if (p == NULL) {
+	if (logsrvs == NULL) {
 		if (!LIST_ISEMPTY(&global.logsrvs)) {
 			logsrvs = &global.logsrvs;
 		}
-	} else {
-		if (!LIST_ISEMPTY(&p->logsrvs)) {
-			logsrvs = &p->logsrvs;
-		}
-		if (p->log_tag.area) {
-			tag = &p->log_tag;
-		}
 	}
+	if (!tag || !tag->area)
+		tag = &global.log_tag;
 
-	if (!logsrvs)
+	if (!logsrvs || LIST_ISEMPTY(logsrvs))
 		return;
 
 	if (unlikely(curr_pid != getpid())) {
@@ -1781,7 +1774,7 @@ void __send_log(struct proxy *p, int level, char *message, size_t size, char *sd
 	/* Send log messages to syslog server. */
 	nblogger = 0;
 	list_for_each_entry(logsrv, logsrvs, list) {
-		int in_range = 1;
+		static THREAD_LOCAL int in_range = 1;
 
 		/* we can filter the level of the messages that are sent to each logger */
 		if (level > logsrv->level)
@@ -1921,20 +1914,15 @@ int init_log_buffers()
 /* Deinitialize log buffers used for syslog messages */
 void deinit_log_buffers()
 {
-	void *tmp_startup_logs;
-
 	free(logheader);
 	free(logheader_rfc5424);
 	free(logline);
 	free(logline_rfc5424);
-	tmp_startup_logs = _HA_ATOMIC_XCHG(&startup_logs, NULL);
-	free(tmp_startup_logs);
-
+	ring_free(_HA_ATOMIC_XCHG(&startup_logs, NULL));
 	logheader         = NULL;
 	logheader_rfc5424 = NULL;
 	logline           = NULL;
 	logline_rfc5424   = NULL;
-	startup_logs      = NULL;
 }
 
 /* Builds a log line in <dst> based on <list_format>, and stops before reaching
@@ -1949,7 +1937,7 @@ int sess_build_logline(struct session *sess, struct stream *s, char *dst, size_t
 	struct proxy *be;
 	struct http_txn *txn;
 	const struct strm_logs *logs;
-	const struct connection *be_conn;
+	struct connection *be_conn;
 	unsigned int s_flags;
 	unsigned int uniq_id;
 	struct buffer chunk;
@@ -2066,8 +2054,8 @@ int sess_build_logline(struct session *sess, struct stream *s, char *dst, size_t
 
 			case LOG_FMT_CLIENTIP:  // %ci
 				conn = objt_conn(sess->origin);
-				if (conn)
-					ret = lf_ip(tmplog, (struct sockaddr *)&conn->addr.from, dst + maxsize - tmplog, tmp);
+				if (conn && conn_get_src(conn))
+					ret = lf_ip(tmplog, (struct sockaddr *)conn->src, dst + maxsize - tmplog, tmp);
 				else
 					ret = lf_text_len(tmplog, NULL, 0, dst + maxsize - tmplog, tmp);
 				if (ret == NULL)
@@ -2078,11 +2066,11 @@ int sess_build_logline(struct session *sess, struct stream *s, char *dst, size_t
 
 			case LOG_FMT_CLIENTPORT:  // %cp
 				conn = objt_conn(sess->origin);
-				if (conn) {
-					if (conn->addr.from.ss_family == AF_UNIX) {
+				if (conn && conn_get_src(conn)) {
+					if (conn->src->ss_family == AF_UNIX) {
 						ret = ltoa_o(sess->listener->luid, tmplog, dst + maxsize - tmplog);
 					} else {
-						ret = lf_port(tmplog, (struct sockaddr *)&conn->addr.from,
+						ret = lf_port(tmplog, (struct sockaddr *)conn->src,
 						              dst + maxsize - tmplog, tmp);
 					}
 				}
@@ -2097,9 +2085,8 @@ int sess_build_logline(struct session *sess, struct stream *s, char *dst, size_t
 
 			case LOG_FMT_FRONTENDIP: // %fi
 				conn = objt_conn(sess->origin);
-				if (conn) {
-					conn_get_to_addr(conn);
-					ret = lf_ip(tmplog, (struct sockaddr *)&conn->addr.to, dst + maxsize - tmplog, tmp);
+				if (conn && conn_get_dst(conn)) {
+					ret = lf_ip(tmplog, (struct sockaddr *)conn->dst, dst + maxsize - tmplog, tmp);
 				}
 				else
 					ret = lf_text_len(tmplog, NULL, 0, dst + maxsize - tmplog, tmp);
@@ -2112,12 +2099,11 @@ int sess_build_logline(struct session *sess, struct stream *s, char *dst, size_t
 
 			case  LOG_FMT_FRONTENDPORT: // %fp
 				conn = objt_conn(sess->origin);
-				if (conn) {
-					conn_get_to_addr(conn);
-					if (conn->addr.to.ss_family == AF_UNIX)
+				if (conn && conn_get_dst(conn)) {
+					if (conn->dst->ss_family == AF_UNIX)
 						ret = ltoa_o(sess->listener->luid, tmplog, dst + maxsize - tmplog);
 					else
-						ret = lf_port(tmplog, (struct sockaddr *)&conn->addr.to, dst + maxsize - tmplog, tmp);
+						ret = lf_port(tmplog, (struct sockaddr *)conn->dst, dst + maxsize - tmplog, tmp);
 				}
 				else
 					ret = lf_text_len(tmplog, NULL, 0, dst + maxsize - tmplog, tmp);
@@ -2129,8 +2115,8 @@ int sess_build_logline(struct session *sess, struct stream *s, char *dst, size_t
 				break;
 
 			case LOG_FMT_BACKENDIP:  // %bi
-				if (be_conn)
-					ret = lf_ip(tmplog, (const struct sockaddr *)&be_conn->addr.from, dst + maxsize - tmplog, tmp);
+				if (be_conn && conn_get_src(be_conn))
+					ret = lf_ip(tmplog, (const struct sockaddr *)be_conn->src, dst + maxsize - tmplog, tmp);
 				else
 					ret = lf_text_len(tmplog, NULL, 0, dst + maxsize - tmplog, tmp);
 
@@ -2141,8 +2127,8 @@ int sess_build_logline(struct session *sess, struct stream *s, char *dst, size_t
 				break;
 
 			case LOG_FMT_BACKENDPORT:  // %bp
-				if (be_conn)
-					ret = lf_port(tmplog, (struct sockaddr *)&be_conn->addr.from, dst + maxsize - tmplog, tmp);
+				if (be_conn && conn_get_src(be_conn))
+					ret = lf_port(tmplog, (struct sockaddr *)be_conn->src, dst + maxsize - tmplog, tmp);
 				else
 					ret = lf_text_len(tmplog, NULL, 0, dst + maxsize - tmplog, tmp);
 
@@ -2153,8 +2139,8 @@ int sess_build_logline(struct session *sess, struct stream *s, char *dst, size_t
 				break;
 
 			case LOG_FMT_SERVERIP: // %si
-				if (be_conn)
-					ret = lf_ip(tmplog, (struct sockaddr *)&be_conn->addr.to, dst + maxsize - tmplog, tmp);
+				if (be_conn && conn_get_dst(be_conn))
+					ret = lf_ip(tmplog, (struct sockaddr *)be_conn->dst, dst + maxsize - tmplog, tmp);
 				else
 					ret = lf_text_len(tmplog, NULL, 0, dst + maxsize - tmplog, tmp);
 
@@ -2165,8 +2151,8 @@ int sess_build_logline(struct session *sess, struct stream *s, char *dst, size_t
 				break;
 
 			case LOG_FMT_SERVERPORT: // %sp
-				if (be_conn)
-					ret = lf_port(tmplog, (struct sockaddr *)&be_conn->addr.to, dst + maxsize - tmplog, tmp);
+				if (be_conn && conn_get_dst(be_conn))
+					ret = lf_port(tmplog, (struct sockaddr *)be_conn->dst, dst + maxsize - tmplog, tmp);
 				else
 					ret = lf_text_len(tmplog, NULL, 0, dst + maxsize - tmplog, tmp);
 
@@ -2537,10 +2523,9 @@ int sess_build_logline(struct session *sess, struct stream *s, char *dst, size_t
 			case LOG_FMT_RETRIES:  // %rq
 				if (s_flags & SF_REDISP)
 					LOGCHAR('+');
-				ret = ltoa_o(((s && s->si[1].conn_retries > 0)
-					      ? (be->conn_retries - s->si[1].conn_retries)
-					      : ((s && s->si[1].state != SI_ST_INI) ? be->conn_retries : 0)),
-					     tmplog, dst + maxsize - tmplog);
+				ret = ltoa_o((s && s->si[1].conn_retries > 0) ?
+				                (be->conn_retries - s->si[1].conn_retries) :
+				                be->conn_retries, tmplog, dst + maxsize - tmplog);
 				if (ret == NULL)
 					goto out;
 				tmplog = ret;
@@ -2989,7 +2974,8 @@ void strm_log(struct stream *s)
 	size = build_logline(s, logline, global.max_syslog_len, &sess->fe->logformat);
 	if (size > 0) {
 		_HA_ATOMIC_ADD(&sess->fe->log_count, 1);
-		__send_log(sess->fe, level, logline, size + 1, logline_rfc5424, sd_size);
+		__send_log(&sess->fe->logsrvs, &sess->fe->log_tag, level,
+			   logline, size + 1, logline_rfc5424, sd_size);
 		s->logs.logwait = 0;
 	}
 }
@@ -3027,27 +3013,44 @@ void sess_log(struct session *sess)
 	size = sess_build_logline(sess, NULL, logline, global.max_syslog_len, &sess->fe->logformat);
 	if (size > 0) {
 		_HA_ATOMIC_ADD(&sess->fe->log_count, 1);
-		__send_log(sess->fe, level, logline, size + 1, logline_rfc5424, sd_size);
+		__send_log(&sess->fe->logsrvs, &sess->fe->log_tag, level,
+			   logline, size + 1, logline_rfc5424, sd_size);
 	}
 }
 
-static int cli_io_handler_show_startup_logs(struct appctx *appctx)
+void app_log(struct list *logsrvs, struct buffer *tag, int level, const char *format, ...)
 {
-	struct stream_interface *si = appctx->owner;
-	const char *msg = (startup_logs ? startup_logs : "No startup alerts/warnings.\n");
+	va_list argp;
+	int  data_len;
 
-	if (ci_putstr(si_ic(si), msg) == -1) {
-		si_rx_room_blk(si);
-		return 0;
-	}
-	return 1;
+	if (level < 0 || format == NULL || logline == NULL)
+		return;
+
+	va_start(argp, format);
+	data_len = vsnprintf(logline, global.max_syslog_len, format, argp);
+	if (data_len < 0 || data_len > global.max_syslog_len)
+		data_len = global.max_syslog_len;
+	va_end(argp);
+
+	__send_log(logsrvs, tag, level, logline, data_len, default_rfc5424_sd_log_format, 2);
+}
+
+/* parse the "show startup-logs" command, returns 1 if a message is returned, otherwise zero */
+static int cli_parse_show_startup_logs(char **args, char *payload, struct appctx *appctx, void *private)
+{
+	if (!cli_has_level(appctx, ACCESS_LVL_OPER))
+		return 1;
+
+	if (!startup_logs)
+		return cli_msg(appctx, LOG_INFO, "\n"); // nothing to print
+
+	return ring_attach_cli(startup_logs, appctx);
 }
 
 /* register cli keywords */
 static struct cli_kw_list cli_kws = {{ },{
 	{ { "show", "startup-logs",  NULL },
-	  "show startup-logs : report logs emitted during HAProxy startup",
-	  NULL, cli_io_handler_show_startup_logs },
+	  "show startup-logs : report logs emitted during HAProxy startup", cli_parse_show_startup_logs, NULL, NULL },
 	{{},}
 }};
 
