@@ -105,11 +105,6 @@ int sample_load_map(struct arg *arg, struct sample_conv *conv,
 {
 	struct map_descriptor *desc;
 
-	if (!(global.mode & MODE_STARTING)) {
-		memprintf(err, "map: cannot load map at runtime");
-		return 0;
-	}
-
 	/* create new map descriptor */
 	desc = map_create_descriptor(conv);
 	if (!desc) {
@@ -335,12 +330,10 @@ static int cli_io_handler_pat_list(struct appctx *appctx)
 		 * reference to the last ref_elt being dumped.
 		 */
 		if (appctx->st2 == STAT_ST_LIST) {
-			HA_SPIN_LOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
 			if (!LIST_ISEMPTY(&appctx->ctx.map.bref.users)) {
 				LIST_DEL(&appctx->ctx.map.bref.users);
 				LIST_INIT(&appctx->ctx.map.bref.users);
 			}
-			HA_SPIN_UNLOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
 		}
 		return 1;
 	}
@@ -348,17 +341,27 @@ static int cli_io_handler_pat_list(struct appctx *appctx)
 	switch (appctx->st2) {
 
 	case STAT_ST_INIT:
+		/* the function had not been called yet, let's prepare the
+		 * buffer for a response. We initialize the current stream
+		 * pointer to the first in the global list. When a target
+		 * stream is being destroyed, it is responsible for updating
+		 * this pointer. We know we have reached the end when this
+		 * pointer points back to the head of the streams list.
+		 */
+		HA_SPIN_LOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
+		LIST_INIT(&appctx->ctx.map.bref.users);
+		appctx->ctx.map.bref.ref = appctx->ctx.map.ref->head.n;
+		HA_SPIN_UNLOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
 		appctx->st2 = STAT_ST_LIST;
 		/* fall through */
 
 	case STAT_ST_LIST:
+
 		HA_SPIN_LOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
 
 		if (!LIST_ISEMPTY(&appctx->ctx.map.bref.users)) {
 			LIST_DEL(&appctx->ctx.map.bref.users);
 			LIST_INIT(&appctx->ctx.map.bref.users);
-		} else {
-			appctx->ctx.map.bref.ref = appctx->ctx.map.ref->head.n;
 		}
 
 		while (appctx->ctx.map.bref.ref != &appctx->ctx.map.ref->head) {
@@ -596,31 +599,19 @@ static int cli_parse_get_map(char **args, char *payload, struct appctx *appctx,
 
 		/* No parameter. */
 		if (!*args[2] || !*args[3]) {
-			if (appctx->ctx.map.display_flags == PAT_REF_MAP) {
-				appctx->ctx.cli.severity = LOG_ERR;
-				appctx->ctx.cli.msg = "Missing map identifier and/or key.\n";
-			}
-			else {
-				appctx->ctx.cli.severity = LOG_ERR;
-				appctx->ctx.cli.msg = "Missing ACL identifier and/or key.\n";
-			}
-			appctx->st0 = CLI_ST_PRINT;
-			return 1;
+			if (appctx->ctx.map.display_flags == PAT_REF_MAP)
+				return cli_err(appctx, "Missing map identifier and/or key.\n");
+			else
+				return cli_err(appctx, "Missing ACL identifier and/or key.\n");
 		}
 
 		/* lookup into the maps */
 		appctx->ctx.map.ref = pat_ref_lookup_ref(args[2]);
 		if (!appctx->ctx.map.ref) {
-			if (appctx->ctx.map.display_flags == PAT_REF_MAP) {
-				appctx->ctx.cli.severity = LOG_ERR;
-				appctx->ctx.cli.msg = "Unknown map identifier. Please use #<id> or <file>.\n";
-			}
-			else {
-				appctx->ctx.cli.severity = LOG_ERR;
-				appctx->ctx.cli.msg = "Unknown ACL identifier. Please use #<id> or <file>.\n";
-			}
-			appctx->st0 = CLI_ST_PRINT;
-			return 1;
+			if (appctx->ctx.map.display_flags == PAT_REF_MAP)
+				return cli_err(appctx, "Unknown map identifier. Please use #<id> or <file>.\n");
+			else
+				return cli_err(appctx, "Unknown ACL identifier. Please use #<id> or <file>.\n");
 		}
 
 		/* copy input string. The string must be allocated because
@@ -630,12 +621,8 @@ static int cli_parse_get_map(char **args, char *payload, struct appctx *appctx,
 		appctx->ctx.map.chunk.data = strlen(args[3]);
 		appctx->ctx.map.chunk.size = appctx->ctx.map.chunk.data + 1;
 		appctx->ctx.map.chunk.area = strdup(args[3]);
-		if (!appctx->ctx.map.chunk.area) {
-			appctx->ctx.cli.severity = LOG_ERR;
-			appctx->ctx.cli.msg = "Out of memory error.\n";
-			appctx->st0 = CLI_ST_PRINT;
-			return 1;
-		}
+		if (!appctx->ctx.map.chunk.area)
+			return cli_err(appctx,  "Out of memory error.\n");
 
 		return 0;
 	}
@@ -673,19 +660,11 @@ static int cli_parse_show_map(char **args, char *payload, struct appctx *appctx,
 		appctx->ctx.map.ref = pat_ref_lookup_ref(args[2]);
 		if (!appctx->ctx.map.ref ||
 		    !(appctx->ctx.map.ref->flags & appctx->ctx.map.display_flags)) {
-			if (appctx->ctx.map.display_flags == PAT_REF_MAP) {
-				appctx->ctx.cli.severity = LOG_ERR;
-				appctx->ctx.cli.msg = "Unknown map identifier. Please use #<id> or <file>.\n";
-			}
-			else {
-				appctx->ctx.cli.severity = LOG_ERR;
-				appctx->ctx.cli.msg = "Unknown ACL identifier. Please use #<id> or <file>.\n";
-			}
-			appctx->st0 = CLI_ST_PRINT;
-			return 1;
+			if (appctx->ctx.map.display_flags == PAT_REF_MAP)
+				return cli_err(appctx, "Unknown map identifier. Please use #<id> or <file>.\n");
+			else
+				return cli_err(appctx, "Unknown ACL identifier. Please use #<id> or <file>.\n");
 		}
-
-		LIST_INIT(&appctx->ctx.map.bref.users);
 		appctx->io_handler = cli_io_handler_pat_list;
 		appctx->io_release = cli_release_show_map;
 		return 0;
@@ -703,21 +682,13 @@ static int cli_parse_set_map(char **args, char *payload, struct appctx *appctx,
 		appctx->ctx.map.display_flags = PAT_REF_MAP;
 
 		/* Expect three parameters: map name, key and new value. */
-		if (!*args[2] || !*args[3] || !*args[4]) {
-			appctx->ctx.cli.severity = LOG_ERR;
-			appctx->ctx.cli.msg = "'set map' expects three parameters: map identifier, key and value.\n";
-			appctx->st0 = CLI_ST_PRINT;
-			return 1;
-		}
+		if (!*args[2] || !*args[3] || !*args[4])
+			return cli_err(appctx, "'set map' expects three parameters: map identifier, key and value.\n");
 
 		/* Lookup the reference in the maps. */
 		appctx->ctx.map.ref = pat_ref_lookup_ref(args[2]);
-		if (!appctx->ctx.map.ref) {
-			appctx->ctx.cli.severity = LOG_ERR;
-			appctx->ctx.cli.msg = "Unknown map identifier. Please use #<id> or <file>.\n";
-			appctx->st0 = CLI_ST_PRINT;
-			return 1;
-		}
+		if (!appctx->ctx.map.ref)
+			return cli_err(appctx, "Unknown map identifier. Please use #<id> or <file>.\n");
 
 		/* If the entry identifier start with a '#', it is considered as
 		 * pointer id
@@ -729,38 +700,23 @@ static int cli_parse_set_map(char **args, char *payload, struct appctx *appctx,
 
 			/* Convert argument to integer value. */
 			conv = strtoll(&args[3][1], &error, 16);
-			if (*error != '\0') {
-				appctx->ctx.cli.severity = LOG_ERR;
-				appctx->ctx.cli.msg = "Malformed identifier. Please use #<id> or <file>.\n";
-				appctx->st0 = CLI_ST_PRINT;
-				return 1;
-			}
+			if (*error != '\0')
+				return cli_err(appctx, "Malformed identifier. Please use #<id> or <file>.\n");
 
 			/* Convert and check integer to pointer. */
 			ref = (struct pat_ref_elt *)(long)conv;
-			if ((long long int)(long)ref != conv) {
-				appctx->ctx.cli.severity = LOG_ERR;
-				appctx->ctx.cli.msg = "Malformed identifier. Please use #<id> or <file>.\n";
-				appctx->st0 = CLI_ST_PRINT;
-				return 1;
-			}
+			if ((long long int)(long)ref != conv)
+				return cli_err(appctx, "Malformed identifier. Please use #<id> or <file>.\n");
 
 			/* Try to modify the entry. */
 			err = NULL;
 			HA_SPIN_LOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
 			if (!pat_ref_set_by_id(appctx->ctx.map.ref, ref, args[4], &err)) {
 				HA_SPIN_UNLOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
-				if (err) {
-					memprintf(&err, "%s.\n", err);
-					appctx->ctx.cli.err = err;
-					appctx->st0 = CLI_ST_PRINT_FREE;
-				}
-				else {
-					appctx->ctx.cli.severity = LOG_ERR;
-					appctx->ctx.cli.msg = "Failed to update an entry.\n";
-					appctx->st0 = CLI_ST_PRINT;
-				}
-				return 1;
+				if (err)
+					return cli_dynerr(appctx, memprintf(&err, "%s.\n", err));
+				else
+					return cli_err(appctx, "Failed to update an entry.\n");
 			}
 			HA_SPIN_UNLOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
 		}
@@ -772,17 +728,10 @@ static int cli_parse_set_map(char **args, char *payload, struct appctx *appctx,
 			HA_SPIN_LOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
 			if (!pat_ref_set(appctx->ctx.map.ref, args[3], args[4], &err)) {
 				HA_SPIN_UNLOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
-				if (err) {
-					memprintf(&err, "%s.\n", err);
-					appctx->ctx.cli.err = err;
-					appctx->st0 = CLI_ST_PRINT_FREE;
-				}
-				else {
-					appctx->ctx.cli.severity = LOG_ERR;
-					appctx->ctx.cli.msg = "Failed to update an entry.\n";
-					appctx->st0 = CLI_ST_PRINT;
-				}
-				return 1;
+				if (err)
+					return cli_dynerr(appctx, memprintf(&err, "%s.\n", err));
+				else
+					return cli_err(appctx, "Failed to update an entry.\n");
 			}
 			HA_SPIN_UNLOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
 		}
@@ -828,35 +777,21 @@ static int cli_parse_add_map(char **args, char *payload, struct appctx *appctx,
 		 */
 		if (appctx->ctx.map.display_flags == PAT_REF_MAP) {
 			if ((!payload && (!*args[2] || !*args[3] || !*args[4])) ||
-			    (payload && !*args[2])) {
-				appctx->ctx.cli.severity = LOG_ERR;
-				appctx->ctx.cli.msg = "'add map' expects three parameters (map identifier, key and value) or one parameter (map identifier) and a payload\n";
-				appctx->st0 = CLI_ST_PRINT;
-				return 1;
-			}
-		}
-		else {
-			if (!*args[2] || !*args[3]) {
-				appctx->ctx.cli.severity = LOG_ERR;
-				appctx->ctx.cli.msg = "'add acl' expects two parameters: ACL identifier and pattern.\n";
-				appctx->st0 = CLI_ST_PRINT;
-				return 1;
-			}
+			    (payload && !*args[2]))
+				return cli_err(appctx,
+					       "'add map' expects three parameters (map identifier, key and value)"
+					       " or one parameter (map identifier) and a payload\n");
 		}
+		else if (!*args[2] || !*args[3])
+			return cli_err(appctx, "'add acl' expects two parameters: ACL identifier and pattern.\n");
 
 		/* Lookup for the reference. */
 		appctx->ctx.map.ref = pat_ref_lookup_ref(args[2]);
 		if (!appctx->ctx.map.ref) {
-			if (appctx->ctx.map.display_flags == PAT_REF_MAP) {
-				appctx->ctx.cli.severity = LOG_ERR;
-				appctx->ctx.cli.msg = "Unknown map identifier. Please use #<id> or <file>.\n";
-			}
-			else {
-				appctx->ctx.cli.severity = LOG_ERR;
-				appctx->ctx.cli.msg = "Unknown ACL identifier. Please use #<id> or <file>.\n";
-			}
-			appctx->st0 = CLI_ST_PRINT;
-			return 1;
+			if (appctx->ctx.map.display_flags == PAT_REF_MAP)
+				return cli_err(appctx, "Unknown map identifier. Please use #<id> or <file>.\n");
+			else
+				return cli_err(appctx, "Unknown ACL identifier. Please use #<id> or <file>.\n");
 		}
 
 		/* The command "add acl" is prohibited if the reference
@@ -864,29 +799,19 @@ static int cli_parse_add_map(char **args, char *payload, struct appctx *appctx,
 		 */
 		if ((appctx->ctx.map.display_flags & PAT_REF_ACL) &&
 		    (appctx->ctx.map.ref->flags & PAT_REF_SMP)) {
-			appctx->ctx.cli.severity = LOG_ERR;
-			appctx->ctx.cli.msg = "This ACL is shared with a map containing samples. "
-				"You must use the command 'add map' to add values.\n";
-			appctx->st0 = CLI_ST_PRINT;
-			return 1;
+			return cli_err(appctx,
+				       "This ACL is shared with a map containing samples. "
+				       "You must use the command 'add map' to add values.\n");
 		}
-
 		/* Add value(s). */
 		err = NULL;
 		if (!payload) {
 			ret = map_add_key_value(appctx, args[3], args[4], &err);
 			if (!ret) {
-				if (err) {
-					memprintf(&err, "%s.\n", err);
-					appctx->ctx.cli.err = err;
-					appctx->st0 = CLI_ST_PRINT_FREE;
-				}
-				else {
-					appctx->ctx.cli.severity = LOG_ERR;
-					appctx->ctx.cli.msg = "Failed to add an entry.\n";
-					appctx->st0 = CLI_ST_PRINT;
-				}
-				return 1;
+				if (err)
+					return cli_dynerr(appctx, memprintf(&err, "%s.\n", err));
+				else
+					return cli_err(appctx, "Failed to add an entry.\n");
 			}
 		}
 		else {
@@ -901,12 +826,9 @@ static int cli_parse_add_map(char **args, char *payload, struct appctx *appctx,
 				l = strcspn(key, " \t");
 				payload += l;
 
-				if (!*payload && appctx->ctx.map.display_flags == PAT_REF_MAP) {
-					memprintf(&err, "Missing value for key '%s'.\n", key);
-					appctx->ctx.cli.err = err;
-					appctx->st0 = CLI_ST_PRINT_FREE;
-					return 1;
-				}
+				if (!*payload && appctx->ctx.map.display_flags == PAT_REF_MAP)
+					return cli_dynerr(appctx, memprintf(&err, "Missing value for key '%s'.\n", key));
+
 				key[l] = 0;
 				payload++;
 
@@ -921,17 +843,10 @@ static int cli_parse_add_map(char **args, char *payload, struct appctx *appctx,
 
 				ret = map_add_key_value(appctx, key, value, &err);
 				if (!ret) {
-					if (err) {
-						memprintf(&err, "%s.\n", err);
-						appctx->ctx.cli.err = err;
-						appctx->st0 = CLI_ST_PRINT_FREE;
-					}
-					else {
-						appctx->ctx.cli.severity = LOG_ERR;
-						appctx->ctx.cli.msg = "Failed to add a key.\n";
-						appctx->st0 = CLI_ST_PRINT;
-					}
-					return 1;
+					if (err)
+						return cli_dynerr(appctx, memprintf(&err, "%s.\n", err));
+					else
+						return cli_err(appctx, "Failed to add a key.\n");
 				}
 			}
 		}
@@ -952,33 +867,18 @@ static int cli_parse_del_map(char **args, char *payload, struct appctx *appctx,
 		appctx->ctx.map.display_flags = PAT_REF_ACL;
 
 	/* Expect two parameters: map name and key. */
-	if (appctx->ctx.map.display_flags == PAT_REF_MAP) {
-		if (!*args[2] || !*args[3]) {
-			appctx->ctx.cli.severity = LOG_ERR;
-			appctx->ctx.cli.msg = "This command expects two parameters: map identifier and key.\n";
-			appctx->st0 = CLI_ST_PRINT;
-			return 1;
-		}
-	}
-
-	else {
-		if (!*args[2] || !*args[3]) {
-			appctx->ctx.cli.severity = LOG_ERR;
-			appctx->ctx.cli.msg = "This command expects two parameters: ACL identifier and key.\n";
-			appctx->st0 = CLI_ST_PRINT;
-			return 1;
-		}
+	if (!*args[2] || !*args[3]) {
+		if (appctx->ctx.map.display_flags == PAT_REF_MAP)
+			return cli_err(appctx, "This command expects two parameters: map identifier and key.\n");
+		else
+			return cli_err(appctx, "This command expects two parameters: ACL identifier and key.\n");
 	}
 
 	/* Lookup the reference in the maps. */
 	appctx->ctx.map.ref = pat_ref_lookup_ref(args[2]);
 	if (!appctx->ctx.map.ref ||
-	    !(appctx->ctx.map.ref->flags & appctx->ctx.map.display_flags)) {
-		appctx->ctx.cli.severity = LOG_ERR;
-		appctx->ctx.cli.msg = "Unknown map identifier. Please use #<id> or <file>.\n";
-		appctx->st0 = CLI_ST_PRINT;
-		return 1;
-	}
+	    !(appctx->ctx.map.ref->flags & appctx->ctx.map.display_flags))
+		return cli_err(appctx, "Unknown map identifier. Please use #<id> or <file>.\n");
 
 	/* If the entry identifier start with a '#', it is considered as
 	 * pointer id
@@ -990,31 +890,20 @@ static int cli_parse_del_map(char **args, char *payload, struct appctx *appctx,
 
 		/* Convert argument to integer value. */
 		conv = strtoll(&args[3][1], &error, 16);
-		if (*error != '\0') {
-			appctx->ctx.cli.severity = LOG_ERR;
-			appctx->ctx.cli.msg = "Malformed identifier. Please use #<id> or <file>.\n";
-			appctx->st0 = CLI_ST_PRINT;
-			return 1;
-		}
+		if (*error != '\0')
+			return cli_err(appctx, "Malformed identifier. Please use #<id> or <file>.\n");
 
 		/* Convert and check integer to pointer. */
 		ref = (struct pat_ref_elt *)(long)conv;
-		if ((long long int)(long)ref != conv) {
-			appctx->ctx.cli.severity = LOG_ERR;
-			appctx->ctx.cli.msg = "Malformed identifier. Please use #<id> or <file>.\n";
-			appctx->st0 = CLI_ST_PRINT;
-			return 1;
-		}
+		if ((long long int)(long)ref != conv)
+			return cli_err(appctx, "Malformed identifier. Please use #<id> or <file>.\n");
 
 		/* Try to delete the entry. */
 		HA_SPIN_LOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
 		if (!pat_ref_delete_by_id(appctx->ctx.map.ref, ref)) {
 			HA_SPIN_UNLOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
 			/* The entry is not found, send message. */
-			appctx->ctx.cli.severity = LOG_ERR;
-			appctx->ctx.cli.msg = "Key not found.\n";
-			appctx->st0 = CLI_ST_PRINT;
-			return 1;
+			return cli_err(appctx, "Key not found.\n");
 		}
 		HA_SPIN_UNLOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
 	}
@@ -1026,10 +915,7 @@ static int cli_parse_del_map(char **args, char *payload, struct appctx *appctx,
 		if (!pat_ref_delete(appctx->ctx.map.ref, args[3])) {
 			HA_SPIN_UNLOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
 			/* The entry is not found, send message. */
-			appctx->ctx.cli.severity = LOG_ERR;
-			appctx->ctx.cli.msg = "Key not found.\n";
-			appctx->st0 = CLI_ST_PRINT;
-			return 1;
+			return cli_err(appctx, "Key not found.\n");
 		}
 		HA_SPIN_UNLOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
 	}
@@ -1040,24 +926,6 @@ static int cli_parse_del_map(char **args, char *payload, struct appctx *appctx,
 }
 
 
-/* continue to clear a map which was started in the parser */
-static int cli_io_handler_clear_map(struct appctx *appctx)
-{
-	struct stream_interface *si = appctx->owner;
-	int finished;
-
-	HA_SPIN_LOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
-	finished = pat_ref_prune(appctx->ctx.map.ref);
-	HA_SPIN_UNLOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
-
-	if (!finished) {
-		/* let's come back later */
-		si_rx_endp_more(si);
-		return 0;
-	}
-	return 1;
-}
-
 static int cli_parse_clear_map(char **args, char *payload, struct appctx *appctx, void *private)
 {
 	if (strcmp(args[1], "map") == 0 || strcmp(args[1], "acl") == 0) {
@@ -1069,50 +937,44 @@ static int cli_parse_clear_map(char **args, char *payload, struct appctx *appctx
 
 		/* no parameter */
 		if (!*args[2]) {
-			if (appctx->ctx.map.display_flags == PAT_REF_MAP) {
-				appctx->ctx.cli.severity = LOG_ERR;
-				appctx->ctx.cli.msg = "Missing map identifier.\n";
-			}
-			else {
-				appctx->ctx.cli.severity = LOG_ERR;
-				appctx->ctx.cli.msg = "Missing ACL identifier.\n";
-			}
-			appctx->st0 = CLI_ST_PRINT;
-			return 1;
+			if (appctx->ctx.map.display_flags == PAT_REF_MAP)
+				return cli_err(appctx, "Missing map identifier.\n");
+			else
+				return cli_err(appctx, "Missing ACL identifier.\n");
 		}
 
 		/* lookup into the refs and check the map flag */
 		appctx->ctx.map.ref = pat_ref_lookup_ref(args[2]);
 		if (!appctx->ctx.map.ref ||
 		    !(appctx->ctx.map.ref->flags & appctx->ctx.map.display_flags)) {
-			if (appctx->ctx.map.display_flags == PAT_REF_MAP) {
-				appctx->ctx.cli.severity = LOG_ERR;
-				appctx->ctx.cli.msg = "Unknown map identifier. Please use #<id> or <file>.\n";
-			}
-			else {
-				appctx->ctx.cli.severity = LOG_ERR;
-				appctx->ctx.cli.msg = "Unknown ACL identifier. Please use #<id> or <file>.\n";
-			}
-			appctx->st0 = CLI_ST_PRINT;
-			return 1;
+			if (appctx->ctx.map.display_flags == PAT_REF_MAP)
+				return cli_err(appctx, "Unknown map identifier. Please use #<id> or <file>.\n");
+			else
+				return cli_err(appctx, "Unknown ACL identifier. Please use #<id> or <file>.\n");
 		}
 
-		/* delegate the clearing to the I/O handler which can yield */
-		return 0;
+		/* Clear all. */
+		HA_SPIN_LOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
+		pat_ref_prune(appctx->ctx.map.ref);
+		HA_SPIN_UNLOCK(PATREF_LOCK, &appctx->ctx.map.ref->lock);
+
+		/* return response */
+		appctx->st0 = CLI_ST_PROMPT;
+		return 1;
 	}
-	return 1;
+	return 0;
 }
 
 /* register cli keywords */
 
 static struct cli_kw_list cli_kws = {{ },{
 	{ { "add",   "acl", NULL }, "add acl        : add acl entry", cli_parse_add_map, NULL },
-	{ { "clear", "acl", NULL }, "clear acl <id> : clear the content of this acl", cli_parse_clear_map, cli_io_handler_clear_map, NULL },
+	{ { "clear", "acl", NULL }, "clear acl <id> : clear the content of this acl", cli_parse_clear_map, NULL },
 	{ { "del",   "acl", NULL }, "del acl        : delete acl entry", cli_parse_del_map, NULL },
 	{ { "get",   "acl", NULL }, "get acl        : report the patterns matching a sample for an ACL", cli_parse_get_map, cli_io_handler_map_lookup, cli_release_mlook },
 	{ { "show",  "acl", NULL }, "show acl [id]  : report available acls or dump an acl's contents", cli_parse_show_map, NULL },
 	{ { "add",   "map", NULL }, "add map        : add map entry", cli_parse_add_map, NULL },
-	{ { "clear", "map", NULL }, "clear map <id> : clear the content of this map", cli_parse_clear_map, cli_io_handler_clear_map, NULL },
+	{ { "clear", "map", NULL }, "clear map <id> : clear the content of this map", cli_parse_clear_map, NULL },
 	{ { "del",   "map", NULL }, "del map        : delete map entry", cli_parse_del_map, NULL },
 	{ { "get",   "map", NULL }, "get map        : report the keys and values matching a sample for a map", cli_parse_get_map, cli_io_handler_map_lookup, cli_release_mlook },
 	{ { "set",   "map", NULL }, "set map        : modify map entry", cli_parse_set_map, NULL },
