@@ -25,9 +25,7 @@
 
 #include <sys/param.h>
 #include <sys/socket.h>
-#include <sys/stat.h>
 #include <sys/types.h>
-#include <sys/un.h>
 
 #include <netinet/tcp.h>
 #include <netinet/in.h>
@@ -55,7 +53,7 @@
 #include <proto/log.h>
 #include <proto/port_range.h>
 #include <proto/protocol.h>
-#include <proto/proto_http.h>
+#include <proto/http_ana.h>
 #include <proto/proto_tcp.h>
 #include <proto/proxy.h>
 #include <proto/sample.h>
@@ -242,6 +240,7 @@ int tcp_bind_socket(int fd, int flags, struct sockaddr_storage *local, struct so
 	return 0;
 }
 
+/* conn->dst MUST be valid */
 static int create_server_socket(struct connection *conn)
 {
 	const struct netns_entry *ns = NULL;
@@ -254,13 +253,13 @@ static int create_server_socket(struct connection *conn)
 			ns = __objt_server(conn->target)->netns;
 	}
 #endif
-	return my_socketat(ns, conn->addr.to.ss_family, SOCK_STREAM, IPPROTO_TCP);
+	return my_socketat(ns, conn->dst->ss_family, SOCK_STREAM, IPPROTO_TCP);
 }
 
 /*
  * This function initiates a TCP connection establishment to the target assigned
- * to connection <conn> using (si->{target,addr.to}). A source address may be
- * pointed to by conn->addr.from in case of transparent proxying. Normal source
+ * to connection <conn> using (si->{target,dst}). A source address may be
+ * pointed to by conn->src in case of transparent proxying. Normal source
  * bind addresses are still determined locally (due to the possible need of a
  * source port). conn->target may point either to a valid server or to a backend,
  * depending on conn->target. Only OBJ_TYPE_PROXY and OBJ_TYPE_SERVER are
@@ -320,6 +319,11 @@ int tcp_connect_server(struct connection *conn, int flags)
 		return SF_ERR_INTERNAL;
 	}
 
+	if (!conn->dst) {
+		conn->flags |= CO_FL_ERROR;
+		return SF_ERR_INTERNAL;
+	}
+
 	fd = conn->handle.fd = create_server_socket(conn);
 
 	if (fd == -1) {
@@ -399,7 +403,7 @@ int tcp_connect_server(struct connection *conn, int flags)
 	if (src) {
 		int ret, flags = 0;
 
-		if (is_inet_addr(&conn->addr.from)) {
+		if (conn->src && is_inet_addr(conn->src)) {
 			switch (src->opts & CO_SRC_TPROXY_MASK) {
 			case CO_SRC_TPROXY_CLI:
 				conn->flags |= CO_FL_PRIVATE;
@@ -448,7 +452,7 @@ int tcp_connect_server(struct connection *conn, int flags)
 				fdinfo[fd].port_range = src->sport_range;
 				set_host_port(&sa, fdinfo[fd].local_port);
 
-				ret = tcp_bind_socket(fd, flags, &sa, &conn->addr.from);
+				ret = tcp_bind_socket(fd, flags, &sa, conn->src);
 				if (ret != 0)
 					conn->err_code = CO_ER_CANT_BIND;
 			} while (ret != 0); /* binding NOK */
@@ -458,7 +462,7 @@ int tcp_connect_server(struct connection *conn, int flags)
 			static THREAD_LOCAL int bind_address_no_port = 1;
 			setsockopt(fd, SOL_IP, IP_BIND_ADDRESS_NO_PORT, (const void *) &bind_address_no_port, sizeof(int));
 #endif
-			ret = tcp_bind_socket(fd, flags, &src->source_addr, &conn->addr.from);
+			ret = tcp_bind_socket(fd, flags, &src->source_addr, conn->src);
 			if (ret != 0)
 				conn->err_code = CO_ER_CANT_BIND;
 		}
@@ -515,7 +519,7 @@ int tcp_connect_server(struct connection *conn, int flags)
 	if (global.tune.server_rcvbuf)
                 setsockopt(fd, SOL_SOCKET, SO_RCVBUF, &global.tune.server_rcvbuf, sizeof(global.tune.server_rcvbuf));
 
-	addr = (conn->flags & CO_FL_SOCKS4) ? &srv->socks4_addr : &conn->addr.to;
+	addr = (conn->flags & CO_FL_SOCKS4) ? &srv->socks4_addr : conn->dst;
 	if (connect(fd, (const struct sockaddr *)addr, get_addr_len(addr)) == -1) {
 		if (errno == EINPROGRESS || errno == EALREADY) {
 			/* common case, let's wait for connect status */
@@ -572,6 +576,9 @@ int tcp_connect_server(struct connection *conn, int flags)
 	conn_ctrl_init(conn);       /* registers the FD */
 	fdtab[fd].linger_risk = 1;  /* close hard if needed */
 
+	if (conn->flags & CO_FL_WAIT_L4_CONN)
+		fd_cant_recv(fd); // we'll change this once the connection is validated
+
 	if (conn_xprt_init(conn) < 0) {
 		conn_full_close(conn);
 		conn->flags |= CO_FL_ERROR;
@@ -660,7 +667,7 @@ int tcp_connect_probe(struct connection *conn)
 	if (!(conn->flags & CO_FL_WAIT_L4_CONN))
 		return 1; /* strange we were called while ready */
 
-	if (!fd_send_ready(fd) && !(fdtab[fd].ev & (FD_POLL_ERR|FD_POLL_HUP)))
+	if (!fd_send_ready(fd))
 		return 0;
 
 	/* we might be the first witness of FD_POLL_ERR. Note that FD_POLL_HUP
@@ -685,7 +692,7 @@ int tcp_connect_probe(struct connection *conn)
 	 *  - connecting (EALREADY, EINPROGRESS)
 	 *  - connected (EISCONN, 0)
 	 */
-	addr = &conn->addr.to;
+	addr = conn->dst;
 	if ((conn->flags & CO_FL_SOCKS4) && obj_type(conn->target) == OBJ_TYPE_SERVER)
 		addr = &objt_server(conn->target)->socks4_addr;
 
@@ -707,6 +714,8 @@ int tcp_connect_probe(struct connection *conn)
 	 * data layer.
 	 */
 	conn->flags &= ~CO_FL_WAIT_L4_CONN;
+	fd_may_send(fd);
+	fd_cond_recv(fd);
 	return 1;
 
  out_error:
@@ -832,8 +841,8 @@ int tcp_bind_listener(struct listener *listener, char *errmsg, int errlen)
 			if (getsockopt(fd, IPPROTO_TCP, TCP_MAXSEG, &default_tcp_maxseg,
 			    &ready_len) == -1)
 				ha_warning("Failed to get the default value of TCP_MAXSEG\n");
-			close(fd);
 		}
+		close(fd);
 	}
 	if (default_tcp6_maxseg == -1) {
 		default_tcp6_maxseg = -2;
@@ -989,9 +998,9 @@ int tcp_bind_listener(struct listener *listener, char *errmsg, int errlen)
 			defaultmss = default_tcp6_maxseg;
 
 		getsockopt(fd, IPPROTO_TCP, TCP_MAXSEG, &tmpmaxseg, &len);
-		if (defaultmss > 0 &&
-		    tmpmaxseg != defaultmss &&
-		    setsockopt(fd, IPPROTO_TCP, TCP_MAXSEG, &defaultmss, sizeof(defaultmss)) == -1) {
+		if (tmpmaxseg != defaultmss && setsockopt(fd, IPPROTO_TCP,
+						TCP_MAXSEG, &defaultmss,
+						sizeof(defaultmss)) == -1) {
 			msg = "cannot set MSS";
 			err |= ERR_WARN;
 		}
@@ -1082,7 +1091,7 @@ int tcp_bind_listener(struct listener *listener, char *errmsg, int errlen)
 	listener->state = LI_LISTEN;
 
 	fd_insert(fd, listener, listener->proto->accept,
-	          thread_mask(listener->bind_conf->bind_thread) & all_threads_mask);
+	          thread_mask(listener->bind_conf->bind_thread));
 
  tcp_return:
 	if (msg && errlen) {
@@ -1183,21 +1192,21 @@ enum act_return tcp_action_req_set_src(struct act_rule *rule, struct proxy *px,
 {
 	struct connection *cli_conn;
 
-	if ((cli_conn = objt_conn(sess->origin)) && conn_ctrl_ready(cli_conn)) {
+	if ((cli_conn = objt_conn(sess->origin)) && conn_get_src(cli_conn)) {
 		struct sample *smp;
 
 		smp = sample_fetch_as_type(px, sess, s, SMP_OPT_DIR_REQ|SMP_OPT_FINAL, rule->arg.expr, SMP_T_ADDR);
 		if (smp) {
-			int port = get_net_port(&cli_conn->addr.from);
+			int port = get_net_port(cli_conn->src);
 
 			if (smp->data.type == SMP_T_IPV4) {
-				((struct sockaddr_in *)&cli_conn->addr.from)->sin_family = AF_INET;
-				((struct sockaddr_in *)&cli_conn->addr.from)->sin_addr.s_addr = smp->data.u.ipv4.s_addr;
-				((struct sockaddr_in *)&cli_conn->addr.from)->sin_port = port;
+				((struct sockaddr_in *)cli_conn->src)->sin_family = AF_INET;
+				((struct sockaddr_in *)cli_conn->src)->sin_addr.s_addr = smp->data.u.ipv4.s_addr;
+				((struct sockaddr_in *)cli_conn->src)->sin_port = port;
 			} else if (smp->data.type == SMP_T_IPV6) {
-				((struct sockaddr_in6 *)&cli_conn->addr.from)->sin6_family = AF_INET6;
-				memcpy(&((struct sockaddr_in6 *)&cli_conn->addr.from)->sin6_addr, &smp->data.u.ipv6, sizeof(struct in6_addr));
-				((struct sockaddr_in6 *)&cli_conn->addr.from)->sin6_port = port;
+				((struct sockaddr_in6 *)cli_conn->src)->sin6_family = AF_INET6;
+				memcpy(&((struct sockaddr_in6 *)cli_conn->src)->sin6_addr, &smp->data.u.ipv6, sizeof(struct in6_addr));
+				((struct sockaddr_in6 *)cli_conn->src)->sin6_port = port;
 			}
 		}
 		cli_conn->flags |= CO_FL_ADDR_FROM_SET;
@@ -1215,21 +1224,20 @@ enum act_return tcp_action_req_set_dst(struct act_rule *rule, struct proxy *px,
 {
 	struct connection *cli_conn;
 
-	if ((cli_conn = objt_conn(sess->origin)) && conn_ctrl_ready(cli_conn)) {
+	if ((cli_conn = objt_conn(sess->origin)) && conn_get_dst(cli_conn)) {
 		struct sample *smp;
 
 		smp = sample_fetch_as_type(px, sess, s, SMP_OPT_DIR_REQ|SMP_OPT_FINAL, rule->arg.expr, SMP_T_ADDR);
 		if (smp) {
-			int port = get_net_port(&cli_conn->addr.to);
+			int port = get_net_port(cli_conn->dst);
 
 			if (smp->data.type == SMP_T_IPV4) {
-				((struct sockaddr_in *)&cli_conn->addr.to)->sin_family = AF_INET;
-				((struct sockaddr_in *)&cli_conn->addr.to)->sin_addr.s_addr = smp->data.u.ipv4.s_addr;
-				((struct sockaddr_in *)&cli_conn->addr.to)->sin_port = port;
+				((struct sockaddr_in *)cli_conn->dst)->sin_family = AF_INET;
+				((struct sockaddr_in *)cli_conn->dst)->sin_addr.s_addr = smp->data.u.ipv4.s_addr;
 			} else if (smp->data.type == SMP_T_IPV6) {
-				((struct sockaddr_in6 *)&cli_conn->addr.to)->sin6_family = AF_INET6;
-				memcpy(&((struct sockaddr_in6 *)&cli_conn->addr.to)->sin6_addr, &smp->data.u.ipv6, sizeof(struct in6_addr));
-				((struct sockaddr_in6 *)&cli_conn->addr.to)->sin6_port = port;
+				((struct sockaddr_in6 *)cli_conn->dst)->sin6_family = AF_INET6;
+				memcpy(&((struct sockaddr_in6 *)cli_conn->dst)->sin6_addr, &smp->data.u.ipv6, sizeof(struct in6_addr));
+				((struct sockaddr_in6 *)cli_conn->dst)->sin6_port = port;
 			}
 			cli_conn->flags |= CO_FL_ADDR_TO_SET;
 		}
@@ -1248,21 +1256,19 @@ enum act_return tcp_action_req_set_src_port(struct act_rule *rule, struct proxy
 {
 	struct connection *cli_conn;
 
-	if ((cli_conn = objt_conn(sess->origin)) && conn_ctrl_ready(cli_conn)) {
+	if ((cli_conn = objt_conn(sess->origin)) && conn_get_src(cli_conn)) {
 		struct sample *smp;
 
-		conn_get_from_addr(cli_conn);
-
 		smp = sample_fetch_as_type(px, sess, s, SMP_OPT_DIR_REQ|SMP_OPT_FINAL, rule->arg.expr, SMP_T_SINT);
 		if (smp) {
-			if (cli_conn->addr.from.ss_family == AF_INET6) {
-				((struct sockaddr_in6 *)&cli_conn->addr.from)->sin6_port = htons(smp->data.u.sint);
+			if (cli_conn->src->ss_family == AF_INET6) {
+				((struct sockaddr_in6 *)cli_conn->src)->sin6_port = htons(smp->data.u.sint);
 			} else {
-				if (cli_conn->addr.from.ss_family != AF_INET) {
-					cli_conn->addr.from.ss_family = AF_INET;
-					((struct sockaddr_in *)&cli_conn->addr.from)->sin_addr.s_addr = 0;
+				if (cli_conn->src->ss_family != AF_INET) {
+					cli_conn->src->ss_family = AF_INET;
+					((struct sockaddr_in *)cli_conn->src)->sin_addr.s_addr = 0;
 				}
-				((struct sockaddr_in *)&cli_conn->addr.from)->sin_port = htons(smp->data.u.sint);
+				((struct sockaddr_in *)cli_conn->src)->sin_port = htons(smp->data.u.sint);
 			}
 		}
 	}
@@ -1280,21 +1286,19 @@ enum act_return tcp_action_req_set_dst_port(struct act_rule *rule, struct proxy
 {
 	struct connection *cli_conn;
 
-	if ((cli_conn = objt_conn(sess->origin)) && conn_ctrl_ready(cli_conn)) {
+	if ((cli_conn = objt_conn(sess->origin)) && conn_get_dst(cli_conn)) {
 		struct sample *smp;
 
-		conn_get_to_addr(cli_conn);
-
 		smp = sample_fetch_as_type(px, sess, s, SMP_OPT_DIR_REQ|SMP_OPT_FINAL, rule->arg.expr, SMP_T_SINT);
 		if (smp) {
-			if (cli_conn->addr.to.ss_family == AF_INET6) {
-				((struct sockaddr_in6 *)&cli_conn->addr.to)->sin6_port = htons(smp->data.u.sint);
+			if (cli_conn->dst->ss_family == AF_INET6) {
+				((struct sockaddr_in6 *)cli_conn->dst)->sin6_port = htons(smp->data.u.sint);
 			} else {
-				if (cli_conn->addr.to.ss_family != AF_INET) {
-					cli_conn->addr.to.ss_family = AF_INET;
-					((struct sockaddr_in *)&cli_conn->addr.to)->sin_addr.s_addr = 0;
+				if (cli_conn->dst->ss_family != AF_INET) {
+					cli_conn->dst->ss_family = AF_INET;
+					((struct sockaddr_in *)cli_conn->dst)->sin_addr.s_addr = 0;
 				}
-				((struct sockaddr_in *)&cli_conn->addr.to)->sin_port = htons(smp->data.u.sint);
+				((struct sockaddr_in *)cli_conn->dst)->sin_port = htons(smp->data.u.sint);
 			}
 		}
 	}
@@ -1344,27 +1348,16 @@ static enum act_return tcp_exec_action_silent_drop(struct act_rule *rule, struct
 	 * network and has no effect on local net.
 	 */
 #ifdef IP_TTL
-	if (conn->addr.from.ss_family == AF_INET)
-		setsockopt(conn->handle.fd, SOL_IP, IP_TTL, &one, sizeof(one));
-#endif
-#ifdef IPV6_UNICAST_HOPS
-#if defined(SOL_IPV6)
-	if (conn->addr.from.ss_family == AF_INET6)
-		setsockopt(conn->handle.fd, SOL_IPV6, IPV6_UNICAST_HOPS, &one, sizeof(one));
-#elif defined(IPPROTO_IPV6)
-	if (conn->addr.from.ss_family == AF_INET6)
-		setsockopt(conn->handle.fd, IPPROTO_IPV6, IPV6_UNICAST_HOPS, &one, sizeof(one));
-#endif
+	setsockopt(conn->handle.fd, SOL_IP, IP_TTL, &one, sizeof(one));
 #endif
  out:
 	/* kill the stream if any */
 	if (strm) {
 		channel_abort(&strm->req);
 		channel_abort(&strm->res);
-		strm->req.analysers &= AN_REQ_FLT_END;
-		strm->res.analysers &= AN_RES_FLT_END;
-		if (strm->flags & SF_BE_ASSIGNED)
-			_HA_ATOMIC_ADD(&strm->be->be_counters.denied_req, 1);
+		strm->req.analysers = 0;
+		strm->res.analysers = 0;
+		_HA_ATOMIC_ADD(&strm->be->be_counters.denied_req, 1);
 		if (!(strm->flags & SF_ERR_MASK))
 			strm->flags |= SF_ERR_PRXCOND;
 		if (!(strm->flags & SF_FINST_MASK))
@@ -1372,7 +1365,7 @@ static enum act_return tcp_exec_action_silent_drop(struct act_rule *rule, struct
 	}
 
 	_HA_ATOMIC_ADD(&sess->fe->fe_counters.denied_req, 1);
-	if (sess->listener && sess->listener->counters)
+	if (sess->listener->counters)
 		_HA_ATOMIC_ADD(&sess->listener->counters->denied_req, 1);
 
 	return ACT_RET_STOP;
@@ -1448,13 +1441,16 @@ int smp_fetch_src(const struct arg *args, struct sample *smp, const char *kw, vo
 	if (!cli_conn)
 		return 0;
 
-	switch (cli_conn->addr.from.ss_family) {
+	if (!conn_get_src(cli_conn))
+		return 0;
+
+	switch (cli_conn->src->ss_family) {
 	case AF_INET:
-		smp->data.u.ipv4 = ((struct sockaddr_in *)&cli_conn->addr.from)->sin_addr;
+		smp->data.u.ipv4 = ((struct sockaddr_in *)cli_conn->src)->sin_addr;
 		smp->data.type = SMP_T_IPV4;
 		break;
 	case AF_INET6:
-		smp->data.u.ipv6 = ((struct sockaddr_in6 *)&cli_conn->addr.from)->sin6_addr;
+		smp->data.u.ipv6 = ((struct sockaddr_in6 *)cli_conn->src)->sin6_addr;
 		smp->data.type = SMP_T_IPV6;
 		break;
 	default:
@@ -1474,8 +1470,11 @@ smp_fetch_sport(const struct arg *args, struct sample *smp, const char *k, void
 	if (!cli_conn)
 		return 0;
 
+	if (!conn_get_src(cli_conn))
+		return 0;
+
 	smp->data.type = SMP_T_SINT;
-	if (!(smp->data.u.sint = get_host_port(&cli_conn->addr.from)))
+	if (!(smp->data.u.sint = get_host_port(cli_conn->src)))
 		return 0;
 
 	smp->flags = 0;
@@ -1491,15 +1490,16 @@ smp_fetch_dst(const struct arg *args, struct sample *smp, const char *kw, void *
 	if (!cli_conn)
 		return 0;
 
-	conn_get_to_addr(cli_conn);
+	if (!conn_get_dst(cli_conn))
+		return 0;
 
-	switch (cli_conn->addr.to.ss_family) {
+	switch (cli_conn->dst->ss_family) {
 	case AF_INET:
-		smp->data.u.ipv4 = ((struct sockaddr_in *)&cli_conn->addr.to)->sin_addr;
+		smp->data.u.ipv4 = ((struct sockaddr_in *)cli_conn->dst)->sin_addr;
 		smp->data.type = SMP_T_IPV4;
 		break;
 	case AF_INET6:
-		smp->data.u.ipv6 = ((struct sockaddr_in6 *)&cli_conn->addr.to)->sin6_addr;
+		smp->data.u.ipv6 = ((struct sockaddr_in6 *)cli_conn->dst)->sin6_addr;
 		smp->data.type = SMP_T_IPV6;
 		break;
 	default:
@@ -1521,13 +1521,12 @@ int smp_fetch_dst_is_local(const struct arg *args, struct sample *smp, const cha
 	if (!conn)
 		return 0;
 
-	conn_get_to_addr(conn);
-	if (!(conn->flags & CO_FL_ADDR_TO_SET))
+	if (!conn_get_dst(conn))
 		return 0;
 
 	smp->data.type = SMP_T_BOOL;
 	smp->flags = 0;
-	smp->data.u.sint = addr_is_local(li->netns, &conn->addr.to);
+	smp->data.u.sint = addr_is_local(li->netns, conn->dst);
 	return smp->data.u.sint >= 0;
 }
 
@@ -1542,13 +1541,12 @@ int smp_fetch_src_is_local(const struct arg *args, struct sample *smp, const cha
 	if (!conn)
 		return 0;
 
-	conn_get_from_addr(conn);
-	if (!(conn->flags & CO_FL_ADDR_FROM_SET))
+	if (!conn_get_src(conn))
 		return 0;
 
 	smp->data.type = SMP_T_BOOL;
 	smp->flags = 0;
-	smp->data.u.sint = addr_is_local(li->netns, &conn->addr.from);
+	smp->data.u.sint = addr_is_local(li->netns, conn->src);
 	return smp->data.u.sint >= 0;
 }
 
@@ -1561,10 +1559,11 @@ smp_fetch_dport(const struct arg *args, struct sample *smp, const char *kw, void
 	if (!cli_conn)
 		return 0;
 
-	conn_get_to_addr(cli_conn);
+	if (!conn_get_dst(cli_conn))
+		return 0;
 
 	smp->data.type = SMP_T_SINT;
-	if (!(smp->data.u.sint = get_host_port(&cli_conn->addr.to)))
+	if (!(smp->data.u.sint = get_host_port(cli_conn->dst)))
 		return 0;
 
 	smp->flags = 0;
@@ -1917,10 +1916,8 @@ static int bind_parse_interface(char **args, int cur_arg, struct proxy *px, stru
 	}
 
 	list_for_each_entry(l, &conf->listeners, by_bind) {
-		if (l->addr.ss_family == AF_INET || l->addr.ss_family == AF_INET6) {
-			free(l->interface);
+		if (l->addr.ss_family == AF_INET || l->addr.ss_family == AF_INET6)
 			l->interface = strdup(args[cur_arg + 1]);
-		}
 	}
 
 	return 0;
