@@ -41,11 +41,10 @@
 #include <proto/backend.h>
 #include <proto/fd.h>
 #include <proto/filters.h>
-#include <proto/hdr_idx.h>
 #include <proto/listener.h>
 #include <proto/log.h>
 #include <proto/proto_tcp.h>
-#include <proto/proto_http.h>
+#include <proto/http_ana.h>
 #include <proto/proxy.h>
 #include <proto/server.h>
 #include <proto/signal.h>
@@ -106,15 +105,14 @@ const struct cfg_opt cfg_opts2[] =
 	{ "socket-stats",                 PR_O2_SOCKSTAT,  PR_CAP_FE, 0, 0 },
 	{ "tcp-smart-accept",             PR_O2_SMARTACC,  PR_CAP_FE, 0, 0 },
 	{ "tcp-smart-connect",            PR_O2_SMARTCON,  PR_CAP_BE, 0, 0 },
-	{ "independant-streams",          PR_O2_INDEPSTR,  PR_CAP_FE|PR_CAP_BE, 0, 0 },
 	{ "independent-streams",          PR_O2_INDEPSTR,  PR_CAP_FE|PR_CAP_BE, 0, 0 },
 	{ "http-use-proxy-header",        PR_O2_USE_PXHDR, PR_CAP_FE, 0, PR_MODE_HTTP },
 	{ "http-pretend-keepalive",       PR_O2_FAKE_KA,   PR_CAP_BE, 0, PR_MODE_HTTP },
 	{ "http-no-delay",                PR_O2_NODELAY,   PR_CAP_FE|PR_CAP_BE, 0, PR_MODE_HTTP },
-	{ "http-use-htx",                 PR_O2_USE_HTX,   PR_CAP_FE|PR_CAP_BE, 0, 0 },
-	{"h1-case-adjust-bogus-client",   PR_O2_H1_ADJ_BUGCLI, PR_CAP_FE, 0, 0 },
-	{"h1-case-adjust-bogus-server",   PR_O2_H1_ADJ_BUGSRV, PR_CAP_BE, 0, 0 },
-	{"disable-h2-upgrade",            PR_O2_NO_H2_UPGRADE, PR_CAP_FE, 0, PR_MODE_HTTP },
+	{ "http-use-htx",                 0,               PR_CAP_FE|PR_CAP_BE, 0, 0 }, // deprecated
+
+	{"h1-case-adjust-bogus-client",   PR_O2_H1_ADJ_BUGCLI, PR_CAP_FE, 0, PR_MODE_HTTP },
+	{"h1-case-adjust-bogus-server",   PR_O2_H1_ADJ_BUGSRV, PR_CAP_BE, 0, PR_MODE_HTTP },
 	{ NULL, 0, 0, 0 }
 };
 
@@ -207,7 +205,6 @@ static int proxy_parse_timeout(char **args, int section, struct proxy *proxy,
 	const char *res, *name;
 	int *tv = NULL;
 	int *td = NULL;
-	int warn = 0;
 
 	retval = 0;
 
@@ -216,7 +213,7 @@ static int proxy_parse_timeout(char **args, int section, struct proxy *proxy,
 		args++;
 
 	name = args[0];
-	if (!strcmp(args[0], "client") || (!strcmp(args[0], "clitimeout") && (warn = WARN_CLITO_DEPRECATED))) {
+	if (!strcmp(args[0], "client")) {
 		name = "client";
 		tv = &proxy->timeout.client;
 		td = &defpx->timeout.client;
@@ -233,12 +230,12 @@ static int proxy_parse_timeout(char **args, int section, struct proxy *proxy,
 		tv = &proxy->timeout.httpreq;
 		td = &defpx->timeout.httpreq;
 		cap = PR_CAP_FE | PR_CAP_BE;
-	} else if (!strcmp(args[0], "server") || (!strcmp(args[0], "srvtimeout") && (warn = WARN_SRVTO_DEPRECATED))) {
+	} else if (!strcmp(args[0], "server")) {
 		name = "server";
 		tv = &proxy->timeout.server;
 		td = &defpx->timeout.server;
 		cap = PR_CAP_BE;
-	} else if (!strcmp(args[0], "connect") || (!strcmp(args[0], "contimeout") && (warn = WARN_CONTO_DEPRECATED))) {
+	} else if (!strcmp(args[0], "connect")) {
 		name = "connect";
 		tv = &proxy->timeout.connect;
 		td = &defpx->timeout.connect;
@@ -263,6 +260,15 @@ static int proxy_parse_timeout(char **args, int section, struct proxy *proxy,
 		tv = &proxy->timeout.serverfin;
 		td = &defpx->timeout.serverfin;
 		cap = PR_CAP_BE;
+	} else if (!strcmp(args[0], "clitimeout")) {
+		memprintf(err, "the '%s' directive is not supported anymore since HAProxy 2.1. Use 'timeout client'.", args[0]);
+		return -1;
+	} else if (!strcmp(args[0], "srvtimeout")) {
+		memprintf(err, "the '%s' directive is not supported anymore since HAProxy 2.1. Use 'timeout server'.", args[0]);
+		return -1;
+	} else if (!strcmp(args[0], "contimeout")) {
+		memprintf(err, "the '%s' directive is not supported anymore since HAProxy 2.1. Use 'timeout connect'.", args[0]);
+		return -1;
 	} else {
 		memprintf(err,
 		          "'timeout' supports 'client', 'server', 'connect', 'check', "
@@ -303,13 +309,6 @@ static int proxy_parse_timeout(char **args, int section, struct proxy *proxy,
 		memprintf(err, "overwriting 'timeout %s' which was already specified", name);
 		retval = 1;
 	}
-	else if (warn) {
-		if (!already_warned(warn)) {
-			memprintf(err, "the '%s' directive is now deprecated in favor of 'timeout %s', and will not be supported in future versions.",
-				  args[0], name);
-			retval = 1;
-		}
-	}
 
 	if (*args[2] != 0) {
 		memprintf(err, "'timeout %s' : unexpected extra argument '%s' after value '%s'.", name, args[2], args[1]);
@@ -491,10 +490,6 @@ static int proxy_parse_declare(char **args, int section, struct proxy *curpx,
 
 		/* register the capture. */
 		hdr = calloc(1, sizeof(*hdr));
-		if (!hdr) {
-			memprintf(err, "proxy '%s': out of memory while registering a capture", curpx->id);
-			return -1;
-		}
 		hdr->name = NULL; /* not a header capture */
 		hdr->namelen = 0;
 		hdr->len = len;
@@ -822,14 +817,6 @@ int proxy_cfg_ensure_no_http(struct proxy *curproxy)
 		ha_warning("config : cookie will be ignored for %s '%s' (needs 'mode http').\n",
 			   proxy_type_str(curproxy), curproxy->id);
 	}
-	if (curproxy->rsp_exp != NULL) {
-		ha_warning("config : server regular expressions will be ignored for %s '%s' (needs 'mode http').\n",
-			   proxy_type_str(curproxy), curproxy->id);
-	}
-	if (curproxy->req_exp != NULL) {
-		ha_warning("config : client regular expressions will be ignored for %s '%s' (needs 'mode http').\n",
-			   proxy_type_str(curproxy), curproxy->id);
-	}
 	if (curproxy->monitor_uri != NULL) {
 		ha_warning("config : monitor-uri will be ignored for %s '%s' (needs 'mode http').\n",
 			   proxy_type_str(curproxy), curproxy->id);
@@ -870,7 +857,6 @@ void init_new_proxy(struct proxy *p)
 	LIST_INIT(&p->acl);
 	LIST_INIT(&p->http_req_rules);
 	LIST_INIT(&p->http_res_rules);
-	LIST_INIT(&p->block_rules);
 	LIST_INIT(&p->redirect_rules);
 	LIST_INIT(&p->mon_fail_cond);
 	LIST_INIT(&p->switching_rules);
@@ -882,9 +868,7 @@ void init_new_proxy(struct proxy *p)
 	LIST_INIT(&p->tcp_rep.inspect_rules);
 	LIST_INIT(&p->tcp_req.l4_rules);
 	LIST_INIT(&p->tcp_req.l5_rules);
-	LIST_INIT(&p->req_add);
-	LIST_INIT(&p->rsp_add);
-	LIST_INIT(&p->listener_queue);
+	MT_LIST_INIT(&p->listener_queue);
 	LIST_INIT(&p->logsrvs);
 	LIST_INIT(&p->logformat);
 	LIST_INIT(&p->logformat_sd);
@@ -902,9 +886,6 @@ void init_new_proxy(struct proxy *p)
 	/* initial uuid is unassigned (-1) */
 	p->uuid = -1;
 
-	/* HTX is the default mode, for HTTP and TCP */
-	p->options2 |= PR_O2_USE_HTX;
-
 	/* Default to only allow L4 retries */
 	p->retry_type = PR_RE_CONN_FAILED;
 
@@ -1016,14 +997,9 @@ struct task *manage_proxy(struct task *t, void *context, unsigned short state)
 	 * However we protect tables that are being synced to peers.
 	 */
 	if (unlikely(stopping && p->state == PR_STSTOPPED && p->table && p->table->current)) {
-		if (!p->table->refcnt) {
-			/* !table->refcnt means there
-			 * is no more pending full resync
-			 * to push to a new process and
-			 * we are free to flush the table.
-			 */
-			if (stktable_trash_oldest(p->table, p->table->current))
-				pool_gc(NULL);
+		if (!p->table->syncing) {
+			stktable_trash_oldest(p->table, p->table->current);
+			pool_gc(NULL);
 		}
 		if (p->table->current) {
 			/* some entries still remain, let's recheck in one second */
@@ -1058,7 +1034,7 @@ struct task *manage_proxy(struct task *t, void *context, unsigned short state)
 	}
 
 	/* The proxy is not limited so we can re-enable any waiting listener */
-	if (!LIST_ISEMPTY(&p->listener_queue))
+	if (!MT_LIST_ISEMPTY(&p->listener_queue))
 		dequeue_all_listeners(&p->listener_queue);
  out:
 	t->expire = next;
@@ -1099,15 +1075,11 @@ struct task *hard_stop(struct task *t, void *context, unsigned short state)
 {
 	struct proxy *p;
 	struct stream *s;
-	int thr;
 
 	if (killed) {
 		ha_warning("Some tasks resisted to hard-stop, exiting now.\n");
 		send_log(NULL, LOG_WARNING, "Some tasks resisted to hard-stop, exiting now.\n");
 		killed = 2;
-		for (thr = 0; thr < global.nbthread; thr++)
-			if (((all_threads_mask & ~tid_bit) >> thr) & 1)
-				wake_thread(thr);
 		t->expire = TICK_ETERNITY;
 		return t;
 	}
@@ -1124,12 +1096,9 @@ struct task *hard_stop(struct task *t, void *context, unsigned short state)
 		}
 		p = p->next;
 	}
-
-	thread_isolate();
 	list_for_each_entry(s, &streams, list) {
 		stream_shutdown(s, SF_ERR_KILLED);
 	}
-	thread_release();
 
 	killed = 1;
 	t->expire = tick_add(now_ms, MS_TO_TICKS(1000));
@@ -1148,8 +1117,6 @@ void soft_stop(void)
 	struct task *task;
 
 	stopping = 1;
-	/* disable busy polling to avoid cpu eating for the new process */
-	global.tune.options &= ~GTUNE_BUSY_POLLING;
 	if (tick_isset(global.hard_stop_after)) {
 		task = task_new(MAX_THREADS_MASK);
 		if (task) {
@@ -1167,10 +1134,10 @@ void soft_stop(void)
 		if (p->state == PR_STSTOPPED &&
 		    !LIST_ISEMPTY(&p->conf.listeners) &&
 		    LIST_ELEM(p->conf.listeners.n,
-		    struct listener *, by_fe)->state > LI_ASSIGNED) {
+		    struct listener *, by_fe)->state >= LI_ZOMBIE) {
 			struct listener *l;
 			list_for_each_entry(l, &p->conf.listeners, by_fe) {
-				if (l->state > LI_ASSIGNED)
+				if (l->state >= LI_ZOMBIE)
 					close(l->fd);
 				l->state = LI_INIT;
 			}
@@ -1209,18 +1176,14 @@ void soft_stop(void)
  * listener returns an error, then the proxy state is set to PR_STERROR
  * because we don't know how to resume from this. The function returns 0
  * if it fails, or non-zero on success.
- * The function takes the proxy's lock so it's safe to
- * call from multiple places.
  */
 int pause_proxy(struct proxy *p)
 {
 	struct listener *l;
 
-	HA_SPIN_LOCK(PROXY_LOCK, &p->lock);
-
 	if (!(p->cap & PR_CAP_FE) || p->state == PR_STERROR ||
 	    p->state == PR_STSTOPPED || p->state == PR_STPAUSED)
-		goto end;
+		return 1;
 
 	ha_warning("Pausing %s %s.\n", proxy_cap_str(p->cap), p->id);
 	send_log(p, LOG_WARNING, "Pausing %s %s.\n", proxy_cap_str(p->cap), p->id);
@@ -1233,13 +1196,10 @@ int pause_proxy(struct proxy *p)
 	if (p->state == PR_STERROR) {
 		ha_warning("%s %s failed to enter pause mode.\n", proxy_cap_str(p->cap), p->id);
 		send_log(p, LOG_WARNING, "%s %s failed to enter pause mode.\n", proxy_cap_str(p->cap), p->id);
-		HA_SPIN_UNLOCK(PROXY_LOCK, &p->lock);
 		return 0;
 	}
 
 	p->state = PR_STPAUSED;
-end:
-	HA_SPIN_UNLOCK(PROXY_LOCK, &p->lock);
 	return 1;
 }
 
@@ -1286,8 +1246,7 @@ void zombify_proxy(struct proxy *p)
  * to be called when going down in order to release the ports so that another
  * process may bind to them. It must also be called on disabled proxies at the
  * end of start-up. If all listeners are closed, the proxy is set to the
- * PR_STSTOPPED state.
- * The function takes the proxy's lock so it's safe to
+ * PR_STSTOPPED state. The function takes the proxy's lock so it's safe to
  * call from multiple places.
  */
 void stop_proxy(struct proxy *p)
@@ -1322,18 +1281,14 @@ void stop_proxy(struct proxy *p)
  * listeners and tries to enable them all. If any of them fails, the proxy is
  * put back to the paused state. It returns 1 upon success, or zero if an error
  * is encountered.
- * The function takes the proxy's lock so it's safe to
- * call from multiple places.
  */
 int resume_proxy(struct proxy *p)
 {
 	struct listener *l;
 	int fail;
 
-	HA_SPIN_LOCK(PROXY_LOCK, &p->lock);
-
 	if (p->state != PR_STPAUSED)
-		goto end;
+		return 1;
 
 	ha_warning("Enabling %s %s.\n", proxy_cap_str(p->cap), p->id);
 	send_log(p, LOG_WARNING, "Enabling %s %s.\n", proxy_cap_str(p->cap), p->id);
@@ -1365,13 +1320,9 @@ int resume_proxy(struct proxy *p)
 
 	p->state = PR_STREADY;
 	if (fail) {
-		HA_SPIN_UNLOCK(PROXY_LOCK, &p->lock);
-		/* pause_proxy will take PROXY_LOCK */
 		pause_proxy(p);
 		return 0;
 	}
-end:
-	HA_SPIN_UNLOCK(PROXY_LOCK, &p->lock);
 	return 1;
 }
 
@@ -1461,10 +1412,6 @@ int stream_set_backend(struct stream *s, struct proxy *be)
 			     HA_ATOMIC_ADD(&be->beconn, 1));
 	proxy_inc_be_ctr(be);
 
-	/* HTX/legacy must match */
-	if ((strm_fe(s)->options2 ^ be->options2) & PR_O2_USE_HTX)
-		return 0;
-
 	/* assign new parameters to the stream from the new backend */
 	s->si[1].flags &= ~SI_FL_INDEP_STR;
 	if (be->options2 & PR_O2_INDEPSTR)
@@ -1481,7 +1428,7 @@ int stream_set_backend(struct stream *s, struct proxy *be)
 	s->req.analysers |= be->be_req_ana & ~(strm_li(s) ? strm_li(s)->analysers : 0);
 
 	/* If the target backend requires HTTP processing, we have to allocate
-	 * the HTTP transaction and hdr_idx if we did not have one.
+	 * the HTTP transaction if we did not have one.
 	 */
 	if (unlikely(!s->txn && be->http_needed)) {
 		if (unlikely(!http_alloc_txn(s)))
@@ -1497,20 +1444,9 @@ int stream_set_backend(struct stream *s, struct proxy *be)
 		s->req.analysers |= AN_REQ_FLT_HTTP_HDRS;
 
 	if (s->txn) {
-		if (be->options2 & PR_O2_RSPBUG_OK)
-			s->txn->rsp.err_pos = -1; /* let buggy responses pass */
-
-		/* If we chain to an HTTP backend running a different HTTP mode, we
-		 * have to re-adjust the desired keep-alive/close mode to accommodate
-		 * both the frontend's and the backend's modes.
-		 */
-		if (strm_fe(s)->mode == PR_MODE_HTTP && be->mode == PR_MODE_HTTP &&
-		    ((strm_fe(s)->options & PR_O_HTTP_MODE) != (be->options & PR_O_HTTP_MODE)))
-			http_adjust_conn_mode(s, s->txn, &s->txn->req);
-
 		/* If we chain a TCP frontend to an HTX backend, we must upgrade
 		 * the client mux */
-		if (!IS_HTX_STRM(s) && be->mode == PR_MODE_HTTP && (be->options2 & PR_O2_USE_HTX)) {
+		if (!IS_HTX_STRM(s) && be->mode == PR_MODE_HTTP) {
 			struct connection  *conn = objt_conn(strm_sess(s)->origin);
 			struct conn_stream *cs   = objt_cs(s->si[0].end);
 
@@ -1523,7 +1459,7 @@ int stream_set_backend(struct stream *s, struct proxy *be)
 				if (s->si[0].wait_event.events)
 					conn->mux->unsubscribe(cs, s->si[0].wait_event.events,
 					    &s->si[0].wait_event);
-				if (conn_upgrade_mux_fe(conn, cs, &s->req.buf, ist(""), PROTO_MODE_HTX)  == -1)
+				if (conn_upgrade_mux_fe(conn, cs, &s->req.buf, ist(""), PROTO_MODE_HTTP)  == -1)
 					return 0;
 				if (!strcmp(conn->mux->name, "H2")) {
 					/* For HTTP/2, destroy the conn_stream,
@@ -1542,27 +1478,9 @@ int stream_set_backend(struct stream *s, struct proxy *be)
 				s->flags |= SF_HTX;
 			}
 		}
-		else if (IS_HTX_STRM(s) && be->mode != PR_MODE_HTTP) {
-			/* If a TCP backend is assgiend to an HTX stream, return
-			 * an error. It may happens for a new stream on a
-			 * previously upgraded connnections. */
-			if (!(s->flags & SF_ERR_MASK))
-				s->flags |= SF_ERR_INTERNAL;
-			return 0;
-		}
-
-		/* If an LB algorithm needs to access some pre-parsed body contents,
-		 * we must not start to forward anything until the connection is
-		 * confirmed otherwise we'll lose the pointer to these data and
-		 * prevent the hash from being doable again after a redispatch.
-		 */
-		if (be->mode == PR_MODE_HTTP &&
-		    (be->lbprm.algo & (BE_LB_KIND | BE_LB_PARM)) == (BE_LB_KIND_HI | BE_LB_HASH_PRM))
-			s->txn->req.flags |= HTTP_MSGF_WAIT_CONN;
 
 		/* we may request to parse a request body */
-		if ((be->options & PR_O_WREQ_BODY) &&
-		    (s->txn->req.body_len || (s->txn->req.flags & HTTP_MSGF_TE_CHNK)))
+		if (be->options & PR_O_WREQ_BODY)
 			s->req.analysers |= AN_REQ_HTTP_BODY;
 	}
 
@@ -1618,7 +1536,7 @@ void proxy_capture_error(struct proxy *proxy, int is_back,
 	es->buf_len = buf_len;
 	es->ev_id   = ev_id;
 
-	len1 = b_size(buf) - b_peek_ofs(buf, buf_out);
+	len1 = b_size(buf) - buf_len;
 	if (len1 > buf_len)
 		len1 = buf_len;
 
@@ -1633,8 +1551,8 @@ void proxy_capture_error(struct proxy *proxy, int is_back,
 	es->when    = date; // user-visible date
 	es->srv     = objt_server(target);
 	es->oe      = other_end;
-	if (objt_conn(sess->origin))
-		es->src  = __objt_conn(sess->origin)->addr.from;
+	if (objt_conn(sess->origin) && conn_get_src(__objt_conn(sess->origin)))
+		es->src  = *__objt_conn(sess->origin)->src;
 	else
 		memset(&es->src, 0, sizeof(es->src));
 
@@ -1664,8 +1582,8 @@ void proxy_capture_error(struct proxy *proxy, int is_back,
 	} else {
 		es = HA_ATOMIC_XCHG(&proxy->invalid_req, es);
 	}
-	HA_SPIN_UNLOCK(PROXY_LOCK, &proxy->lock);
 	free(es);
+	HA_SPIN_UNLOCK(PROXY_LOCK, &proxy->lock);
 }
 
 /* Configure all proxies which lack a maxconn setting to use the global one by
@@ -1747,9 +1665,9 @@ void proxy_adjust_all_maxconn()
 static struct cfg_kw_list cfg_kws = {ILH, {
 	{ CFG_GLOBAL, "hard-stop-after", proxy_parse_hard_stop_after },
 	{ CFG_LISTEN, "timeout", proxy_parse_timeout },
-	{ CFG_LISTEN, "clitimeout", proxy_parse_timeout },
-	{ CFG_LISTEN, "contimeout", proxy_parse_timeout },
-	{ CFG_LISTEN, "srvtimeout", proxy_parse_timeout },
+	{ CFG_LISTEN, "clitimeout", proxy_parse_timeout }, /* This keyword actually fails to parse, this line remains for better error messages. */
+	{ CFG_LISTEN, "contimeout", proxy_parse_timeout }, /* This keyword actually fails to parse, this line remains for better error messages. */
+	{ CFG_LISTEN, "srvtimeout", proxy_parse_timeout }, /* This keyword actually fails to parse, this line remains for better error messages. */
 	{ CFG_LISTEN, "rate-limit", proxy_parse_rate_limit },
 	{ CFG_LISTEN, "max-keep-alive-queue", proxy_parse_max_ka_queue },
 	{ CFG_LISTEN, "declare", proxy_parse_declare },
@@ -1768,17 +1686,13 @@ struct proxy *cli_find_frontend(struct appctx *appctx, const char *arg)
 	struct proxy *px;
 
 	if (!*arg) {
-		appctx->ctx.cli.severity = LOG_ERR;
-		appctx->ctx.cli.msg = "A frontend name is expected.\n";
-		appctx->st0 = CLI_ST_PRINT;
+		cli_err(appctx, "A frontend name is expected.\n");
 		return NULL;
 	}
 
 	px = proxy_fe_by_name(arg);
 	if (!px) {
-		appctx->ctx.cli.severity = LOG_ERR;
-		appctx->ctx.cli.msg = "No such frontend.\n";
-		appctx->st0 = CLI_ST_PRINT;
+		cli_err(appctx, "No such frontend.\n");
 		return NULL;
 	}
 	return px;
@@ -1793,17 +1707,13 @@ struct proxy *cli_find_backend(struct appctx *appctx, const char *arg)
 	struct proxy *px;
 
 	if (!*arg) {
-		appctx->ctx.cli.severity = LOG_ERR;
-		appctx->ctx.cli.msg = "A backend name is expected.\n";
-		appctx->st0 = CLI_ST_PRINT;
+		cli_err(appctx, "A backend name is expected.\n");
 		return NULL;
 	}
 
 	px = proxy_be_by_name(arg);
 	if (!px) {
-		appctx->ctx.cli.severity = LOG_ERR;
-		appctx->ctx.cli.msg = "No such backend.\n";
-		appctx->st0 = CLI_ST_PRINT;
+		cli_err(appctx, "No such backend.\n");
 		return NULL;
 	}
 	return px;
@@ -1823,26 +1733,23 @@ static int cli_parse_show_servers(char **args, char *payload, struct appctx *app
 		/* read server state from local file */
 		px = proxy_be_by_name(args[3]);
 
-		if (!px) {
-			appctx->ctx.cli.severity = LOG_ERR;
-			appctx->ctx.cli.msg = "Can't find backend.\n";
-			appctx->st0 = CLI_ST_PRINT;
-			return 1;
-		}
+		if (!px)
+			return cli_err(appctx, "Can't find backend.\n");
+
 		appctx->ctx.cli.p0 = px;
 		appctx->ctx.cli.i0 = px->uuid;
 	}
 	return 0;
 }
 
-/* dumps server state information for all the servers found in backend cli.p0.
+/* dumps server state information into <buf> for all the servers found in backend cli.p0.
  * These information are all the parameters which may change during HAProxy runtime.
  * By default, we only export to the last known server state file format.
  * These information can be used at next startup to recover same level of server state.
  * It uses the proxy pointer from cli.p0, the proxy's id from cli.i0 and the server's
  * pointer from cli.p1.
  */
-static int dump_servers_state(struct stream_interface *si)
+static int dump_servers_state(struct stream_interface *si, struct buffer *buf)
 {
 	struct appctx *appctx = __objt_appctx(si->end);
 	struct proxy *px = appctx->ctx.cli.p0;
@@ -1884,7 +1791,7 @@ static int dump_servers_state(struct stream_interface *si)
 		if (srv->srvrq && srv->srvrq->name)
 			srvrecord = srv->srvrq->name;
 
-		chunk_printf(&trash,
+		chunk_appendf(buf,
 				"%d %s "
 				"%d %s %s "
 				"%d %d %d %d %ld "
@@ -1936,7 +1843,7 @@ static int cli_io_handler_servers_state(struct appctx *appctx)
 		curproxy = appctx->ctx.cli.p0;
 		/* servers are only in backends */
 		if (curproxy->cap & PR_CAP_BE) {
-			if (!dump_servers_state(si))
+			if (!dump_servers_state(si, &trash))
 				return 0;
 		}
 		/* only the selected proxy is dumped */
@@ -2071,20 +1978,12 @@ static int cli_parse_set_dyncookie_key_backend(char **args, char *payload, struc
 	if (!px)
 		return 1;
 
-	if (!*args[4]) {
-		appctx->ctx.cli.severity = LOG_ERR;
-		appctx->ctx.cli.msg = "String value expected.\n";
-		appctx->st0 = CLI_ST_PRINT;
-		return 1;
-	}
+	if (!*args[4])
+		return cli_err(appctx, "String value expected.\n");
 
 	newkey = strdup(args[4]);
-	if (!newkey) {
-		appctx->ctx.cli.severity = LOG_ERR;
-		appctx->ctx.cli.msg = "Failed to allocate memory.\n";
-		appctx->st0 = CLI_ST_PRINT;
-		return 1;
-	}
+	if (!newkey)
+		return cli_err(appctx, "Failed to allocate memory.\n");
 
 	/* Note: this lock is to make sure this doesn't change while another
 	 * thread is in srv_set_dyncookie().
@@ -2120,20 +2019,12 @@ static int cli_parse_set_maxconn_frontend(char **args, char *payload, struct app
 	if (!px)
 		return 1;
 
-	if (!*args[4]) {
-		appctx->ctx.cli.severity = LOG_ERR;
-		appctx->ctx.cli.msg = "Integer value expected.\n";
-		appctx->st0 = CLI_ST_PRINT;
-		return 1;
-	}
+	if (!*args[4])
+		return cli_err(appctx, "Integer value expected.\n");
 
 	v = atoi(args[4]);
-	if (v < 0) {
-		appctx->ctx.cli.severity = LOG_ERR;
-		appctx->ctx.cli.msg = "Value out of range.\n";
-		appctx->st0 = CLI_ST_PRINT;
-		return 1;
-	}
+	if (v < 0)
+		return cli_err(appctx, "Value out of range.\n");
 
 	/* OK, the value is fine, so we assign it to the proxy and to all of
 	 * its listeners. The blocked ones will be dequeued.
@@ -2146,7 +2037,7 @@ static int cli_parse_set_maxconn_frontend(char **args, char *payload, struct app
 			resume_listener(l);
 	}
 
-	if (px->maxconn > px->feconn && !LIST_ISEMPTY(&px->listener_queue))
+	if (px->maxconn > px->feconn && !MT_LIST_ISEMPTY(&px->listener_queue))
 		dequeue_all_listeners(&px->listener_queue);
 
 	HA_SPIN_UNLOCK(PROXY_LOCK, &px->lock);
@@ -2169,12 +2060,8 @@ static int cli_parse_shutdown_frontend(char **args, char *payload, struct appctx
 	if (!px)
 		return 1;
 
-	if (px->state == PR_STSTOPPED) {
-		appctx->ctx.cli.severity = LOG_NOTICE;
-		appctx->ctx.cli.msg = "Frontend was already shut down.\n";
-		appctx->st0 = CLI_ST_PRINT;
-		return 1;
-	}
+	if (px->state == PR_STSTOPPED)
+		return cli_msg(appctx, LOG_NOTICE, "Frontend was already shut down.\n");
 
 	ha_warning("Proxy %s stopped (FE: %lld conns, BE: %lld conns).\n",
 		   px->id, px->fe_counters.cum_conn, px->be_counters.cum_conn);
@@ -2201,29 +2088,19 @@ static int cli_parse_disable_frontend(char **args, char *payload, struct appctx
 	if (!px)
 		return 1;
 
-	if (px->state == PR_STSTOPPED) {
-		appctx->ctx.cli.severity = LOG_NOTICE;
-		appctx->ctx.cli.msg = "Frontend was previously shut down, cannot disable.\n";
-		appctx->st0 = CLI_ST_PRINT;
-		return 1;
-	}
+	if (px->state == PR_STSTOPPED)
+		return cli_msg(appctx, LOG_NOTICE, "Frontend was previously shut down, cannot disable.\n");
 
-	if (px->state == PR_STPAUSED) {
-		appctx->ctx.cli.severity = LOG_NOTICE;
-		appctx->ctx.cli.msg = "Frontend is already disabled.\n";
-		appctx->st0 = CLI_ST_PRINT;
-		return 1;
-	}
+	if (px->state == PR_STPAUSED)
+		return cli_msg(appctx, LOG_NOTICE, "Frontend is already disabled.\n");
 
-	/* pause_proxy will take PROXY_LOCK */
+	HA_SPIN_LOCK(PROXY_LOCK, &px->lock);
 	ret = pause_proxy(px);
+	HA_SPIN_UNLOCK(PROXY_LOCK, &px->lock);
+
+	if (!ret)
+		return cli_err(appctx, "Failed to pause frontend, check logs for precise cause.\n");
 
-	if (!ret) {
-		appctx->ctx.cli.severity = LOG_ERR;
-		appctx->ctx.cli.msg = "Failed to pause frontend, check logs for precise cause.\n";
-		appctx->st0 = CLI_ST_PRINT;
-		return 1;
-	}
 	return 1;
 }
 
@@ -2243,29 +2120,18 @@ static int cli_parse_enable_frontend(char **args, char *payload, struct appctx *
 	if (!px)
 		return 1;
 
-	if (px->state == PR_STSTOPPED) {
-		appctx->ctx.cli.severity = LOG_ERR;
-		appctx->ctx.cli.msg = "Frontend was previously shut down, cannot enable.\n";
-		appctx->st0 = CLI_ST_PRINT;
-		return 1;
-	}
+	if (px->state == PR_STSTOPPED)
+		return cli_err(appctx, "Frontend was previously shut down, cannot enable.\n");
 
-	if (px->state != PR_STPAUSED) {
-		appctx->ctx.cli.severity = LOG_NOTICE;
-		appctx->ctx.cli.msg = "Frontend is already enabled.\n";
-		appctx->st0 = CLI_ST_PRINT;
-		return 1;
-	}
+	if (px->state != PR_STPAUSED)
+		return cli_msg(appctx, LOG_NOTICE, "Frontend is already enabled.\n");
 
-	/* resume_proxy will take PROXY_LOCK */
+	HA_SPIN_LOCK(PROXY_LOCK, &px->lock);
 	ret = resume_proxy(px);
+	HA_SPIN_UNLOCK(PROXY_LOCK, &px->lock);
 
-	if (!ret) {
-		appctx->ctx.cli.severity = LOG_ERR;
-		appctx->ctx.cli.msg = "Failed to resume frontend, check logs for precise cause (port conflict?).\n";
-		appctx->st0 = CLI_ST_PRINT;
-		return 1;
-	}
+	if (!ret)
+		return cli_err(appctx, "Failed to resume frontend, check logs for precise cause (port conflict?).\n");
 	return 1;
 }
 
@@ -2286,12 +2152,8 @@ static int cli_parse_show_errors(char **args, char *payload, struct appctx *appc
 		else
 			appctx->ctx.errors.iid = atoi(args[2]);
 
-		if (!appctx->ctx.errors.iid) {
-			appctx->ctx.cli.severity = LOG_ERR;
-			appctx->ctx.cli.msg = "No such proxy.\n";
-			appctx->st0 = CLI_ST_PRINT;
-			return 1;
-		}
+		if (!appctx->ctx.errors.iid)
+			return cli_err(appctx, "No such proxy.\n");
 	}
 	else
 		appctx->ctx.errors.iid	= -1; // dump all proxies
