@@ -77,7 +77,7 @@
 #include <eb32tree.h>
 
 #include <proto/http_rules.h>
-#include <proto/proto_http.h>
+#include <proto/http_ana.h>
 #include <proto/queue.h>
 #include <proto/sample.h>
 #include <proto/server.h>
@@ -306,16 +306,14 @@ static int pendconn_process_next_strm(struct server *srv, struct proxy *px)
 }
 
 /* Manages a server's connection queue. This function will try to dequeue as
- * many pending streams as possible, and wake them up. <server_locked> must
- * only be set if the caller already hold the server lock.
+ * many pending streams as possible, and wake them up.
  */
-void process_srv_queue(struct server *s, int server_locked)
+void process_srv_queue(struct server *s)
 {
 	struct proxy  *p = s->proxy;
 	int maxconn;
 
-	if (!server_locked)
-		HA_SPIN_LOCK(SERVER_LOCK, &s->lock);
+	HA_SPIN_LOCK(SERVER_LOCK, &s->lock);
 	HA_SPIN_LOCK(PROXY_LOCK,  &p->lock);
 	maxconn = srv_dynamic_maxconn(s);
 	while (s->served < maxconn) {
@@ -324,8 +322,7 @@ void process_srv_queue(struct server *s, int server_locked)
 			break;
 	}
 	HA_SPIN_UNLOCK(PROXY_LOCK,  &p->lock);
-	if (!server_locked)
-		HA_SPIN_UNLOCK(SERVER_LOCK, &s->lock);
+	HA_SPIN_UNLOCK(SERVER_LOCK, &s->lock);
 }
 
 /* Adds the stream <strm> to the pending connection queue of server <strm>->srv
@@ -421,7 +418,6 @@ int pendconn_redistribute(struct server *s)
 		p->strm_flags &= ~(SF_DIRECT | SF_ASSIGNED | SF_ADDR_SET);
 
 		task_wakeup(p->strm->task, TASK_WOKEN_RES);
-		xferred++;
 	}
 	return xferred;
 }
@@ -505,15 +501,11 @@ int pendconn_dequeue(struct stream *strm)
 	/* the pendconn is not queued anymore and will not be so we're safe
 	 * to proceed.
 	 */
+	if (p->target)
+		strm->target = &p->target->obj_type;
+
 	strm->flags &= ~(SF_DIRECT | SF_ASSIGNED | SF_ADDR_SET);
 	strm->flags |= p->strm_flags & (SF_DIRECT | SF_ASSIGNED | SF_ADDR_SET);
-
-	if (p->target) {
-		/* a server picked this pendconn, it must skip LB */
-		strm->target = &p->target->obj_type;
-		strm->flags |= SF_ASSIGNED;
-	}
-
 	strm->pend_pos = NULL;
 	pool_free(pool_head_pendconn, p);
 	return 0;
