@@ -381,12 +381,7 @@ int conn_si_send_proxy(struct connection *conn, unsigned int flag)
 			/* The target server expects a LOCAL line to be sent first. Retrieving
 			 * local or remote addresses may fail until the connection is established.
 			 */
-			conn_get_from_addr(conn);
-			if (!(conn->flags & CO_FL_ADDR_FROM_SET))
-				goto out_wait;
-
-			conn_get_to_addr(conn);
-			if (!(conn->flags & CO_FL_ADDR_TO_SET))
+			if (!conn_get_src(conn) || !conn_get_dst(conn))
 				goto out_wait;
 
 			ret = make_proxy_line(trash.area, trash.size,
@@ -574,23 +569,6 @@ static void stream_int_notify(struct stream_interface *si)
 		ic->flags &= ~CF_READ_DONTWAIT;
 }
 
-/* The stream interface is only responsible for the connection during the early
- * states, before plugging a mux. Thus it should only care about CO_FL_ERROR
- * before SI_ST_EST, and after that it must absolutely ignore it since the mux
- * may hold pending data. This function returns true if such an error was
- * reported. Both the CS and the CONN must be valid.
- */
-static inline int si_is_conn_error(const struct stream_interface *si)
-{
-	struct connection *conn;
-
-	if (si->state >= SI_ST_EST)
-		return 0;
-
-	conn = __objt_cs(si->end)->conn;
-	BUG_ON(!conn);
-	return !!(conn->flags & CO_FL_ERROR);
-}
 
 /* Called by I/O handlers after completion.. It propagates
  * connection flags to the stream interface, updates the stream (which may or
@@ -621,11 +599,9 @@ static int si_cs_process(struct conn_stream *cs)
 	 *       wake callback. Otherwise si_cs_recv()/si_cs_send() already take
 	 *       care of it.
 	 */
-
-	if (si->state >= SI_ST_CON) {
-		if ((cs->flags & CS_FL_ERROR) || si_is_conn_error(si))
-			si->flags |= SI_FL_ERR;
-	}
+	if (si->state >= SI_ST_CON &&
+	    (conn->flags & CO_FL_ERROR || cs->flags & CS_FL_ERROR))
+		si->flags |= SI_FL_ERR;
 
 	/* If we had early data, and the handshake ended, then
 	 * we can remove the flag, and attempt to wake the task up,
@@ -661,8 +637,7 @@ static int si_cs_process(struct conn_stream *cs)
 	 * stream-int status.
 	 */
 	stream_int_notify(si);
-	channel_release_buffer(ic, &(si_strm(si)->buffer_wait));
-
+	stream_release_buffers(si_strm(si));
 	return 0;
 }
 
@@ -680,7 +655,7 @@ int si_cs_send(struct conn_stream *cs)
 	int ret;
 	int did_send = 0;
 
-	if (cs->flags & (CS_FL_ERROR|CS_FL_ERR_PENDING) || si_is_conn_error(si)) {
+	if (conn->flags & CO_FL_ERROR || cs->flags & (CS_FL_ERROR|CS_FL_ERR_PENDING)) {
 		/* We're probably there because the tasklet was woken up,
 		 * but process_stream() ran before, detected there were an
 		 * error and put the si back to SI_ST_TAR. There's still
@@ -701,10 +676,6 @@ int si_cs_send(struct conn_stream *cs)
 	if (conn->flags & CO_FL_SOCK_WR_SH || oc->flags & CF_SHUTW)
 		return 1;
 
-	/* we must wait because the mux is not installed yet */
-	if (!conn->mux)
-		return 0;
-
 	if (oc->pipe && conn->xprt->snd_pipe && conn->mux->snd_pipe) {
 		ret = conn->mux->snd_pipe(cs, oc->pipe);
 		if (ret > 0)
@@ -803,7 +774,7 @@ int si_cs_send(struct conn_stream *cs)
 		si_rx_room_rdy(si_opposite(si));
 	}
 
-	if (cs->flags & (CS_FL_ERROR|CS_FL_ERR_PENDING)) {
+	if (conn->flags & CO_FL_ERROR || cs->flags & (CS_FL_ERROR|CS_FL_ERR_PENDING)) {
 		si->flags |= SI_FL_ERR;
 		return 1;
 	}
@@ -835,6 +806,7 @@ struct task *si_cs_io_cb(struct task *t, void *ctx, unsigned short state)
 	if (ret != 0)
 		si_cs_process(cs);
 
+	stream_release_buffers(si_strm(si));
 	return (NULL);
 }
 
@@ -955,22 +927,6 @@ void si_sync_send(struct stream_interface *si)
 		return;
 
 	si_cs_send(cs);
-
-	if (likely(oc->flags & CF_WRITE_ACTIVITY)) {
-		struct channel *ic = si_ic(si);
-
-		if (tick_isset(ic->rex) && !(si->flags & SI_FL_INDEP_STR)) {
-			/* Note: to prevent the client from expiring read timeouts
-			 * during writes, we refresh it. We only do this if the
-			 * interface is not configured for "independent streams",
-			 * because for some applications it's better not to do this,
-			 * for instance when continuously exchanging small amounts
-			 * of data which can full the socket buffers long before a
-			 * write timeout is detected.
-			 */
-			ic->rex = tick_add_ifset(now_ms, ic->rto);
-		}
-	}
 }
 
 /* Updates at once the channel flags, and timers of both stream interfaces of a
@@ -1165,7 +1121,7 @@ static void stream_int_chk_snd_conn(struct stream_interface *si)
 	struct channel *oc = si_oc(si);
 	struct conn_stream *cs = __objt_cs(si->end);
 
-	if (unlikely(!si_state_in(si->state, SI_SB_RDY|SI_SB_EST) ||
+	if (unlikely(!si_state_in(si->state, SI_SB_CON|SI_SB_RDY|SI_SB_EST) ||
 	    (oc->flags & CF_SHUTW)))
 		return;
 
@@ -1179,7 +1135,7 @@ static void stream_int_chk_snd_conn(struct stream_interface *si)
 	if (!(si->wait_event.events & SUB_RETRY_SEND) && !channel_is_empty(si_oc(si)))
 		si_cs_send(cs);
 
-	if (cs->flags & (CS_FL_ERROR|CS_FL_ERR_PENDING) || si_is_conn_error(si)) {
+	if (cs->flags & (CS_FL_ERROR|CS_FL_ERR_PENDING) || cs->conn->flags & CO_FL_ERROR) {
 		/* Write error on the file descriptor */
 		if (si->state >= SI_ST_CON)
 			si->flags |= SI_FL_ERR;
@@ -1277,10 +1233,6 @@ int si_cs_recv(struct conn_stream *cs)
 	if (ic->flags & CF_SHUTR)
 		return 1;
 
-	/* we must wait because the mux is not installed yet */
-	if (!conn->mux)
-		return 0;
-
 	/* stop here if we reached the end of data */
 	if (cs->flags & CS_FL_EOS)
 		goto end_recv;
@@ -1294,7 +1246,7 @@ int si_cs_recv(struct conn_stream *cs)
 	if (!(cs->flags & CS_FL_RCV_MORE)) {
 		if (!conn_xprt_ready(conn))
 			return 0;
-		if (cs->flags & CS_FL_ERROR)
+		if (conn->flags & CO_FL_ERROR || cs->flags & CS_FL_ERROR)
 			goto end_recv;
 	}
 
@@ -1316,7 +1268,7 @@ int si_cs_recv(struct conn_stream *cs)
 	/* First, let's see if we may splice data across the channel without
 	 * using a buffer.
 	 */
-	if (cs->flags & CS_FL_MAY_SPLICE &&
+	if (conn->xprt->rcv_pipe && conn->mux->rcv_pipe &&
 	    (ic->pipe || ic->to_forward >= MIN_SPLICE_FORWARD) &&
 	    ic->flags & CF_KERN_SPLICING) {
 		if (c_data(ic)) {
@@ -1351,7 +1303,7 @@ int si_cs_recv(struct conn_stream *cs)
 			ic->flags |= CF_READ_PARTIAL;
 		}
 
-		if (cs->flags & (CS_FL_EOS|CS_FL_ERROR))
+		if (conn->flags & CO_FL_ERROR || cs->flags & (CS_FL_EOS|CS_FL_ERROR))
 			goto end_recv;
 
 		if (conn->flags & CO_FL_WAIT_ROOM) {
@@ -1375,40 +1327,17 @@ int si_cs_recv(struct conn_stream *cs)
 		ic->pipe = NULL;
 	}
 
-	if (ic->pipe && ic->to_forward && !(flags & CO_RFL_BUF_FLUSH) && cs->flags & CS_FL_MAY_SPLICE) {
-		/* don't break splicing by reading, but still call rcv_buf()
-		 * to pass the flag.
-		 */
-		goto done_recv;
-	}
-
 	/* now we'll need a input buffer for the stream */
 	if (!si_alloc_ibuf(si, &(si_strm(si)->buffer_wait)))
 		goto end_recv;
 
-	/* For an HTX stream, if the buffer is stuck (no output data with some
-	 * input data) and if the HTX message is fragmented or if its free space
-	 * wraps, we force an HTX deframentation. It is a way to have a
-	 * contiguous free space nad to let the mux to copy as much data as
-	 * possible.
-	 *
-	 * NOTE: A possible optim may be to let the mux decides if defrag is
-	 *       required or not, depending on amount of data to be xferred.
-	 */
-	if (IS_HTX_STRM(si_strm(si)) && !co_data(ic)) {
-		struct htx *htx = htxbuf(&ic->buf);
-
-		if (htx_is_not_empty(htx) && ((htx->flags & HTX_FL_FRAGMENTED) || htx_space_wraps(htx)))
-			htx_defrag(htxbuf(&ic->buf), NULL, 0);
-	}
-
 	/* Important note : if we're called with POLL_IN|POLL_HUP, it means the read polling
 	 * was enabled, which implies that the recv buffer was not full. So we have a guarantee
 	 * that if such an event is not handled above in splice, it will be handled here by
 	 * recv().
 	 */
 	while ((cs->flags & CS_FL_RCV_MORE) ||
-	       (!(conn->flags & (CO_FL_WAIT_ROOM | CO_FL_HANDSHAKE)) &&
+	    (!(conn->flags & (CO_FL_ERROR | CO_FL_WAIT_ROOM | CO_FL_HANDSHAKE)) &&
 	       (!(cs->flags & (CS_FL_ERROR|CS_FL_EOS))) && !(ic->flags & CF_SHUTR))) {
 		/* <max> may be null. This is the mux responsibility to set
 		 * CS_FL_RCV_MORE on the CS if more space is needed.
@@ -1416,13 +1345,8 @@ int si_cs_recv(struct conn_stream *cs)
 		max = channel_recv_max(ic);
 		ret = cs->conn->mux->rcv_buf(cs, &ic->buf, max, flags | (co_data(ic) ? CO_RFL_BUF_WET : 0));
 
-		if (cs->flags & CS_FL_WANT_ROOM) {
+		if (cs->flags & CS_FL_WANT_ROOM)
 			si_rx_room_blk(si);
-			/* Add READ_PARTIAL because some data are pending but
-			 * cannot be xferred to the channel
-			 */
-			ic->flags |= CF_READ_PARTIAL;
-		}
 
 		if (cs->flags & CS_FL_READ_PARTIAL) {
 			if (tick_isset(ic->rex))
@@ -1431,18 +1355,10 @@ int si_cs_recv(struct conn_stream *cs)
 		}
 
 		if (ret <= 0) {
-			/* if we refrained from reading because we asked for a
-			 * flush to satisfy rcv_pipe(), we must not subscribe
-			 * and instead report that there's not enough room
-			 * here to proceed.
-			 */
-			if (flags & CO_RFL_BUF_FLUSH)
-				si_rx_room_blk(si);
 			break;
 		}
 
-		/* L7 retries enabled and maximum connection retries not reached */
-		if ((si->flags & SI_FL_L7_RETRY) && si->conn_retries) {
+		if (si->flags & SI_FL_L7_RETRY) {
 			struct htx *htx;
 			struct htx_sl *sl;
 
@@ -1479,14 +1395,6 @@ int si_cs_recv(struct conn_stream *cs)
 		ic->flags |= CF_READ_PARTIAL;
 		ic->total += ret;
 
-		/* End-of-input reached, we can leave. In this case, it is
-		 * important to break the loop to not block the SI because of
-		 * the channel's policies.This way, we are still able to receive
-		 * shutdowns.
-		 */
-		if (cs->flags & CS_FL_EOI)
-			break;
-
 		if ((ic->flags & CF_READ_DONTWAIT) || --read_poll <= 0) {
 			/* we're stopped by the channel's policy */
 			si_rx_chan_blk(si);
@@ -1575,7 +1483,8 @@ int si_cs_recv(struct conn_stream *cs)
 		ret = 1;
 	}
 
-	if (cs->flags & CS_FL_ERROR) {
+	if (conn->flags & CO_FL_ERROR || cs->flags & CS_FL_ERROR) {
+		cs->flags |= CS_FL_ERROR;
 		si->flags |= SI_FL_ERR;
 		ret = 1;
 	}
@@ -1672,6 +1581,7 @@ void si_applet_wake_cb(struct stream_interface *si)
 
 	/* update the stream-int, channels, and possibly wake the stream up */
 	stream_int_notify(si);
+	stream_release_buffers(si_strm(si));
 
 	/* stream_int_notify may have passed through chk_snd and released some
 	 * RXBLK flags. Process_stream will consider those flags to wake up the
