@@ -16,7 +16,6 @@
 
 #include <common/config.h>
 #include <common/standard.h>
-#include <common/ticks.h>
 #include <common/time.h>
 #include <common/hathreads.h>
 
@@ -26,14 +25,15 @@ THREAD_LOCAL unsigned int   samp_time;       /* total elapsed time over current
 THREAD_LOCAL unsigned int   idle_time;       /* total idle time over current sample */
 THREAD_LOCAL struct timeval now;             /* internal date is a monotonic function of real clock */
 THREAD_LOCAL struct timeval date;            /* the real current date */
-struct timeval start_date;                   /* the process's start date */
-struct timeval ready_date;                   /* date when the process was considered ready */
+struct timeval start_date;      /* the process's start date */
 THREAD_LOCAL struct timeval before_poll;     /* system date before calling poll() */
 THREAD_LOCAL struct timeval after_poll;      /* system date after leaving poll() */
 
 static THREAD_LOCAL struct timeval tv_offset;  /* per-thread time ofsset relative to global time */
-volatile unsigned long long global_now;      /* common date between all threads (32:32) */
-volatile unsigned int global_now_ms;         /* common date in milliseconds (may wrap) */
+static volatile unsigned long long global_now; /* common date between all threads (32:32) */
+
+static THREAD_LOCAL unsigned int iso_time_sec;     /* last iso time value for this thread */
+static THREAD_LOCAL char         iso_time_str[28]; /* ISO time representation of gettimeofday() */
 
 /*
  * adds <ms> ms to <from>, set the result to <tv> and returns a pointer <tv>
@@ -179,7 +179,6 @@ REGPRM2 void tv_update_date(int max_wait, int interrupted)
 {
 	struct timeval adjusted, deadline, tmp_now, tmp_adj;
 	unsigned int   curr_sec_ms;     /* millisecond of current second (0..999) */
-	unsigned int old_now_ms, new_now_ms;
 	unsigned long long old_now;
 	unsigned long long new_now;
 
@@ -263,18 +262,36 @@ REGPRM2 void tv_update_date(int max_wait, int interrupted)
 	 */
 	ms_left_scaled = (999U - curr_sec_ms) * 4294967U;
 	now_ms = now.tv_sec * 1000 + curr_sec_ms;
-
-	/* update the global current millisecond */
-	old_now_ms = global_now_ms;
-	do {
-		new_now_ms = old_now_ms;
-		if (tick_is_lt(new_now_ms, now_ms) || !new_now_ms)
-			new_now_ms = now_ms;
-	}  while (!_HA_ATOMIC_CAS(&global_now_ms, &old_now_ms, new_now_ms));
-
 	return;
 }
 
+/* returns the current date as returned by gettimeofday() in ISO+microsecond
+ * format. It uses a thread-local static variable that the reader can consume
+ * for as long as it wants until next call. Thus, do not call it from a signal
+ * handler. If <pad> is non-0, a trailing space will be added. It will always
+ * return exactly 26 or 27 characters (depending on padding) and will always be
+ * zero-terminated, thus it will always fit into a 28 bytes buffer.
+ */
+char *timeofday_as_iso_us(int pad)
+{
+	struct timeval new_date;
+	struct tm tm;
+
+	gettimeofday(&new_date, NULL);
+	if (new_date.tv_sec != iso_time_sec || !new_date.tv_sec) {
+		get_localtime(new_date.tv_sec, &tm);
+		if (unlikely(strftime(iso_time_str, sizeof(iso_time_str), "%Y-%m-%dT%H:%M:%S.000000", &tm) != 26))
+			strcpy(iso_time_str, "YYYY-mm-ddTHH:MM:SS.000000"); // make the failure visible but respect format.
+		iso_time_sec = new_date.tv_sec;
+	}
+	utoa_pad(new_date.tv_usec, iso_time_str + 20, 7);
+	if (pad) {
+		iso_time_str[26] = ' ';
+		iso_time_str[27] = 0;
+	}
+	return iso_time_str;
+}
+
 /*
  * Local variables:
  *  c-indent-level: 8
